deferred class  AGUI_BASE_CLIENT_CONTROL


inherit
   AGUI_CONTROL
      end
   AGUI_COMPOSITE
      end   
      
   
feature {ANY}  -- subchild management


   position (child: AGUI_WINDOW; x,y,w,h: INTEGER) is
	 -- change size and position of a child window
      require
	 not is_zombie
	 child.parent_window = Current
      do
	 efox_base_client_control__position(child.handle,x,y,w,h)
      end
   

feature {ANY}   -- display contents


   invalidate (x,y,w,h: INTEGER) is
	 -- Marks the given area as dirty.
      require
	 not is_zombie
      deferred
      end
   
   
   invalidate_all is
	 -- Marks the whole window as dirty.
      require
	 not is_zombie
      do
	 efox_base_client_control__invalidate_all(handle)
      end
   
   
   paint_context : AGUI_PAINT_CONTEXT is
	 -- Assigned paint context for drawing.
	 -- This is only useable during a 'begin_paint' / 'end_paint'
	 -- operation.
      require
	 not is_zombie
	 painting_state /= 0
      do
	 result := paint_context_private
      end
   
   
   begin_paint is
	 -- Start a "painting" operation. You must always call
	 -- 'end_paint' when finished and you are not allowed to
	 -- start another painting operation in the meantime.
      require
	 not is_zombie
	 painting_state = 0
      do
	 painting_state := 1
	 paint_context.efox_begin_paint(handle)
      ensure
	 painting_state = 1
      end
   
   
   end_paint is
	 -- Ends a "painting" operation.
      require
	 not is_zombie
	 painting_state = 1
      do
	 paint_context.efox_end_paint(handle)
	 painting_state := 0
      ensure
	 painting_state = 0
      end
   
   
   begin_buffered_paint is
	 -- Start a buffered "painting" operation. You must always call
	 -- 'end_buffered_paint' when finished. The painting is
	 -- done in a pixmap.
      require
	 not is_zombie
	 painting_state = 0
      do
	 painting_state := 2
	 paint_context.efox_begin_buffered_paint(handle)
      ensure
	 painting_state = 2
      end
   
   
   end_buffered_paint is
	 -- Ends a "painting" operation. It blits the pixmap to the
	 -- window.
      require
	 not is_zombie
	 painting_state = 2
      do
	 paint_context.efox_end_buffered_paint(handle)
	 painting_state := 0
      ensure
	 painting_state = 0
      end
   
   
   painting_state : INTEGER
	 -- State of the "painting" operation. 
	 --  0 => no paint operation
	 --  1 => simple paint operation
	 --  2 => bufferered paint operation
   
   
feature {ANY} -- event handling
   
   
   on_paint : PROCEDURE[ANY,TUPLE[INTEGER,INTEGER,INTEGER,INTEGER]] is
      do
	 result ?= find_callback_agent(On_paint_event)
      end
   
   set_on_paint (action: PROCEDURE[ANY,TUPLE[INTEGER,INTEGER,INTEGER,INTEGER]]) is
      do
	 set_callback_agent(On_paint_event, action)
      end
   
   
feature {NONE}
    
   
   paint_context_private : AGUI_PAINT_CONTEXT
   
   efox_base_client_control__invalidate_all(hnd: POINTER) is
      external "C"
      end
      
   efox_base_client_control__position(hnd: POINTER; x,y,w,h: INTEGER) is
      external "C"
      end
   
   
invariant
   
   painting_state.in_range(0,2)
   
end   
