class AGUI_COLOR_WELL
   
   
--  A color well is a widget which controls color settings.
--  Colors may be dragged and dropped from one color well to another.
--  A double-click inside a color well will bring up the standard
--  color dialog panel to edit the color well's color.
--  Colors may be also pasted by name using middle-mouse click into/out of
--  color wells from/to other selection-capable applications; for example,
--  you can highlight the word `red' and paste it into a color well.
--  While the color value is being changed, the color well invokes the 'on_changed'
--  agent at the end of the change, it invokes the 'on_command' agent.
--  The argument to the agent is the current color value.
   
   
inherit
   AGUI_CONTROL
      redefine 
	 efox__callback_command,
	 efox__callback_changed
      end
   AGUI_FRAME_MIXIN
      redefine mixin_window
      end
   AGUI_PACKER_MIXIN
      redefine mixin_window
      end
   
   
creation
   make
   
   
feature {ANY} -- properties
   
   
   set_color (col: AGUI_COLOR) is
	 -- Set Color.
      do
	 efox_color_well__set_color(handle,col.to_integer)
      end
   
   
   color : AGUI_COLOR is
	 -- Get Color.
      do
	 result.from_integer(efox_color_well__color(handle))
      end
   
   
   opaque_colors_only : BOOLEAN is
	 -- Return true if only opaque colors allowed.
      do
	 efox_color_well__opaque_colors_only(handle)
      end
   
   
   set_opaque_colors_only (flag: BOOLEAN) is
	 -- Change opaque only mode
      do
	 efox_color_well__set_opaque_colors_only(handle, flag)
      end
   
   

feature {ANY} -- event handler
   
   
   on_changed: PROCEDURE[ANY,TUPLE[AGUI_COLOR]] is
	 -- Action that is invoked when the user changes the color.   
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_change_event)
      end
   
   set_on_changed (action: PROCEDURE[ANY,TUPLE[AGUI_COLOR]]) is
	 -- Sets a new handler for the 'on_changed' event.
      require
	 not is_zombie
      do
	 set_callback_agent(On_change_event, action)
      end
   

   on_command: PROCEDURE[ANY,TUPLE[AGUI_COLOR]] is
	 -- Action that is invoked when the user has selected a color.   
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_command_event)
      end
   
   set_on_command (action: PROCEDURE[ANY,TUPLE[AGUI_COLOR]]) is
	 -- Sets a new handler for the 'on_command' event.
      require
	 not is_zombie
      do
	 set_callback_agent(On_command_event, action)
      end
   
   
feature {ANY}  -- mixin configure
   
   
   mixin_window : AGUI_WINDOW is
	 -- redefine the mixin window references to this window
      do
	 result := Current
      end
   
  
feature {NONE}  -- constructor
   
   
   make (p: AGUI_COMPOSITE;  n: STRING) is
	 -- Create a color well control widget under parent 'p' and give 
	 -- it the name 'n'.
      require
	 p /= Void and n /= Void
      do
	 handle := efox_color_well__make(p.handle, n.to_external, $temp_px)
	 efox__window_hash_table.put(Current, temp_px)
      end
   

feature {NONE} -- private C binding
   
   
   efox__callback_command(idx: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 col: AGUI_COLOR
      do
	 if on_command /= Void then
	    col.from_integer(idx)
	    on_command.call([col])
	 end
      end
   
   efox__callback_changed(idx: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 col: AGUI_COLOR
      do
	 if on_changed /= Void then
	    col.from_integer(idx)
	    on_changed.call([col])
	 end
      end
   
   efox_color_well__make(this:like Current; hnd,n,keyptr: POINTER) : POINTER is
      external "C"
      end   
   
   efox_color_well__opaque_colors_only(h: POINTER) is
      external "C"
      end
   
   efox_color_well__set_opaque_colors_only(h: POINTER; f: BOOLEAN) is
      external "C"
      end
   
   efox_color_well__color(h: POINTER) : INTEGER is
      external "C"
      end
   
   efox_color_well__set_color(h: POINTER; n: INTEGER) is
      external "C"
      end
   
   
end
