class  AGUI_CURSOR_MANAGER
   
-- This Class manages all defined icons in the system. There are two kinds
-- of icons: Stock icons and loaded icons. The first are always available
-- and the second are only available when loaded from a 'GIF"
-- file via the register feature.
   
   
creation
   {AGUI_APPLICATION} make
   
   
feature {ANY}  -- Default cursors
   
   
   Cursor_arrow : STRING is "Arrow"
   Cursor_right_arrow : STRING is "RightArrow"
   Cursor_text : STRING is "Text"
   Cursor_hsplit : STRING is "HSplit"
   Cursor_vsplit : STRING is "VSplit"
   Cursor_xsplit : STRING is "XSplit"
   Cursor_swatch : STRING is "Swatch"
   Cursor_move : STRING is "Move"
   Cursor_drag_horizontal : STRING is "DragHorizontal"
   Cursor_drag_vertical : STRING is "DragVertical"
   Cursor_drag_upper_left : STRING is "DragUpperLeft"
   Cursor_drag_upper_right : STRING is "DragUpperRight"
   Cursor_drag_drop_stop : STRING is "DndStop"
   Cursor_drag_drop_copy : STRING is "DndCopy"
   Cursor_drag_drop_move : STRING is "DndMove"
   Cursor_drag_drop_link : STRING is "DndLink"
   Cursor_crosshair : STRING is "CrossHair"
   Cursor_corner_north_east : STRING is "CornerNE"
   Cursor_corner_north_west : STRING is "CornerNW"
   Cursor_corner_south_east : STRING is "CornerSE"
   Cursor_corner_south_west : STRING is "CornerSW"
   Cursor_rotate : STRING is "Rotate"
      
   
feature {ANY}  -- Operations
   
   
   contains (name: STRING) : BOOLEAN is
	 -- check if the curson with the given 'name' exists
      require
	 name /= Void
	 name.count > 0
      do
	 result := efox_cursor_manager__contains(name.to_external)
      end
   
   
   find (name: STRING) : AGUI_CURSOR is
	 -- looks for an cursor with the given 'name'
      require
	 name /= Void
	 name.count > 0
      do
	 result := efox_cursor_manager__find(name.to_external)
      end
   
   
   list_all_cursors : ARRAY[AGUI_FONT] is
	 -- Returns an array with all registered cursors.
	 -- The array is newly created on every call.
      local
	 s : ARRAY[STRING]
	 i,n: INTEGER
      do
	 s := list_all_names
	 create result.with_capacity(s.count,1)
	 from
	    i := s.lower
	    n := s.upper
	 until
	    i > n
	 loop
	    result.add_last(find(s.item(i)))
	    i := i + 1
	 end
      ensure
	 result /= Void
      end
   
   
   list_all_names : ARRAY[STRING] is
	 -- Returns an array with the name of all registered cursors.
	 -- The array and all strings are newly created on every call.
      local
	 s : STRING
      do
	 create s.from_external_copy(efox_cursor_manager__list_all_names)
	 result := s.split
	 if result = Void then
	    create result.with_capacity(0,1)
	 end
      ensure
	 result /= Void
      end
   
   
   register (name: STRING; file: STRING) is
	 -- Register a new icon from an image in a file. The file
	 -- must be in GIF format.
      obsolete "Not yet implemented"
      require
	 name /= Void and then name.count > 0
	 file /= Void
	 not contains(name)
      do
	 --TODO
      end
   
   
   last_register_status : BOOLEAN
	 -- Success status of last register operation.
   
   
feature {ANY} -- operations
   
   begin_wait_cursor is
      do
	 efox_cursor_manager__begin_wait_cursor
      end
   
   end_wait_cursor is
      do
	 efox_cursor_manager__end_wait_cursor
      end
   
   set_wait_cursor (cur: AGUI_CURSOR) is
      do
	 efox_cursor_manager__set_wait_cursor(cur.handle)
      end
   
   wait_cursor : AGUI_CURSOR is
      do
	 result := efox_cursor_manager__wait_cursor
      end
   
   set_default_cursor (cur : AGUI_CURSOR; n: INTEGER) is
      do
	 efox_cursor_manager__set_default_cursor(cur.handle,n)
      end
   
   default_cursor (n: INTEGER) : AGUI_CURSOR is
      do
	 result := efox_cursor_manager__default_cursor(n)
      end
       
   
feature {NONE} -- private implementation
   
   make is
      do
	 create gc_protection_table.with_capacity(20)
	 efox_cursor_manager__make(Current)
      end
   
   register_callback (hnd: POINTER; w,h,hx,hy: INTEGER) : AGUI_CURSOR is
      do
	 create result.make(hnd,w,h,hx,hy)
	 gc_protection_table.add(result,hnd)
      end   
   
   destroy_callback (ic: POINTER) is
      require
	 gc_protection_table.has(ic)
      do
	 gc_protection_table.remove(ic)
      end
   
   gc_protection_table : DICTIONARY[AGUI_CURSOR,POINTER]
   
   
feature {NONE}  -- private c implementation
   
   
   efox_cursor_manager__make(this:like Current) is
      external "C"
      end
   
   efox_cursor_manager__contains(str: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_cursor_manager__find(str: POINTER) : AGUI_CURSOR is
      external "C"
      end
   
   efox_cursor_manager__list_all_names : POINTER is
      external "C"
      end   
   
   efox_cursor_manager__begin_wait_cursor is
      external "C"
      end
   
   efox_cursor_manager__end_wait_cursor is
      external "C"
      end
   
   efox_cursor_manager__set_wait_cursor (chnd: POINTER) is
      external "C"
      end
   
   efox_cursor_manager__wait_cursor : AGUI_CURSOR is
      external "C"
      end
   
   efox_cursor_manager__set_default_cursor (chnd : POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_cursor_manager__default_cursor(n: INTEGER) : AGUI_CURSOR is
      external "C"
      end
   
end

      
