class AGUI_DIALOG_WINDOW
   
--  You are not allowed to destroy a dialog window, when
--  it is running modal      
--  If you click the "close" Window button than the dialog
--  will be hidden and destroyed just before return from
--  the modal loop , which means just before returning from
--  "execute_modal".
--  The normal way of use is:   
--
--     create Dialog 
--     build_dialg_widgets
--     dialog.execute_modal
--     check dialog.last_execute_result
--
   
inherit
   AGUI_TOPLEVEL_WINDOW
      rename make as make_toplevel
      end
     
     
creation
   {ANY} make, make_and_stay_on_top
     

feature {NONE} -- constructor
   
   
   make (name: STRING) is 
      -- creates a hidden toplevel window
      require
	 name /= Void
      do
	 handle := efox_dialog_window__make(Current,default_pointer, name.to_external,$temp_px)
	 efox__window_hash_table.put(Current, temp_px)
	 create accelerator_table.make(handle)
	 set_on_query_close(agent on_query_close_agent)
      end
   
   
   make_and_stay_on_top(win: AGUI_WINDOW; name: STRING) is
	 -- creates a hidden toplevel window, that will always stay
	 -- on top of win.
      require
	 name /= Void
	 win /= Void and then not win.is_zombie
      do
	 handle := efox_dialog_window__make(Current,win.handle,name.to_external,$temp_px)
	 efox__window_hash_table.put(Current, temp_px)
	 create accelerator_table.make(handle)
	 set_on_query_close(agent on_query_close_agent)
      end
      
   
feature {ANY}  -- controlling the modal loop
   
   
   execute_modal (placement: INTEGER) is
	 -- Places the window, makes it visible and starts the modal loop.
	 -- After the call check the status of "last_execute_result" 
	 -- to get the return code.
      require
	 not is_zombie
	 not is_running_modal
	 placement.abs.in_range(Placement_default,Placement_fixed)
      do
	 destroy_after_modal_flag := false
	 last_execute_result := efox_dialog_window__execute_modal(handle, placement)
	 if destroy_after_modal_flag then destroy end
      end
   
      
   last_execute_result : INTEGER
        -- return value from last modal loop
   
   
   is_running_modal : BOOLEAN is
      do
	 result := efox_dialog_window__is_running_modal(handle)
      end
   
   
   end_modal (val: INTEGER) is
      do
	 efox_dialog_window__end_modal(handle,val)
      end
   
   
feature {NONE}  -- private implementation
   
   
   destroy_after_modal_flag : BOOLEAN
   
   
   on_query_close_agent : BOOLEAN is
      do
	 if is_running_modal then
	    -- this also breaks the modal loop
	    set_visible(false)
	    -- set flag so that we know we must delete ourself 
	    -- after returning from modal loop
	    destroy_after_modal_flag := true
	 else
	    destroy
	 end
	 result := true
      end
      
   
feature {NONE} -- private C binding   
   
   
   efox_dialog_window__make(this:like Current; hnd, n, keyptr: POINTER) : POINTER is
      external "C"
      end
   
   efox_dialog_window__execute_modal(hnd: POINTER; p: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_dialog_window__end_modal(hnd: POINTER; p: INTEGER) is
      external "C"
      end
   
   efox_dialog_window__is_running_modal(hnd: POINTER) : BOOLEAN is
      external "C"
      end

end
	 
   
