class  AGUI_EVENT_PORT
   
-- Used for communication between two processes.
   

creation 
   {ANY} make


feature {ANY}
   
   
   
   transmit (recipient: STRING; kind, param1, param2: INTEGER) is
	 -- Transmit a message to a recipient.
      do
	 transmit_result := efox_event_port__transmit(recipient.to_external, kind, param1, param2)
      end
   
   
   transmit_result : BOOLEAN
	 -- Result of the last transmit handler.
   
   
   on_receive : ROUTINE[ANY,TUPLE[INTEGER,INTEGER,INTEGER]]
	 -- The agent which is called whenever an event is received.
	 -- The first integer parameter is the message kind, the second
	 -- are 'param1' and 'param2'. 
	 -- If we receive an agent an no agent is installed then the
	 -- message is lost.
   
   
   set_on_receive (action: ROUTINE[ANY,TUPLE[INTEGER,INTEGER,INTEGER]]) is
	 -- The the receive agent.
      do
	 on_receive := action
      end
   
   
   is_open : BOOLEAN is
	 -- Test if port is open.
      do
	 result := handle /= default_pointer
      end
   
   
   open is
	 -- Open a new port.
      require
	 not is_open
      local
	 p : POINTER
      do
	 p := efox_event_port__open(Current,$handle)
	 if p /= default_pointer then
	    check handle /= default_pointer end
	    create name.from_external_copy(p)
	 else
	    check handle = default_pointer end
	    name := Void
	 end
      ensure
	 is_open implies name /= Void
	 not is_open implies name = Void
      end
   
   
   close is
	 -- Close the port. No more messages will be received and
	 -- all who wants to send messages to this port fail.
      require
	 is_open
      do
	 efox_event_port__close(handle)
	 name := Void
	 handle := default_pointer
      end
   
   
   name : STRING
	 -- Name of port which can must be used in the transmit operation.
	 -- You can pass the name to child processes.
   
   
   register_message (message_name: STRING) : INTEGER is
	 -- Register the message described by 'string' as a unique system 
	 -- integer value which can be used by 'transmit'.
      do
	 result := efox_event_port__register_message(message_name.to_external)
      end
   
   

feature {NONE} -- constructor
   
   
   make is
      do
      end
   

feature {NONE} -- private C binding
   
   handle : POINTER
	 -- Handle for the c side
   
   efox_event_port__open(this:like Current; p: POINTER) : POINTER is
      external "C"
      end
   
   efox_event_port__close (p: POINTER) is
      external "C"
      end

   efox_event_port__transmit (p: POINTER;  k,p1,p2: INTEGER) : BOOLEAN is
      external "C"
      end      
   
   efox_event_port__register_message(msg: POINTER) : INTEGER is
      external "C"
      end
   
   
   efox__callback_receive (k,p1,p2: INTEGER) is
      do
	 if on_receive /= Void then
	    on_receive.call([k,p1,p2])
	 end
      end

end
