deferred class  AGUI_EXPANDED_LIST_BASE
   
   
inherit
   AGUI_LIST_BASE
      redefine 
	 item_type, 
	 efox__callback_context_menu,
	 efox__callback_query_tip,
	 efox__callback_specific1
      end
   AGUI_SCROLLING_MIXIN
      end
   
   
   
feature {ANY}
   
   item_type : AGUI_EXPANDED_LIST_BASE_ITEM
   
   
feature {ANY} -- selection mangement
   
   
   List_extended_selection : INTEGER is 0
	 -- Extended selection mode allows for drag-selection of ranges of items
   
   List_single_selection : INTEGER is 1
	 -- Single selection mode allows up to one item to be selected
   
   List_browse_selection : INTEGER is 2
	 -- Browse selection mode enforces one single item to be selected at all times
   
   List_multiple_selection : INTEGER is 3
	 -- Multiple selection mode is used for selection of individual items
   

   set_select_style (n: INTEGER) is
      require
	 not is_zombie
	 n.in_range(List_extended_selection, List_multiple_selection)
      do
	 efox_expanded_list_base__set_style(handle,n)
      end
   
   
   select_style : INTEGER is
      require
	 not is_zombie
      do
	 result := efox_expanded_list_base__style(handle)
      end
 
   
   count_selected : INTEGER is
	 -- Number of selected items.
      require
	 not is_zombie
      do
	 result := efox_expanded_list_base__count_selected(handle)
      end
     
   
   toggle_selected is
	 -- Toggle the select state for all items.
      require
	 not is_zombie
      do
	 efox_expanded_list_base__toggle_selected(handle)
      end
   
   
   unselect_all is
	 -- Deselect all items.
      require
	 not is_zombie
      do
	 efox_expanded_list_base__unselect_all(handle)
      end
   
   
   select_all is
	 -- Select all items.
      require
	 not is_zombie
      do
	 efox_expanded_list_base__select_all(handle)
      end
   
   
   do_all_selected (action: ROUTINE[ANY,TUPLE[AGUI_LIST_BASE_ITEM]]) is
      require
	 not is_zombie
	 action /= Void
      local
	 i,n: INTEGER
	 obj: like item_type
      do
	 from
	    i := 1
	    n := count
	 until
	    i > n
	 loop
	    obj := item(i)
	    if obj.selected then
	       action.call([obj])      
	    end
	    i := i + 1
	 end
      end
   
   
   append_selected_in (collection: COLLECTION[like item_type]) is
	 -- Travers the list and add all selected items to the collection.
      obsolete "Smalleiffel compiler error, so we removed this method"
      require
	 not is_zombie
	 collection /= Void
      local
	 i,n : INTEGER
	 p : like item_type
      do
	 from
	    i := 1
	    n := count
	 until
	    i > n
	 loop
	    p := item(i)
	    if p.selected then	    
	       collection.add_first(p)  -- this is a smalleiffel error
	    end
	    i := i + 1
	 end
      end
   

feature {ANY}  -- misc
   
   
   item_at (x,y: INTEGER) : like item_type is
	 -- Return item at coordinate (x,y), if any.
      require
	 not is_zombie
      do
	 result := efox_expanded_list_base__item_at(handle,x,y)
      end
   
   
   move_current_item (updir, wrap: BOOLEAN) is
	 -- Moves the current item one step up or down.
	 -- If at beginning and 'wrap' we go to the last item.
      require
	 not is_zombie
      local
	 li: like item_type
	 idx: INTEGER
      do
	 li := current_item
	 if li /= Void then
	    idx := li.index
	    if count > 0 then
	       if updir then
		  if idx = 1 then
		     if wrap then
			set_current_item(item(count))
		     end
		  else
		     set_current_item(item(idx-1))
		  end
	       else
		  if  idx = count then
		     if wrap then
			set_current_item(item(1))
		     end
		  else
		     set_current_item(item(idx+1))
		  end
	       end
	    end
	    current_item.make_visible
	 end
      end
   
   
feature {ANY} -- event handling
   
   
   on_query_tip : PROCEDURE[ANY,TUPLE[AGUI_EXPANDED_LIST_BASE_ITEM,STRING]] is
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_tooltip_event)
      end
   
   
   set_on_query_tip(action : PROCEDURE[ANY,TUPLE[AGUI_EXPANDED_LIST_BASE_ITEM,STRING]]) is
      require
	 not is_zombie
      do
	 set_callback_agent(On_tooltip_event, action)
      end
   
      
   on_context_menu : PROCEDURE[ANY,TUPLE[AGUI_EXPANDED_LIST_BASE_ITEM,AGUI_POPUP_MENU]] is
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_context_menu_event)
      end
   
   
   set_on_context_menu(action : PROCEDURE[ANY,TUPLE[AGUI_EXPANDED_LIST_BASE_ITEM,AGUI_POPUP_MENU]]) is
      require
	 not is_zombie
      do
	 set_callback_agent(On_context_menu_event, action)
      end
   
      
   on_touched : PROCEDURE[ANY,TUPLE[AGUI_EXPANDED_LIST_BASE_ITEM]] is
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_window_specific1_event)
      end
   
   
   set_on_touched (action : PROCEDURE[ANY,TUPLE[AGUI_EXPANDED_LIST_BASE_ITEM]]) is
      require
	 not is_zombie
      do
	 set_callback_agent(On_window_specific1_event, action)
      end

feature {ANY} -- size spezifikation
   
   
   visible_width : INTEGER is
	 -- default width of the list, measured in characters if greater
	 -- then 0 or in pixel if less then 0.
      require
         not is_zombie
      do
	 result := efox_expanded_list_base__visible_width(handle)
      end
   

   visible_height : INTEGER is
	 -- default height of the list, measured in rows if greater
	 -- then 0 or in pixel if less then 0.
      require
         not is_zombie
      do
	 result := efox_expanded_list_base__visible_height(handle)
      end
            

   set_visible_size (w, h: INTEGER) is
         -- if you set the visible size to 0, then the size is based on 
         -- the content. Otherwise you can specify in character or pixel.
      require
         not is_zombie
      do
         efox_expanded_list_base__set_visible_size(handle, w, h)
      end
   
      
feature {NONE} -- private section
   
   
   efox__callback_specific1(what: INTEGER; state: BOOLEAN obj: ANY) is
      local
	 a: ROUTINE[ANY, TUPLE[AGUI_EXPANDED_LIST_BASE_ITEM]]
	 i: AGUI_EXPANDED_LIST_BASE_ITEM
      do
	 a := on_touched
	 if a /= Void then
	    i ?= obj
	    check i /= Void end
	    a.call([i])
	 end
      end
   
   efox__callback_query_tip (major,minor: INTEGER; obj: ANY) : POINTER is
      local
	 a: ROUTINE[ANY, TUPLE[AGUI_EXPANDED_LIST_BASE_ITEM,STRING]]
	 i: AGUI_EXPANDED_LIST_BASE_ITEM
      do
	 efox__shared_tooltip_string.clear
	 a := on_query_tip
	 if a /= Void then
	    i ?= obj
	    check i /= Void end
	    a.call([i,efox__shared_tooltip_string])
	 end
	 result := efox__shared_tooltip_string.to_external
      end
   
   efox__callback_context_menu (idx: INTEGER; obj: ANY) : POINTER is
      local
	 a: PROCEDURE[ANY,TUPLE[AGUI_EXPANDED_LIST_BASE_ITEM,AGUI_POPUP_MENU]]
	 i: AGUI_EXPANDED_LIST_BASE_ITEM
      do
	 a := on_context_menu
	 if a /= Void then
	    i ?= obj
	    check i /= Void end
	    efox__shared_context_menu.clear
	    a.call([i,efox__shared_context_menu])
	    result := efox__shared_context_menu.handle
	 end	 
      end
      
      
   efox_expanded_list_base__set_style(hnd: POINTER; n: INTEGER) is
      external "C"
      end

   efox_expanded_list_base__style(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_expanded_list_base__count_selected(hnd: POINTER) : INTEGER is
      external "C"
      end

   efox_expanded_list_base__unselect_all(hnd: POINTER) is
      external "C"
      end
   
   efox_expanded_list_base__select_all(hnd: POINTER) is
      external "C"
      end   
	 
   efox_expanded_list_base__toggle_selected(hnd: POINTER) is
      external "C"
      end
   
   efox_expanded_list_base__item_at(hnd: POINTER; x,y: INTEGER) : like item_type is
      external "C"
      end

   efox_expanded_list_base__visible_height(hnd: POINTER): INTEGER is
      external "C"
      end
   
   efox_expanded_list_base__visible_width(hnd: POINTER): INTEGER is
      external "C"
      end
 
   efox_expanded_list_base__set_visible_size(hnd: POINTER; w,h: INTEGER) is
      external "C"
      end
   
end
