deferred class AGUI_EXPANDED_TREE_BASE
   
   
inherit
   AGUI_TREE_BASE
      redefine
	 efox__callback_specific1,
	 efox__callback_specific2,
	 efox__callback_query_tip,
	 efox__callback_context_menu
      end
   AGUI_SCROLLING_MIXIN
      end
   

feature {ANY} -- selection mangement
   
   
   Tree_extended_selection : INTEGER is 0
	 -- Extended selection mode allows for drag-selection of ranges of items
   
   Tree_single_selection : INTEGER is 1
	 -- Single selection mode allows up to one item to be selected
   
   Tree_browse_selection : INTEGER is 2
	 -- Browse selection mode enforces one single item to be selected at all times
   
   Tree_multiple_selection : INTEGER is 3
	 -- Multiple selection mode is used for selection of individual items
      
   
   select_style : INTEGER is
	 -- Return tree style.
      require
         not is_zombie
      do
	 result := efox_expanded_tree_base__select_style(handle)
      ensure
	 result.in_range(Tree_extended_selection, Tree_multiple_selection)
      end
   
   
   set_select_style (n: INTEGER) is
	 -- Change tree style.
      require
         not is_zombie
	 n.in_range(Tree_extended_selection, Tree_multiple_selection)
      do
	 efox_expanded_tree_base__set_select_style(handle,n)
      end
   
   
feature {ANY}  -- retrieve item
   
   
   item_at (x,y: INTEGER) : AGUI_TREE_BASE_ITEM is
	 -- Return item at coordinate (x,y) or Void.
      do
	 result := efox_expanded_tree_base__item_at(handle,x,y)
      end
   
   
feature {ANY}  -- specify default size
   
   
   visible_width : INTEGER is
	 -- Default width of the list, measured in characters if greater
	 -- then 0 or in pixel if less then 0.
      require
         not is_zombie
      do
	 result := efox_expanded_tree_base__visible_width(handle)
      end
   
   
   visible_height : INTEGER is
	 -- Default height of the list, measured in rows if greater
	 -- then 0 or in pixel if less then 0.
      require
         not is_zombie
      do
	 result := efox_expanded_tree_base__visible_height(handle)
      end
   
   
   set_visible_size (w, h: INTEGER) is
         -- If you set the visible size to 0, then the size is based on 
         -- the content. Otherwise you can specify in character or pixel.
      require
         not is_zombie
      do
         efox_expanded_tree_base__set_visible_size(handle, w, h)
      end
   
   
feature {ANY}  -- event handlers
   
   
   on_collapsed : PROCEDURE[ANY,TUPLE[AGUI_TREE_BASE_ITEM]] is
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_window_specific2_event)
      end
   
   
   set_on_collapsed(action : PROCEDURE[ANY,TUPLE[AGUI_TREE_BASE_ITEM]]) is
      require
	 not is_zombie
      do
	 set_callback_agent(On_window_specific2_event, action)
      end
   
   
   on_expanded : PROCEDURE[ANY,TUPLE[AGUI_TREE_BASE_ITEM]] is
      require
	 not is_zombie
      do      
	 result ?= find_callback_agent(On_window_specific1_event)
      end
   
   
   set_on_expanded(action : PROCEDURE[ANY,TUPLE[AGUI_TREE_BASE_ITEM]]) is
      require
	 not is_zombie
      do
	 set_callback_agent(On_window_specific1_event, action)
      end
      
   
   on_context_menu : PROCEDURE[ANY,TUPLE[AGUI_TREE_BASE_ITEM,AGUI_POPUP_MENU]] is
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_context_menu_event)
      end
   
   
   set_on_context_menu(action : PROCEDURE[ANY,TUPLE[AGUI_TREE_BASE_ITEM,AGUI_POPUP_MENU]]) is
      require
	 not is_zombie
      do
	 set_callback_agent(On_context_menu_event, action)
      end
   
   
   on_query_tip : PROCEDURE[ANY,TUPLE[AGUI_TREE_BASE_ITEM,STRING]] is
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_tooltip_event)
      end
   
   
   set_on_query_tip(action : PROCEDURE[ANY,TUPLE[AGUI_TREE_BASE_ITEM,STRING]]) is
      require
	 not is_zombie
      do
	 set_callback_agent(On_tooltip_event, action)
      end
   
   
feature {NONE} -- private C binding   
   
   
   efox__callback_query_tip (major,minor: INTEGER; obj: ANY) : POINTER is
      local
	 a: ROUTINE[ANY, TUPLE[AGUI_TREE_BASE_ITEM,STRING]]
	 i: AGUI_TREE_BASE_ITEM
      do
	 efox__shared_tooltip_string.clear
	 a := on_query_tip
	 if a /= Void then
	    i ?= obj
	    check i /= Void end
	    a.call([i,efox__shared_tooltip_string])
	 end
	 result := efox__shared_tooltip_string.to_external
      end
   
      
   efox__callback_context_menu (idx: INTEGER; obj: ANY) : POINTER is
      local
	 a: ROUTINE[ANY, TUPLE[AGUI_TREE_BASE_ITEM, AGUI_POPUP_MENU]]
	 i: AGUI_TREE_BASE_ITEM
      do
	 a := on_context_menu
	 if a /= Void then
	    i ?= obj
	    check i /= Void end
	    efox__shared_context_menu.clear
	    a.call([i,efox__shared_context_menu])
	    result := efox__shared_context_menu.handle
	 end	 
      end
   
   
   efox__callback_specific2(what: INTEGER; state: BOOLEAN obj: ANY) is
      local
	 a: ROUTINE[ANY, TUPLE[AGUI_TREE_BASE_ITEM]]
	 i: AGUI_TREE_BASE_ITEM
      do
	 a := on_collapsed
	 if a /= Void then
	    i ?= obj
	    check i /= Void end
	    a.call([i])
	 end
      end
   
   
   efox__callback_specific1(what: INTEGER; state: BOOLEAN obj: ANY) is
      local
	 a: ROUTINE[ANY, TUPLE[AGUI_TREE_BASE_ITEM]]
	 i: AGUI_TREE_BASE_ITEM
      do
	 a := on_expanded
	 if a /= Void then
	    i ?= obj
	    check i /= Void end
	    a.call([i])
	 end
      end
   
   
   ------------------------------------------
   
   efox_expanded_tree_base__set_select_style(hnd: POINTER; n: INTEGER) is
      external "C"
      end     
      
   efox_expanded_tree_base__select_style(hnd: POINTER) : INTEGER is
      external "C"
      end     

   efox_expanded_tree_base__item_at(hnd: POINTER; x,y: INTEGER) : AGUI_TREE_BASE_ITEM is
      external "C"
      end     
   
   efox_expanded_tree_base__visible_width(hnd: POINTER) : INTEGER is
      external "C"
      end
			    
   efox_expanded_tree_base__visible_height(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_expanded_tree_base__set_visible_size(hnd: POINTER; w,h: INTEGER) is
      external "C"
      end     


end
   
   
   
