class  AGUI_FILE_DIALOG
   
   
inherit
   AGUI_PREDEFINED_DIALOG
      end
   

creation
   make
   
   
feature {ANY}  -- selected files
	 
   
   set_file_name(s: STRING) is
	 -- Change file name.
	 -- This is the default file name value. A leading directory is
	 -- ignored and must be set via 'set_directory'.
      require
	 not is_zombie
      do
	 efox_file_dialog__set_file_name(handle, s.to_external)
      end
   
   
   file_names : ARRAY[STRING] is
	 -- Return all selected filenames.
	 -- Use this methods only in multiple select modus.
      require
	 not is_zombie
      local
	 p: POINTER
	 s: STRING
	 stools : AGUI_STRING_TOOLS
      do
	 p := efox_file_dialog__file_names(handle)
	 if p = default_pointer then
	    create result.with_capacity(1,1)
	 else
	    create s.from_external_copy(p)
	    result := stools.split_newline_separated_string(s)
	 end
      ensure
	 result /= Void
	 result.lower = 1
      end
   
   
   file_name : STRING is
	 -- Return the selected filename.
	 -- Use this methods only in singe select modus.
	 -- The result is empty if there is no selection made.
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_file_dialog__file_name(handle))
      end
      
      
feature {ANY}  -- properties
         
   
   Select_any : INTEGER is 0
	 -- A single file, existing or not (to save to)
   
   Select_existing : INTEGER is 1
	 -- An existing file (to load)
   
   Select_multiple : INTEGER is 2
	 -- Multiple existing files
   
   Select_directory : INTEGER is 3
	 -- Existing directory
   
   Select_multiple_all : INTEGER is 666
	 -- Multiple existing files or directories.
   
   
   set_pattern_list(a: ARRAY[STRING]) is
	 -- Set the file patterns shown in the file dialog.
	 -- Each pattern comprises an optional name, followed by a pattern in
	 -- parentheses. For example,
	 --        << "*" , "*.cpp,*.cc" , "*.hpp,*.hh,*.h" >>
	 -- and
	 --       <<"All Files (*)", "C++ Sources (*.cpp,*.cc)", "C++ Headers (*.hpp,*.hh,*.h)" >>
	 -- will set the same three patterns, but the former shows no pattern names.
      require
	 not is_zombie
      local
	 s: STRING
	 i,n: INTEGER
      do
	 create s.make(120)
	 from
	    i := a.lower
	    n := a.upper
	 until
	    i > n 
	 loop
	    s.append(a.item(i))
	    s.extend('%N')
	    i := i + 1
	 end
	 efox_file_dialog__set_pattern_list(handle, s.to_external)
      end
   
   
   pattern_list : ARRAY[STRING] is
	 -- Return a new array with all defined patterns.
      require
	 not is_zombie
      local
	 s : STRING
	 stools : AGUI_STRING_TOOLS
      do
	 create s.from_external_copy(efox_file_dialog__pattern_list(handle))
	 result := stools.split_newline_separated_string(s)
      ensure
	 result /= Void
	 result.lower = 1
      end
   
   
   set_current_pattern (n: INTEGER) is
	 -- After setting the list of patterns, this call will
	 -- initially select pattern 'n' as the active one.
      require
	 not is_zombie
	 n.in_range(1, pattern_list.count)
      do
	 efox_file_dialog__set_current_pattern(handle, n)
      end
   
   
   current_pattern : INTEGER is
	 -- Return current pattern number
      require
	 not is_zombie
      do
	 result := efox_file_dialog__current_pattern(handle)
      ensure
	 result.in_range(1, pattern_list.count)
      end
   
   
   set_select_mode (n: INTEGER) is
	 -- Change file selection mode
      require
	 not is_zombie
	 n.in_range(Select_any, Select_directory) or else n=Select_multiple_all
      do
	 efox_file_dialog__set_select_mode(handle, n)
      end
   
   
   select_mode : INTEGER is
	 -- Return file selection mode
      require
	 not is_zombie
      do
	 result := efox_file_dialog__select_mode(handle)
      end
   
   
   directory : STRING is
	 -- Get directory.
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_file_dialog__directory(handle))
      end
   
   
   set_directory (str: STRING) is
	 -- Set default directory.
      require
	 not is_zombie
      do
	 efox_file_dialog__set_directory(handle, str.to_external)
      end
   
   
feature {NONE} -- Constructor
   
   
   make (owner: AGUI_WINDOW) is
	 -- Create dialog that stays on top of 'owner'.
      require
	 not owner.is_zombie
      do
	 handle := efox_file_dialog__make(Current,owner.handle,$temp_key)
	 check not efox__window_hash_table.has(temp_key) end
	 efox__window_hash_table.put(Current, temp_key)
      end
   

feature {NONE} -- private C binding
   
   
   efox_file_dialog__make(this:like Current; ptr,keyptr: POINTER) : POINTER is
      external "C"
      end
      
   efox_file_dialog__directory(h: POINTER) : POINTER is
      external "C"
      end
			    
   efox_file_dialog__set_directory(h,p: POINTER) is
      external "C"
      end
   
   efox_file_dialog__set_pattern_list(h,p: POINTER) is
      external "C"
      end

   efox_file_dialog__pattern_list(h: POINTER) : POINTER is
      external "C"
      end

   efox_file_dialog__set_current_pattern(h: POINTER; n: INTEGER) is
      external "C"
      end

   efox_file_dialog__current_pattern(h: POINTER) : INTEGER is
      external "C"
      end
   
   efox_file_dialog__set_select_mode(h: POINTER; n: INTEGER) is
      external "C"
      end

   efox_file_dialog__select_mode(h: POINTER) : INTEGER is
      external "C"
      end
   
   efox_file_dialog__set_file_name(h,s: POINTER) is
      external "C"
      end
   
   efox_file_dialog__file_names (h: POINTER) : POINTER is
      external "C"
      end
      
   efox_file_dialog__file_name (h: POINTER) : POINTER is
      external "C"
      end
      
end
