expanded class  AGUI_FILE_TOOLS

--TODO remember send mail to FOX list that 'size' is 32 bit only
--TODO Error in concatenate + absolute_from|absolute   | touch => set-modified-time


feature {ANY} -- file name matching
   
   --
   -- Match patterns are as follows:
   -- 
   --   ?           Matches single character.
   --   *           Matches zero or more characters.
   --   [abc]       Matches a single character, which must be a, b, or c.
   --   [^abc]      Matches a single character, which must be anything other than a, b, or c.
   --   [!abc]      Ditto.
   --   [a-zA-Z]    Matches single character, which must be one of a-z or A-Z.
   --   [^a-zA-Z]   Matches single character, which must be anything other than a-z or A-Z.
   --   [!a-zA-Z]   Ditto.
   --   pat1|pat2   Matches either pat1 or pat2.
   --   pat1,pat2   Ditto.
   --   (pat1|pat2) Matches either pat1 or pat2; patterns may be nested.
   --   (pat1,pat2) Ditto.
   --   
   -- Examples:
   -- 
   --   *.cpp|*.cc|*.cxx|*.C  Matches some common extensions for C++ source files.
   --   
   --   image.(bmp,gif,jpg)   Matches a file called image given as either bmp, gif, or jpg.
   --   
   --   *.[^o]                Matches any file except object files.
   --  
   -- - You can escape meta characters like '?', '*', '(', ')', '|', '^', '!', and ',' 
   --   with the backslash '\'.
   --   
   -- Note that under Windows, FILEMATCH_NOESCAPE must be passed
   --
   
   
   Match_file_name : INTEGER is 1
	 -- No wildcard can ever match "/" (or "\","/" under Windows).
   
   Match_no_escape : INTEGER is 2
	 -- Backslashes don't quote special chars
  
   Match_period : INTEGER is 4
	 -- Leading "." is matched only explicitly (Useful to match hidden files on Unix).
   
   Match_leading_directory : INTEGER is 8
	 -- Ignore "/..." after a match.
   
   Match_case_insensitive : INTEGER is 16
	 -- Compare without regard to case.
   
   
   match_name (pattern, filename: STRING; flags: INTEGER) : BOOLEAN is
	 -- test if 'filename' matches 'pattern'
      require
	 flags.in_range(0,31)
      do
	 result := efox_file_tools__match_name(pattern.to_external, filename.to_external, flags)
      end
   
   
feature {ANY}  -- directory file names
   
   
   path_separator : STRING is 
	 -- String with the character that separates directories.
      local
	 s: AGUI_SYSTEM
      do
	 result := s.path_separator
      end
   
   
   pathlist_separator : STRING is 
	 -- String with the character that separates entries in a pathlist.
      local
	 s: AGUI_SYSTEM
      do
	 result := s.pathlist_separator
      end
   
   
   home_directory : STRING is
	 -- Return the home directory for the current user.
      do
	 create result.from_external_copy(efox_file_tools__home_directory)
      ensure
	 result /= Void
      end
   
   
   user_directory (user_name: STRING) : STRING is
	 -- Return the home directory for a given user.
      require
	 user_name /= Void
      do
	 create result.from_external_copy(efox_file_tools__user_directory(user_name.to_external))
      ensure
	 result /= Void
      end
   
   
   current_drive : STRING is
	 --Return the current drive (for Win32 systems)
      do
	 create result.from_external_copy(efox_file_tools__current_drive)
      ensure
	 result /= Void
      end
   
   
   current_directory : STRING is
	 -- Get the current working directory
      do
	 create result.from_external_copy(efox_file_tools__current_directory)
      ensure
	 result /= Void
      end
   
   
   set_current_directory (str: STRING) is
	 -- Change current directory
      require
	 str /= Void
      do
	 efox_file_tools__set_current_directory(str.to_external)
      end
   
   
   set_current_drive (str: STRING) is
	 --  Change current drive prefix "a:"
      require
	 str /= Void
      do
	 efox_file_tools__set_current_drive(str.to_external)
      end
   
   
   binary_file : STRING is
	 -- Returns the file name of the executeable
      do
	 create result.from_external_copy(efox_file_tools__binary_file)
      end

   
feature {ANY}  -- file name and path operations
   
   
   enquote (path: STRING; force_quotes: BOOLEAN) : STRING is
	 -- Enquote filename to make safe for shell
	 -- If 'force_quotes' is true than it is garanteed that the
	 -- modified path is surrounded by '' (UNIX) or "" (WINDOWS) quotes.
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__enquote(path.to_external,force_quotes))
      end
   
   
   dequote (path: STRING) : STRING is
	 -- Decode filename to get original again
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__dequote(path.to_external))
      end      
   
   
   drive (path: STRING) : STRING is
	 -- Return drive letter prefix "c:"
	 -- Under UNIX it always returns ""
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__drive(path.to_external))
      end
   
   
   directory (path: STRING) : STRING is
	 -- Return directory part of pathname, assuming full pathname
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__directory(path.to_external))
      end

   
   name (path: STRING) : STRING is
	 -- Return name and extension part of pathname
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__name(path.to_external))
      end
   
   
   title (path: STRING) : STRING is
	 -- Return file title, i.e. document name only
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__title(path.to_external))
      end
      
   
   extension (path: STRING) : STRING is
	 -- Return extension, if there is one
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__extension(path.to_external))
      end
   
   
   strip_extension (path: STRING) : STRING is
	 -- Remove extension from file name
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__strip_extension(path.to_external))
      end
   
   
   simplify_path (path: STRING) : STRING is
      -- Simplify a file path; the path will remain relative if it was relative
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__simplify_path(path.to_external))
      end
   
   
   expand (path: STRING) : STRING is
	 -- Perform tilde or environment variable expansion
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__expand(path.to_external))
      end
   
   
   absolute(path: STRING) : STRING is
	 -- Build absolute pathname
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__absolute(path.to_external))
      end
      
   
   absolute_from_base(base: STRING; file: STRING) : STRING is
	 -- Build absolute pathname from parts.
	 -- Result is returned in 'file'
      require
	 file /= Void
	 base /= Void
      do
	 create result.from_external_copy(efox_file_tools__absolute_from_base(base.to_external,file.to_external))
      end
   
   
   unique_name (template: STRING) : STRING is
	 -- Generate unique filename of the form pathnameXXX.ext, where
	 -- pathname.ext is the original input file, and XXX is a number,
	 -- possibly empty, that makes the file unique.
	 -- Returns an empty string if there are to much (1000) temporary files.
      require
	 template /= Void
      do
	 create result.from_external_copy(efox_file_tools__unique_name(template.to_external))
      end
   
   
   up_level (path: STRING) : STRING is
	 -- Up one level, given absolute path
	 -- Does nothing if this is the top directory.
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__up_level(path.to_external))
      end
   
   
   is_absolute (path: STRING) : BOOLEAN is
	 -- Check if file represents absolute pathname
      require
	 path /= Void
      do
	 result := efox_file_tools__is_absolute(path.to_external)
      end
   
   
   is_top_directory (path: STRING) : BOOLEAN is
	 -- Does file represent topmost directory
      require
	 path /= Void
      do
	 result := efox_file_tools__is_top_directory(path.to_external)
      end
   
   
   search (pathlist, file: STRING) : STRING is      
	 -- Search pathlist for file.
      require
	 pathlist /= Void
	 file /= Void
      do
	 create result.from_external_copy(efox_file_tools__search(pathlist.to_external, file.to_external))
      end
	 
   
feature {ANY}  -- information about files 
   
   
   is_file (path: STRING) : BOOLEAN is
      require
	 path /= Void
      do
	 result := efox_file_tools__is_file(path.to_external)
      end
   
   is_directory (path: STRING) : BOOLEAN is
      require
	 path /= Void
      do
	 result := efox_file_tools__is_directory(path.to_external)
      end
   
   is_link (path: STRING) : BOOLEAN is
      require
	 path /= Void
      do
	 result := efox_file_tools__is_link(path.to_external)
      end
   
   is_archived (path: STRING) : BOOLEAN is
	 -- Test if the file has the archived bit set.
	 -- This method has an effect only under Windows.
      do
	 result := efox_file_tools__is_archived(path.to_external)
      end
   
   is_readable (path: STRING) : BOOLEAN is
      require
	 path /= Void
      do
	 result := efox_file_tools__is_readable(path.to_external)
      end
   
   is_writeable (path: STRING) : BOOLEAN is
      require
	 path /= Void
      do
	 result := efox_file_tools__is_writeable(path.to_external)
      end
   
   is_executable (path: STRING) : BOOLEAN is
      require
	 path /= Void
      local
	 s: AGUI_SYSTEM
      do
	 if s.is_windows then
	    result := path.has_suffix(".exe") or else path.has_suffix(".com") or else path.has_suffix(".bat")
	 else
	    result := efox_file_tools__is_executable(path.to_external)
	 end
      end
   
   
   modified_time(path: STRING) : TIME is
      do
	 result := file_tools.last_change_of(path)
      end
      
   
   created_time(path: STRING) : TIME is
      obsolete "Do Not Use - Not implemented"
      do
	 --TODO result := file_tools.created_of(path)
      end
   
   
   size (path: STRING) : INTEGER is
	 -- Return size of file.
      require
	 path /= Void
      do
	 result := efox_file_tools__size(path.to_external)
      end
   
   
   exists (path: STRING) : BOOLEAN is
      require
	 path /= Void
      do
	 result := efox_file_tools__exists(path.to_external)
      end
   
   
   is_empty (path: STRING) : BOOLEAN is
	 -- True if 'path' file exists, is readable and is an empty file
      require
	 path /= Void
      do
	 result := efox_file_tools__is_empty(path.to_external)
      end
   
   
   read_symbolic_link (path: STRING) : STRING is
      -- Read symbolic link
      require
	 path /= Void
      do
	 create result.from_external_copy(efox_file_tools__read_symlink(path.to_external))
      ensure
	 result /= Void
      end
      
   
   same_files (path1, path2: STRING) : BOOLEAN is
	 -- True if the 'path1' file exists and has refers to the 
	 -- same file data as file 'path2'.
      require
	 path1 /= Void
	 path2 /= Void
      do
	 result := efox_file_tools__same_files(path1.to_external, path2.to_external)
      end
	

feature {ANY}  -- Basic file operations
   
   
   set_read_only (path: STRING; new_mode: BOOLEAN) is
	 -- Sets/unsets the write permission for the file.
      do
	 efox_file_tools__set_read_only(path.to_external,new_mode)
      end
   
   
   set_archived (path: STRING; new_mode: BOOLEAN) is
	 -- Sets the archived state of the file.
	 -- This method has an effect only under Windows.
      do
	 efox_file_tools__set_archived(path.to_external,new_mode)
      end
   
   
   create_directory (path: STRING) is
	 -- Create new directory
      require
	 path /= Void
      do
	 efox_file_tools__create_directory(path.to_external)
      end
   
   
   create_file (path: STRING) is
	 -- Create new (empty) file
      require
	 path /= Void
      do
	 efox_file_tools__create_file(path.to_external)
      end

   
   concatenate (file1, file2, dest: STRING;  overwrite: BOOLEAN) is
	 -- Concatenate srcfile1 and srcfile2 to a dstfile
      require
	 file1 /= Void
	 file2 /= Void
	 dest /= Void
      do
	 efox_file_tools__concatenate(file1.to_external,file2.to_external,dest.to_external,overwrite)
      end

   
   destroy, delete, remove (path: STRING) is
	 -- Remove file or directory
      require
	 path /= Void
      do
	 efox_file_tools__remove(path.to_external)
      end
   
   copy_file (old_file, new_file: STRING; overwrite: BOOLEAN) is
	 --  Copy file or directory
      require
	 old_file /= Void
	 new_file /= Void
      do
	 efox_file_tools__copy(old_file.to_external, new_file.to_external, overwrite)
      end
   
   
   rename_to, move (old_file, new_file: STRING; overwrite: BOOLEAN) is
	 -- Rename or move file, or copy and delete old if different file systems
      require
	 old_file /= Void
	 new_file /= Void
      do
	 efox_file_tools__move(old_file.to_external, new_file.to_external, overwrite)
      end
   
   
   link (old_file, new_file: STRING; overwrite: BOOLEAN) is
	 --  Link file or directory
      require
	 old_file /= Void
	 new_file /= Void
      do
	 efox_file_tools__link(old_file.to_external, new_file.to_external, overwrite)
      end
   
   
   symbolic_link (old_file, new_file: STRING; overwrite: BOOLEAN) is
	 --  Create Symbolic link
      require
	 old_file /= Void
	 new_file /= Void
      do
	 efox_file_tools__symlink(old_file.to_external, new_file.to_external, overwrite)
      end
   
   
   scan_directory (path: STRING) : ARRAY[STRING] is
	 -- Read a directory and return a field with all names in that directory.
      require
	 path /= Void
      local
	 dir : DIRECTORY
	 i,n : INTEGER
	 s : STRING
      do
	 create result.with_capacity(20,1)
	 create dir.make
	 dir.scan(path)
	 if dir.last_scan_status then
	    from 
	       i := dir.lower 
	       n := dir.upper 
	    until 
	       i > n
	    loop
	       s := dir.name(i)
	       if not (s.is_equal(".") or else s.is_equal("..")) then
		  result.add_last(s)
	       end
	       i := i + 1
	    end
	 end
      ensure
	 result /= Void
      end
   
   
feature {ANY}  -- loading and saving of files
   

   load_content (path, content: STRING; binary: BOOLEAN) is
	 -- IMPROVE speed
      require
	 not_void: path /= Void and content /= Void
      local
	 file : TEXT_FILE_READ
	 bfile : BINARY_FILE_READ
      do
	 content.clear
	 if binary then
	    create bfile.make
	    bfile.connect_to(path)
	    if bfile.is_connected then
	       from
		  bfile.read_byte
	       until
		  bfile.end_of_input
	       loop
		  content.extend(bfile.last_byte.to_character)
		  bfile.read_byte
	       end
	       bfile.disconnect
	    end
	 else
	    create file.make
	    file.connect_to(path)
	    if file.is_connected then
	       file.read_tail_in(content)
	       file.disconnect
	    end
	 end
      end
   
   
   store_content (path, content: STRING; binary: BOOLEAN) is
	 -- IMPROVE speed
      require
	 not_void: path /= Void and content /= Void
      local
	 file : TEXT_FILE_WRITE
	 bfile : BINARY_FILE_WRITE
	 i,n: INTEGER
      do
	 if binary then
	    create bfile.make
	    bfile.connect_to(path)
	    if bfile.is_connected then
	       from
		  i := 1
		  n := content.count
	       until
		  i > n 
	       loop
		  bfile.put_byte(content.item(i).to_integer)
		  i := i + 1
	       end
	       bfile.disconnect
	    end
	 else
	    create file.make
	    file.connect_to(path)
	    if file.is_connected then
	       file.put_string(content)
	       file.disconnect
	    end
	 end
      end
   
   
feature {NONE} 
   
   -- file name matching
   
   efox_file_tools__match_name(p,f: POINTER; n: INTEGER) : BOOLEAN is
      external "C"
      end
   
   -- directory file names
   
   efox_file_tools__home_directory : POINTER is
      external "C"
      end
   
   efox_file_tools__user_directory (p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__current_drive : POINTER is
      external "C"
      end
   
   efox_file_tools__current_directory : POINTER is
      external "C"
      end
   
   efox_file_tools__set_current_directory(s: POINTER) is
      external "C"
      end
   
   efox_file_tools__set_current_drive(s: POINTER) is
      external "C"
      end
      
   efox_file_tools__binary_file : POINTER is
      external "C"
      end
   
   --- file name and path operations
   
   efox_file_tools__enquote(p: POINTER; force_quotes: BOOLEAN) : POINTER is
      external "C"
      end
   
   efox_file_tools__dequote(p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__drive (p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__directory (p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__name (p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__title (p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__extension (p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__strip_extension (p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__simplify_path (p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__expand (p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__absolute (p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__absolute_from_base (b,p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__unique_name (p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__up_level (p: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__is_absolute (p: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_file_tools__is_top_directory (p: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_file_tools__search (p,f: POINTER) : POINTER is
      external "C"
      end
   
   -- information about files 
   
   efox_file_tools__is_file (p: POINTER) : BOOLEAN is
      external "C"
      end
      
   efox_file_tools__is_directory (p: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_file_tools__is_link (p: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_file_tools__is_readable (p: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_file_tools__is_writeable (p: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_file_tools__is_executable (p: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_file_tools__is_archived(p: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_file_tools__size (p: POINTER) : INTEGER is
      external "C"
      end
   
   efox_file_tools__exists (p: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_file_tools__is_empty (p: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_file_tools__read_symlink(path: POINTER) : POINTER is
      external "C"
      end
   
   efox_file_tools__same_files(p,q: POINTER) : BOOLEAN is
      external "C"
      end

   -- basic file operations   
   
   efox_file_tools__set_read_only(p: POINTER; f: BOOLEAN) is
      external "C"
      end
   
   efox_file_tools__set_archived(p: POINTER; f: BOOLEAN) is
      external "C"
      end
	 
   efox_file_tools__create_directory (s: POINTER) is
      external "C"
      end
      
   efox_file_tools__create_file (s: POINTER) is
      external "C"
      end
      
   efox_file_tools__concatenate (f1,f2,d: POINTER; fl: BOOLEAN) is
      external "C"
      end
      
   efox_file_tools__remove (s: POINTER) is
      external "C"
      end
   
  efox_file_tools__copy (s,d: POINTER; f: BOOLEAN) is
      external "C"
      end
      
  efox_file_tools__move (s,d: POINTER; f: BOOLEAN) is
      external "C"
      end
      
  efox_file_tools__link (s,d: POINTER; f: BOOLEAN) is
      external "C"
      end
      
  efox_file_tools__symlink (s,d: POINTER; f: BOOLEAN) is
      external "C"
      end      
   
   -- loading and saving of files
       
   efox_file_tools__load_content(p,c: POINTER; fl: BOOLEAN) is
      external "C"
      end
   
   efox_file_tools__store_content(p,c: POINTER; fl: BOOLEAN) is
      external "C"
      end
      
end
   
