class  AGUI_HEADER
   
-- IMPROVE:  is_column_visible + make_column_visible
-- IMPROVE:  support the sort icon and the column marker icon
   
   
inherit
   AGUI_CONTROL
      redefine
	 efox__callback_context_menu,
	 efox__callback_query_tip,
	 efox__callback_changed,
	 efox__callback_command
      end
   AGUI_FRAME_MIXIN
      end
   
   
creation
   {ANY} make
   
   
feature {ANY} -- header style
   
   
   horizontal_orientation : BOOLEAN is
	 -- current state of orientation
      require
	 not is_zombie
      do
	 result := efox_header__horizontal_orientation(handle)
      end
   
   
   set_horizontal_orientation (flag: BOOLEAN) is
	 -- set the orientation of the header
      require
	 not is_zombie
      do
	 efox_header__set_horizontal_orientation(handle,flag)
      end
   
   
   tracking : BOOLEAN is
	 -- current state of tracking mode
      require
	 not is_zombie
      do
	 result := efox_header__tracking(handle)
      end
   
   
   set_tracking (flag: BOOLEAN) is
	 -- set if tracking information should be generated continously
      require
	 not is_zombie
      do
	 efox_header__set_tracking(handle,flag)
      end
   
   
   button_mode : BOOLEAN is
	 -- button mode state for header
      require
	 not is_zombie
      do
	 result := efox_header__button_mode(handle)
      end
   
   
   set_button_mode (flag: BOOLEAN) is
	 -- set the button mode
      require
	 not is_zombie
      do
	 efox_header__set_button_mode(handle, flag)
      end
   
   
feature {ANY} -- appearance properties
   
   
   font : AGUI_FONT is
	 -- return font
      require
	 not is_zombie
      do
	 result := efox_header__font(handle)
      end
   
   
   set_font (f: AGUI_FONT) is
	 -- set font
      require
	 not is_zombie
	 f /= Void and not f.is_zombie
      do
	 efox_header__set_font(handle, f.handle)
      end
   
   
   set_text_color (col: AGUI_COLOR) is
	 -- set text color
      require
	 not is_zombie
      do
	 efox_header__set_text_color(handle, col.to_integer)
      end
   
   
   text_color : AGUI_COLOR is
	 -- return text color
      require
	 not is_zombie
      do
	 result.from_integer(efox_header__text_color(handle))
      end
   
     
feature {ANY} -- item sorting behaviour
      
   
   Arrow_ascending : INTEGER is 0
   Arrow_descending : INTEGER is 1
   Arrow_none : INTEGER is 2
   
   
   arrow_direction (index:INTEGER) : INTEGER is
	 -- return 'Arrow_none' or the direction of the sort marker
      require
	 not is_zombie
	 valid_position(index)
      do
	 result := efox_header__arrow_direction(handle,index)
      ensure
	 result.in_range(Arrow_ascending, Arrow_none)
      end
   
   
   set_arrow_direction (index, direction: INTEGER) is
	 -- sets the arrow sort marker
      require
	 not is_zombie
	 valid_position(index)
	 direction.in_range(Arrow_ascending, Arrow_none)
      do
	 efox_header__set_arrow_direction(handle, index, direction)
      ensure
	 arrow_direction (index) = direction
      end   
   

feature {ANY}  -- item position management
   
   
   item_size (pos: INTEGER) : INTEGER is
	 -- return size of the column
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_header__item_size(handle, pos)
      end
   
   
   set_item_size (index,new_size: INTEGER) is
      require
	 not is_zombie
	 valid_position(index)
	 new_size >= 0
      do
	 efox_header__set_item_size(handle,index,new_size)
      end
   
   
   item_offset (pos: INTEGER) : INTEGER is
	 -- return offset from left side of column
      require
	 not is_zombie	 
	 valid_position(pos)
      do
	 result := efox_header__item_offset(handle, pos)
      end
   
   
   item_at (x_pos: INTEGER) : INTEGER is
	 -- query which column is at the given x pixel position
      require
	 not is_zombie
      do
	 result := efox_header__item_at(handle,x_pos)
      end
   
      
feature {ANY}  -- management of items
   
   
   count : INTEGER is
	 -- return number of columns
      require
	 not is_zombie
      do
	 result := efox_header__count(handle)
      ensure
	 result >= 0
      end
   
   
   clear is
	 -- remove all columns
      require
	 not is_zombie
      do
	 efox_header__clear(handle)
      ensure
	 count = 0
      end
   
   
   remove (pos: INTEGER) is
	 -- remove one column
      require
	 not is_zombie      
	 valid_position(pos)
      do
	 efox_header__remove(handle, pos)
      ensure
	 old count - 1 = count
      end
   
   
   append (str: STRING; icon: AGUI_ICON; size: INTEGER) is
      do
	 insert(count+1, str, icon, size)
      end
   
   insert (pos: INTEGER; str: STRING; icon: AGUI_ICON; size: INTEGER) is
	 -- insert a new column item at index position 'pos' and set the
	 -- width of the column to size pixel
      require
	 not is_zombie
	 valid_position_for_insert(pos)
	 str /= Void
	 icon /= Void implies not icon.is_zombie
	 size >= 0
      do
	 if icon /= Void then
	    efox_header__insert(handle, pos, str.to_external, icon.handle, size)
	 else
	    efox_header__insert(handle, pos, str.to_external, default_pointer, size)
	 end	    
      ensure
	 item_text(pos).is_equal(str)
	 item_icon(pos) = icon
	 old count + 1 = count
      end
   
   
   item_icon (pos: INTEGER) : AGUI_ICON is
	 -- return icon of item at index position
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_header__item_icon(handle, pos)
      end
   
   
   set_item_icon (pos: INTEGER; icon: AGUI_ICON) is
      require
	 not is_zombie
	 valid_position(pos)
	 icon /= Void implies not icon.is_zombie
      do
	 if icon /= Void then
	    efox_header__set_item_icon(handle,pos,icon.handle)
	 else
	    efox_header__set_item_icon(handle,pos,default_pointer)
	 end
      end
   
   
   item_text (pos: INTEGER) : STRING is
	 -- return text of item at index position in a new string
      require
	 not is_zombie
	 valid_position(pos)
      do
	 create result.from_external_copy(efox_header__item_text(handle, pos))
      ensure
	 result /= Void
      end
   
   
   set_item_text (pos: INTEGER; str: STRING) is
      require
	 not is_zombie
	 valid_position(pos)
	 str /= Void
      do
	 efox_header__set_item_text(handle,pos,str.to_external)
      end
   
   
feature {ANY} -- predicates
   
   
   valid_position (pos: INTEGER) : BOOLEAN is
	 -- check if 'pos' is a valid index position
      require
	 not is_zombie
      do
	 result := pos > 0 and pos <= count
      end
   
   
   valid_position_for_insert(pos: INTEGER) : BOOLEAN is
	 -- check if 'pos' is a valid index position for the insert command
      require
	 not is_zombie
      do
	 result := pos > 0 and pos <= count+1
      end
   
   
feature {ANY} -- events 
   
   
   on_context_menu : ROUTINE[ANY, TUPLE[INTEGER, AGUI_POPUP_MENU]] is
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_context_menu_event)
      end
   
   set_on_context_menu (action: ROUTINE[ANY, TUPLE[INTEGER, AGUI_POPUP_MENU]]) is
      require
	 not is_zombie
      do
	 set_callback_agent(On_context_menu_event, action)
      end
   
   
   on_tooltip : ROUTINE[ANY, TUPLE[INTEGER,STRING]] is
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_tooltip_event)
      end
   
   set_on_tooltip (action: ROUTINE[ANY, TUPLE[INTEGER,STRING]]) is      
      require
	 not is_zombie
      do
	 set_callback_agent(On_tooltip_event, action)
      end
   
   
   on_command : ROUTINE[ANY, TUPLE[INTEGER]]  is
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_command_event)
      end
   
   set_on_command (action: ROUTINE[ANY, TUPLE[INTEGER]]) is
      require
	 not is_zombie
      do
	 set_callback_agent(On_command_event, action)
      end
   
   
   on_changed : ROUTINE[ANY, TUPLE[INTEGER]] is
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_change_event)
      end
   
   set_on_changed (action: ROUTINE[ANY, TUPLE[INTEGER]]) is
      require
	 not is_zombie
      do
	 set_callback_agent(On_change_event, action)
      end
   
   
feature {NONE} -- constructor
   
   
   make (p: AGUI_COMPOSITE;  n: STRING) is
	 -- create widget in parent 'p'
      require
	 p /= Void and n /= Void
      do
	 handle := efox_header__make(Current,p.handle, n.to_external, $temp_px)
	 efox__window_hash_table.put(Current, temp_px)
      end
   
   
   make_internal (hashkey: INTEGER; hnd: POINTER) is
      require
	 hnd /= default_pointer
      do
	 handle := hnd
	 efox__window_hash_table.put(Current, hashkey)
      end
   
   
feature {NONE} -- private definitions
   
   efox__callback_context_menu (idx: INTEGER; obj: ANY) : POINTER is
      local
	 a: ROUTINE[ANY, TUPLE[INTEGER, AGUI_POPUP_MENU]]
      do
	 a := on_context_menu
	 if a /= Void then
	    efox__shared_context_menu.clear
	    a.call([idx,efox__shared_context_menu])
	    result := efox__shared_context_menu.handle
	 end	 
      end
   
   efox__callback_query_tip (major,minor: INTEGER; obj: ANY) : POINTER is
      local
	 a: ROUTINE[ANY, TUPLE[INTEGER,STRING]]
      do
	 a := on_tooltip
	 if a /= Void then
	    efox__shared_tooltip_string.clear
	    a.call([major,efox__shared_tooltip_string])
	    result := efox__shared_tooltip_string.to_external
	 end
      end
   
   efox__callback_changed(what: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 a: ROUTINE[ANY, TUPLE[INTEGER]]
      do
	 a := on_changed
	 if a /= Void then
	    a.call([what])
	 end
      end
   
   efox__callback_command(idx: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 a: ROUTINE[ANY, TUPLE[INTEGER]]
      do
	 a := on_command
	 if a /= Void then
	    a.call([idx])
	 end
      end
   

feature {NONE} -- private C binding
   
   
   efox_header__set_horizontal_orientation(hnd: POINTER; fl: BOOLEAN) is
      external "C"
      end
   
   efox_header__horizontal_orientation(hnd: POINTER) : BOOLEAN is
      external "C"
      end

   efox_header__set_tracking(hnd: POINTER; fl: BOOLEAN) is
      external "C"
      end
   
   efox_header__tracking(hnd: POINTER) : BOOLEAN is
      external "C"
      end

   efox_header__set_button_mode(hnd: POINTER; fl: BOOLEAN) is
      external "C"
      end
   
   efox_header__button_mode(hnd: POINTER) : BOOLEAN is
      external "C"
      end

   efox_header__font(hnd: POINTER) : AGUI_FONT is
      external "C"
      end
   
   efox_header__set_font(hnd,fhnd: POINTER) is
      external "C"
      end
   
   efox_header__text_color(hnd: POINTER) : INTEGER is
      external "C"
      end	 
	 
   efox_header__set_text_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end
   
   efox_header__arrow_direction(hnd: POINTER; idx: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_header__set_arrow_direction (hnd: POINTER; idx,dir: INTEGER) is
      external "C"
      end
   
   efox_header__item_size(hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_header__set_item_size(hnd: POINTER; idx,ns: INTEGER) is
      external "C"
      end
   
   efox_header__item_offset(hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
      
   efox_header__item_at(hnd: POINTER; x: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_header__count(hnd: POINTER) : INTEGER is
      external "C"
      end
	 
   efox_header__clear(hnd: POINTER) is
      external "C"
      end
   
   efox_header__remove(hnd: POINTER; n: INTEGER) is
      external "C"
      end
      
   efox_header__insert(hnd: POINTER; pos: INTEGER; str,ic: POINTER; size: INTEGER) is
      external "C"
      end
   
   efox_header__item_icon(hnd: POINTER; pos: INTEGER) : AGUI_ICON is
      external "C"
      end
   
   efox_header__set_item_icon(hnd: POINTER; pos: INTEGER; ic: POINTER) is
      external "C"
      end
   
   efox_header__item_text(hnd: POINTER; pos: INTEGER) : POINTER is
      external "C"
      end
      
   efox_header__set_item_text(hnd: POINTER; pos: INTEGER; str: POINTER) is
      external "C"
      end   
   
   efox_header__make(this:like Current; hnd,n,keyptr: POINTER) : POINTER is
      external "C"
      end   
   
end
   
