class  AGUI_ICON_MANAGER
   
--TODO  application ressource DLL
--TODO  return 1x1 default icon
   
   
creation
   {AGUI_APPLICATION} make
   
   
feature {ANY}
   

   clear_path_list is
	 -- Removes all directories from the search list.
      do
	 efox_icon_manager__clear_path_list(handle)
      end


   add_path (filepath: STRING) is
	 -- adds a directory to the icon search list.
      require
	 filepath /= Void
	 filepath.count > 0
      do
	 efox_icon_manager__add_path(handle, filepath.to_external)
      end
      
   
   contains (name: STRING) : BOOLEAN is
	 -- Is there an already loaded icon with the given name ?
      require
	 name /= Void
	 name.count > 0
      do
	 result := dict.has(name)
      end
   
   
   find (name: STRING) : AGUI_ICON is
	 -- Look for an icon by 'name'. If already loaded it will return
	 -- the in memory copy. Otherwise it looks for an file with the
	 -- same name in all directories. 
	 -- If it can't find an icon in the directories, it returns a
	 -- default icon (a transparent 1x1 sized icon).  --TODO
      require
	 name /= Void
	 name.count > 0
      do
	 if dict.has(name) then
	    result := dict.at(name)
	 end
	 if result = Void then
	    register(name)
	    if last_register_status then
	       result := find(name)
	    else
	       --TODO default transparent item !!!
	    end
	 end
      end
   
   
   list_all_icons : ARRAY[AGUI_ICON] is
	 -- Returns an array with lower = 1 that contains all registered icons.
	 -- The array is newly created on every call.
      do
	 create result.with_capacity(dict.count,1)
	 dict.item_map_in(result)
      ensure
	 result /= Void
      end
   
   
   list_all_names : ARRAY[STRING] is
	 -- Returns an array with lower = 1 that contains the name of all registered icons.
	 -- The array and all strings are newly created on every call.
      local
	 i : INTEGER
      do
	 create result.with_capacity(dict.count,1)
	 dict.key_map_in(result)
	 from
	    i := result.lower
	 until
	    i > result.upper
	 loop
	    result.put(clone(result.item(i)),i)
	    i := i + 1
	 end
      ensure 
	 result /= Void
      end
   
   
   register (name: STRING) is
	 -- Looks in the icon paths for a file with the name of the 
	 -- icon. Extensions ".gif" , ".png" and ".bmp" are extended.
	 -- Sets 'last_register_status' to true if an icon is found.
      require
	 name /= Void
	 name.count > 0
      local
	 icon : AGUI_ICON
	 hnd : POINTER
	 n : STRING
      do
	 if not contains(name) then
	    last_register_status := false
	    hnd := efox_icon_manager__register(handle,name.to_external,$temp_width,$temp_height)
	    if hnd /= default_pointer then
	       create icon.make(hnd, temp_width, temp_height);
	       n := clone(name)
	       efox_icon_manager__attach_name_and_object(hnd, n, icon)
	       dict.put(icon,n)
	       last_register_status := true
	    end
	 end
      end
   
   
   last_register_status : BOOLEAN
	 --  success status of last register operation
   
   
   lookup (s: STRING) : AGUI_ICON is      
      require
	 s /= Void
      do
	 register(s)
	 result := find(s)
      end
   
feature {NONE}
    
   make is
	 -- Create the manager and put the current working directory in the
	 -- icon search path list.
      do
	 handle := efox_icon_manager__make(Current)
	 create dict.with_capacity(50)
	 create zombie_dict.with_capacity(50)
      end
   
   handle : POINTER
	 -- handle to the "efox_icon_manager" object
   
   temp_width, temp_height : INTEGER
	 -- need to pass two integers as reference arguments
   
   
   dict : DICTIONARY[AGUI_ICON,STRING]
	 -- Hashtable with all living registered icons
   
   zombie_dict : DICTIONARY[AGUI_ICON,STRING]
	 -- Hashtable with zombie icons that are still referenced 
	 -- from the C++ side of FOX.
   
   
   unregister (ic: AGUI_ICON; name: STRING; living: BOOLEAN) is
	 -- called when an icon is destroyed.
	 -- Remember when the icon is still referenced from the
	 -- C side it must be keept in the zombie dictionary
	 -- because we must protect it from the GC.
      require
	 name /= Void
	 ic /= Void
      do
	 dict.remove(name)
	 if living then
	    zombie_dict.put(ic,name)
	 end
      end
   
   unregister_zombie (name: STRING) is
      require
	 name /= Void
      do
	 zombie_dict.remove(name)
      end
   
   
feature {NONE} -- private C Binding
   
   
   efox_icon_manager__make(this:like Current) : POINTER is
      external "C"
      end
   
   efox_icon_manager__clear_path_list(hnd: POINTER) is
      external "C"
      end
   
   efox_icon_manager__add_path(hnd,str: POINTER) is
      external "C"
      end
   
   efox_icon_manager__register(hnd, str, wp, hp: POINTER) : POINTER is
      external "C"
      end
   
   efox_icon_manager__attach_name_and_object(hnd: POINTER; n: STRING; i: AGUI_ICON) is
      external "C"
      end

end
   
