deferred class  AGUI_LIST_BASE


inherit
   AGUI_CONTROL
      redefine efox__callback_changed, efox__callback_command
      end
   

feature {ANY} -- dummy declaration for typesafety
   
   
   item_type : AGUI_LIST_BASE_ITEM
	 -- This is redefined in subclasses to concrete item types.
   
   
feature {ANY}  -- appearence properties

   
   font : AGUI_FONT is
	 -- Default font for items
      require
         not is_zombie
      do
         result := efox_list_base__font(handle)
      ensure
         result /= Void
      end   


   set_font (f: AGUI_FONT) is
	 -- Set default font for items
      require
         not is_zombie
         f /= Void
         not f.is_zombie
      do
         efox_list_base__set_font(handle, f.handle)
      end
   
   
   text_color : AGUI_COLOR is
	 -- Default text color for items
      require
         not is_zombie
      do
         result.from_integer(efox_list_base__text_color(handle))
      end

   
   set_text_color (col: AGUI_COLOR) is
	 -- Set default text color for items
      require
         not is_zombie
      do
         efox_list_base__set_text_color(handle, col.to_integer)
      end
   
   
   selected_background_color : AGUI_COLOR is
	 -- Default background color for selected items.
      require
         not is_zombie
      do
         result.from_integer(efox_list_base__selected_background_color(handle))
      end
   

   set_selected_background_color (col: AGUI_COLOR) is
	 -- Default background color for selected items.
      require
         not is_zombie
      do
         efox_list_base__set_selected_background_color(handle,col.to_integer)
      end
   
   
   selected_text_color : AGUI_COLOR is
	 -- Default color for selected items.
      require
         not is_zombie
      do
         result.from_integer(efox_list_base__selected_text_color(handle))
      end
   

   set_selected_text_color (col: AGUI_COLOR) is
	 -- Default color for selected items.
      require
         not is_zombie
      do
         efox_list_base__set_selected_text_color(handle,col.to_integer)
      end
      
   
feature  {ANY}  -- current item
      
   
   current_item : like item_type is
	 -- Return current item - if any - or Void.
      require
	 not is_zombie
      do
	 result := efox_list_base__current_item(handle)
      end
   
   
   set_current_item (i: like item_type) is
	 -- Change current item.
      require
	 not is_zombie
	 i /= Void
      do
	 efox_list_base__set_current_item(handle, i.handle)
      end
   
   
   cursor_item : like item_type is
	 -- Return item under cursor - if any - or Void.
      require
	 not is_zombie
      do
	 result := efox_list_base__cursor_item(handle)
      end
   
   
   
feature {ANY}  -- misc
   
   
   clear is
	 -- remove all items from list
      require
	 not is_zombie
      do
	 efox_list_base__clear(handle)
	 item_dict.clear
      end
   
   
   count : INTEGER is
	 -- return number of items
      require
	 not is_zombie
      do
	 result := efox_list_base__count (handle)
      end
   
      
   item(idx: INTEGER) : like item_type is
	 -- return (first) root item
      require
	 not is_zombie
	 idx.in_range(1,count)
      do
	 result := efox_list_base__item (handle,idx)
      end
   
   
feature {ANY}  -- misc
   
   
   append_in (c: COLLECTION[like item_type]) is
	 -- Travers the list and add all items to the collection.
      require
	 c /= Void
	 not is_zombie
      local
	 i,n : INTEGER
      do
	 from
	    i := 1
	    n := count
	 until
	    i > n
	 loop
	    c.add_last(item(i))
	    i := i + 1
	 end
      end
   
   
   do_all (action: ROUTINE[ANY,TUPLE[AGUI_LIST_BASE_ITEM]]) is
      local
	 i,n: INTEGER
      do
	 from
	    i := 1
	    n := count
	 until
	    i > n
	 loop
	    action.call([item(i)])
	    i := i + 1
	 end
      end
      
   
feature {ANY}  -- sorting
   

   sort is
	 -- Sort all items.
      require
         not is_zombie
      do
	 efox_list_base__sort(handle)
      end


   sort_setup_agent : ROUTINE[ANY,TUPLE[like item_type,STRING]]
	 -- This agent is called before a sort is done. It is called
	 -- for every item in the list and you must set the string
	 -- to the sort key. This key and not the 'text' from the
	 -- item is used in the compare operations during the sort.
	 -- The string comparision is case insensitive.
   
   
   set_sort_setup_agent (action: ROUTINE[ANY,TUPLE[like item_type,STRING]]) is
	 -- Sets up an agent which defines an individual sort order.
      require
         not is_zombie
      do
	 --TODO	 sort_agent := action
      end
   
   
   set_ascending_case_insensitive_sorter is
	 -- Set predefined sorter method to sort items based on  
	 -- 'text in ascending order and case insensitive.
      require
         not is_zombie
      do
	 efox_list_base__set_ascending_case_insensitive_sorter(handle)
      end
   
   
   set_ascending_sorter is
	 -- Set predefined sorter method to sort items based on  
	 -- 'text in ascending order. This is case sensitive.
      require
         not is_zombie
      do
	 efox_list_base__set_ascending_sorter(handle)
      end
   
   
   set_descending_case_insensitive_sorter is
	 -- Set predefined sorter method to sort items based on  
	 -- 'text in descending order and case insensitive.
      require
         not is_zombie
      do
	 efox_list_base__set_descending_case_insensitive_sorter(handle)
      end
      
   set_descending_sorter is
	 -- Set predefined sorter method to sort items based on  
	 -- 'text in descending order. This is case sensitive.
      require
         not is_zombie
      do
	 efox_list_base__set_descending_sorter(handle)
      end
   
   
feature {ANY} -- events
   
   
   on_command : ROUTINE[ANY,TUPLE[AGUI_LIST_BASE_ITEM]] is
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_command_event)
      end
   
   
   set_on_command (action: ROUTINE[ANY,TUPLE[AGUI_LIST_BASE_ITEM]]) is
      require
         not is_zombie
      do
	 set_callback_agent(On_command_event, action)
      end
   
   
   on_changed : ROUTINE[ANY,TUPLE[AGUI_LIST_BASE_ITEM]] is
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_change_event)
      end
   
   
   set_on_changed (action: ROUTINE[ANY,TUPLE[AGUI_LIST_BASE_ITEM]]) is
      require
         not is_zombie
      do
	 set_callback_agent(On_change_event, action)
      end
   
   
feature {AGUI_LIST_BASE,AGUI_LIST_BASE_ITEM} -- private implementation
   
   
   item_dict : DICTIONARY[like item_type, INTEGER]
	 -- Dictionary with all items. We must hold the
	 -- items to protect them from garbage collection.
   
   
feature {NONE} -- private implementation


   efox__callback_changed(what: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 a: ROUTINE[ANY,TUPLE[AGUI_LIST_BASE_ITEM]]
	 i : AGUI_LIST_BASE_ITEM
      do
	 a ?= find_callback_agent(On_change_event)
	 if a /= Void then
	    i ?= obj
	    a.call([i])
	 end
      end
   
   efox__callback_command(idx: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 a: ROUTINE[ANY,TUPLE[AGUI_LIST_BASE_ITEM]]
	 i : AGUI_LIST_BASE_ITEM
      do
	 a ?= find_callback_agent(On_command_event)
	 if a /= Void then
	    i ?= obj
	    a.call([i])
	 end
      end
   
   efox_list_base__set_selected_text_color (hnd: POINTER; col: INTEGER) is
      external "C"
      end

   efox_list_base__selected_text_color (hnd: POINTER) : INTEGER is
      external "C"
      end

   efox_list_base__set_selected_background_color (hnd: POINTER; col: INTEGER) is
      external "C"
      end

   efox_list_base__selected_background_color (hnd: POINTER) : INTEGER is
      external "C"
      end

   efox_list_base__set_text_color (hnd: POINTER; col: INTEGER) is
      external "C"
      end

   efox_list_base__text_color (hnd: POINTER) : INTEGER is
      external "C"
      end

   efox_list_base__set_font(hnd,fhnd: POINTER) is
      external "C"
      end

   efox_list_base__font(hnd: POINTER) : AGUI_FONT is
      external "C"
      end
   
   efox_list_base__current_item (hnd: POINTER) : like item_type is
      external "C"
      end

   efox_list_base__set_current_item(hnd,itemhnd: POINTER) is
      external "C"
      end

   efox_list_base__cursor_item(hnd: POINTER) : like item_type is
      external "C"
      end

   efox_list_base__clear (hnd: POINTER) is
      external "C"
      end

   efox_list_base__count (hnd: POINTER) : INTEGER is
      external "C"
      end

   efox_list_base__item(hnd: POINTER; idx: INTEGER) : like item_type is
      external "C"
      end
   
   efox_list_base__sort(hnd: POINTER) is
      external "C"
      end
   
   efox_list_base__set_ascending_sorter(hnd: POINTER) is
      external "C"
      end
   
   efox_list_base__set_descending_sorter(hnd: POINTER) is
      external "C"
      end
      
   efox_list_base__set_ascending_case_insensitive_sorter(hnd: POINTER) is
      external "C"
      end
   
   efox_list_base__set_descending_case_insensitive_sorter(hnd: POINTER) is
      external "C"
      end
      
end
