deferred class  AGUI_LIST_BASE_ITEM
   

feature {ANY}  -- public attributes
   
   
   text : STRING is
	 -- get a new string with text of this item
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_list_base_item__text(handle,list_kind))
      ensure
	 text /= Void
      end
   
   
   set_text (s: STRING) is
	 -- sets the text for this item
      require
	 not is_zombie
	 s /= Void
      do
	 efox_list_base_item__set_text(handle,list_kind,s.to_external)
      end
   

   icon : AGUI_ICON is
	 -- get the open icon or 'Void'
      require
	 not is_zombie
      do
	 result := efox_list_base_item__icon(handle,list_kind)
      end
   
   
   set_icon (ic: AGUI_ICON) is
	 -- sets the open icon or removes the current if 'ic = Void'
      require
	 not is_zombie
	 ic /= Void implies not ic.is_zombie
      do
	 if ic /= Void then
	    efox_list_base_item__set_icon(handle,list_kind, ic.handle)
	 else
	    efox_list_base_item__set_icon(handle,list_kind, default_pointer)
	 end
      end
   

   data : ANY
	 -- corresponding data item
   
   
   set_data (d: ANY) is
	 -- set the data item
      require
	 not is_zombie
      do
	 data := d
      end
   
   
feature {ANY}  -- queries
   
   
   is_zombie : BOOLEAN is
	 -- test if this item is already destroyed, because it is
	 -- explicitly removed or the list is destroyed
      do
	 result := handle.is_null
      end
   
   
   list_window : AGUI_LIST_BASE is
	 -- get the list that contains this item
      require
	 not is_zombie
      do
	 result := efox_list_base_item__list_window(handle,list_kind)
      end
   
   
   is_current : BOOLEAN is
	 -- test if this is the current item in the list
      require	 
	 not is_zombie
      do
	 result := efox_list_base_item__is_current(handle, list_kind)
      end
   
   index: INTEGER is
      require
	 not is_zombie
      do
	 result := efox_list_base_item__index(handle, list_kind)
      end
   
   
feature {ANY}  -- commands


   move_before (reference_item: like Current) is
	 -- moves this item before 'reference_item'
      require
	 not is_zombie
	 reference_item /= Void
	 not reference_item.is_zombie
	 list_window = reference_item.list_window
      do
	 efox_list_base_item__move_before(handle, list_kind, reference_item.handle)
      end
   
   
   move_after (reference_item: like Current) is
	 -- moves this item after 'reference_item'
      require
	 not is_zombie
	 reference_item /= Void
	 not reference_item.is_zombie
	 list_window = reference_item.list_window
      do
	 efox_list_base_item__move_after(handle, list_kind, reference_item.handle)
      end
  
      
   destroy is
	 -- delete item from list
      require
	 not is_zombie
      do
	 efox_list_base_item__destroy(handle, list_kind)
      ensure
	 is_zombie
      end


feature {AGUI_LIST_BASE_ITEM, AGUI_LIST_BASE} -- private implementation


   handle : POINTER
	 -- pointer to FXListItem or FXIconItem
      
   list_kind : INTEGER
	 -- for what list is this item a member 
   

feature {NONE} -- private C binding


   efox_list_base_item__set_text(hnd: POINTER; lkind: INTEGER; str: POINTER) is
      external "C"
      end

   efox_list_base_item__text(hnd: POINTER; lkind: INTEGER) : POINTER is
      external "C"
      end

   efox_list_base_item__set_icon(hnd: POINTER; lkind: INTEGER;  ic: POINTER) is
      external "C"
      end

   efox_list_base_item__icon(hnd: POINTER; lkind: INTEGER) : AGUI_ICON is
      external "C"
      end

   efox_list_base_item__enabled(hnd: POINTER; lkind: INTEGER) : BOOLEAN is
      external "C"
      end

   efox_list_base_item__set_enabled(hnd: POINTER; lkind: INTEGER;  flag: BOOLEAN) is
      external "C"
      end

   efox_list_base_item__list_window(hnd: POINTER; lkind: INTEGER) : AGUI_LIST_BASE is
      external "C"
      end

   efox_list_base_item__is_current(hnd: POINTER; lkind: INTEGER) : BOOLEAN is
      external "C"
      end

   efox_list_base_item__move_before(hnd: POINTER; lkind: INTEGER;  refhnd: POINTER) is
      external "C"
      end

   efox_list_base_item__move_after(hnd: POINTER; lkind: INTEGER;  refhnd: POINTER) is
      external "C"
      end

   efox_list_base_item__destroy(hnd: POINTER; lkind: INTEGER) is
      external "C"
      end
   
   efox_list_base_item__index(hnd: POINTER; lkind: INTEGER) : INTEGER is
      external "C"
      end
   
end 
