deferred class AGUI_MENU_BASE
   
-- This is the superclass for "normal" menus and for menubars
   

feature {ANY}
   
   
   count : INTEGER is
      require
	 not is_zombie
      do
	 result := menu_items.count
      ensure
	 count >= 0
      end
   
   
   item (index: INTEGER) : AGUI_MENU_ITEM is
      require	 
	 not is_zombie
	 index.in_range(1,count)
      do
	 result := menu_items.item(index)
      ensure
	 result /= Void
	 not result.is_zombie
      end
      
   
   move (old_pos, new_pos : INTEGER) is
	 -- moves an item to a new position
      require
	 not is_zombie
	 new_pos.in_range(1,count)
	 old_pos.in_range(1,count)
      local
	 elt: AGUI_MENU_ITEM
      do
	 if old_pos /= new_pos then
	    efox_menu_base__move(menu_handle, old_pos, new_pos)
	    elt := menu_items.item(old_pos)
	    menu_items.remove(old_pos)
	    menu_items.add(elt, new_pos)
	 end
      ensure
	 count = old count
	 old item(old_pos) = item(new_pos)
      end
   
   
   clear is
	 -- destroys all items in menu
      require
	 not is_zombie
      do
	 efox_menu_base__clear(menu_handle)
      ensure
	 count = 0
      end
   
   
   contains (test: AGUI_MENU_ITEM) : BOOLEAN is
      require
	 not is_zombie
	 not test.is_zombie
      local
	 idx: INTEGER
      do
	 idx := menu_items.fast_index_of(test)
	 result := menu_items.valid_index(idx)
      end
   
   
   is_zombie : BOOLEAN is
      deferred
      end
   
   
feature {AGUI_MENU_ITEM,AGUI_MENU_BASE} -- private implementation
   
   
   menu_items: ARRAY[AGUI_MENU_ITEM]
	 -- This array protects the items from the GC.
   
   menu_handle : POINTER
	 -- handle to the FXMenuPane or FXMenuBar object

   
feature {NONE} -- private C binding
   
   
   efox_menu_base__move(hnd: POINTER; old_pos, new_pos: INTEGER) is
      external "C"
      end
   
   efox_menu_base__clear(hnd: POINTER) is
      external "C"
      end
   
end
   
