deferred class AGUI_MENU_TEXT_ITEM
   
   
inherit
   AGUI_MENU_ITEM
      end
   
   
feature {ANY} -- public properties
   
   
   enabled : BOOLEAN is
	 -- Test if item is enabled.
      require
	 not is_zombie
      do
	 result := efox_menu_text_item__enabled(handle)
      end
      
   
   set_enabled (flag: BOOLEAN) is
	 -- Set the enabled state of the item.
      require
	 not is_zombie
      do
	 efox_menu_text_item__set_enabled(handle, flag)
      ensure
	 enabled = flag
      end
   
   
   text : STRING is
      -- Get new string with text of the menu item.
      do
	 create result.from_external_copy(efox_menu_text_item__text(handle))
      ensure
	 result /= Void
      end 
   
   
   set_text (str: STRING) is
	 -- Sets the text of the menu item (no tab character is allowed).
	 -- If the text includes an %T character this separate the 
	 -- string in two parts, a pure text and an accelerator.
      require
	 not is_zombie	 
      do
	 efox_menu_text_item__set_text(handle,str.to_external)
      end
   
   
   icon : AGUI_ICON is
	 -- Returns the icon.
      require
	 not is_zombie
      do
	 result := efox_menu_text_item__icon(handle)
      end
      
   
   set_icon (new_icon: AGUI_ICON) is
	 -- Sets the icon for the item.
      require
	 not is_zombie
	 new_icon /= Void implies not new_icon.is_zombie	 
      local
	 p : POINTER
      do
	 if new_icon /= Void then p := new_icon.handle end
	 efox_menu_text_item__set_icon(handle, p)
      ensure
	 icon = new_icon
      end
   
   
   tooltip_text : STRING is
	 -- Get new string with the tooltip text.
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_menu_text_item__tooltip_text(handle))
      ensure
	 result /= Void
      end
      
   
   set_tooltip_text (str: STRING) is
	 -- Sets the tooltip text (no tab characters are allowed).
      require
	 not is_zombie
	 str /= Void
      do	 
	 efox_menu_text_item__set_tooltip_text(handle, str.to_external)
      end
   
   
   help_text : STRING is
	 -- Get new string with the help text.
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_menu_text_item__help_text(handle))
      ensure
	 result /= Void
      end
   
   
   set_help_text (str: STRING) is
	 -- Sets the help text (no tab/nl characters are allowed).
      require
	 not is_zombie
	 str /= Void
      do
	 efox_menu_text_item__set_help_text(handle, str.to_external)
      end
   
   
feature {ANY}  --  data attachment
   
   
   data : ANY   
	 -- Data value attached to this window
   
   
   set_data (new_value: ANY) is
	 -- Set new data value.
      require
	 not is_zombie
      do
	 data := new_value
      ensure
	 data = new_value
      end
   

feature {NONE} -- private C binding


   efox__callback_invoke is
      do
      end	 
   
   
   efox_menu_text_item__enabled(hnd: POINTER) : BOOLEAN is
      external "C"
      end      
   
   efox_menu_text_item__set_enabled (hnd: POINTER; flag: BOOLEAN) is
      external "C"
      end   
   
   efox_menu_text_item__help_text(hnd: POINTER) : POINTER is
      external "C"
      end
   
   efox_menu_text_item__set_help_text (hnd,str: POINTER) is
      external "C"
      end
   
   efox_menu_text_item__tooltip_text(hnd: POINTER) : POINTER is
      external "C"
      end
   
   efox_menu_text_item__set_tooltip_text (hnd, str: POINTER) is
      external "C"
      end
      
   efox_menu_text_item__text(hnd: POINTER) : POINTER is
      external "C"
      end
      
   efox_menu_text_item__set_text (hnd, str: POINTER) is
      external "C"
      end
   
   efox_menu_text_item__icon(hnd: POINTER) : AGUI_ICON is
      external "C"
      end
      
   efox_menu_text_item__set_icon (hnd, ihnd: POINTER) is
      external "C"
      end
   
      
end
   
