class  AGUI_OPTION_BUTTON
   
inherit  
   AGUI_LABEL_BASE
      redefine
	 efox__callback_command
      end
   
   
creation
   {ANY} make
   
   
feature {NONE} -- constructor
   
   
   make (p: AGUI_COMPOSITE;  n: STRING) is
      do
	 handle := efox_option_button__make(Current,p.handle,n.to_external,$temp_px)
	 efox__window_hash_table.put(Current, temp_px)
      end
   

feature {ANY} -- event handler   
   
   
   on_command: PROCEDURE[ANY,TUPLE[INTEGER]] is
	 -- Agent that is called when the user selects an option
	 -- in the button. The argument is index of this options. 
	 -- The first option has the index 1.
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_command_event)
      end
   
   
   set_on_command (action: PROCEDURE[ANY,TUPLE[INTEGER]]) is
	 -- Sets the agent for the click action.
      require
	 not is_zombie
      do
	 set_callback_agent(On_command_event, action)
      end
   
   
feature {ANY}
   
   
   current_option : INTEGER is
      require
	 not is_zombie
      do
	 result := efox_option_button__current_option(handle)
      ensure
	 result = 0 or else valid_position(result)
      end
   
   
   set_current_option (index: INTEGER) is
      require
	 not is_zombie
	 valid_position(index)
      do
	 efox_option_button__set_current_option(handle,index)
      ensure
	 current_option = index
      end
   
   
   count : INTEGER is
      require
	 not is_zombie
      do
	 result := efox_option_button__count(handle)
      end
   
   
   valid_position (no: INTEGER) : BOOLEAN is
      require
	 not is_zombie
      do
	 result := efox_option_button__valid_position(handle,no)
      end
   
   
   valid_position_for_insert (no: INTEGER) : BOOLEAN is
      require
	 not is_zombie
      do
	 result := efox_option_button__valid_position_for_insert(handle,no)
      end
   
   
   clear is
      require
	 not is_zombie
      do
	 efox_option_button__clear(handle)
      ensure
	 count = 0
      end   
   
   
   append_option (str: STRING; ic: AGUI_ICON) is
      require
	 not is_zombie
	 ic /= Void implies not ic.is_zombie
	 str /= Void
      local
	 p : POINTER
      do
	 if ic /= Void then p := ic.handle end
	 efox_option_button__insert_option(handle, count+1, str.to_external, p)
      ensure
	 old count + 1 = count
      end  
   
   
   remove_option (index: INTEGER) is
      require
	 not is_zombie
	 valid_position(index)
      do
	 efox_option_button__remove_option(handle,index)
      ensure
	 old count - 1 = count
      end
   
   
   insert_option(index: INTEGER; str: STRING; ic: AGUI_ICON) is
      require
	 not is_zombie
	 str /= Void
	 ic /= Void implies not ic.is_zombie
	 valid_position_for_insert(index)
      local
	 p : POINTER
      do
	 if ic /= Void then p := ic.handle end
	 efox_option_button__insert_option(handle, index, str.to_external, p)
      ensure
	 old count + 1 = count
      end
   
   
   option_text (index: INTEGER) : STRING is
      require
	 not is_zombie
	 valid_position(index)
      do
	 create result.from_external_copy(efox_option_button__option_text(handle,index))
      ensure
	 result /= Void
      end
   
   
   set_option_text (index: INTEGER; str: STRING) is
	 -- Please not that you can't ensure that "option_text.is_equal(str)" because
	 -- you can add a help text to the option with a "%T" separated text part.
      require
	 not is_zombie
	 str /= Void
	 valid_position(index)
      do
	 efox_option_button__set_option_text(handle,index,str.to_external)
      end
   
   
   option_icon (index: INTEGER) : AGUI_ICON is
      require
	 not is_zombie
	 valid_position(index)
      do
	 result := efox_option_button__option_icon(handle, index)
      end   
   
   
   set_option_icon (index: INTEGER; ic: AGUI_ICON) is 
      require
	 not is_zombie
	 ic /= Void implies not ic.is_zombie
	 valid_position(index)
      local
	 p : POINTER
      do
	 if ic /= Void then p := ic.handle end
	 efox_option_button__set_option_icon(handle, index, p)
      ensure
	 option_icon(index) = ic
      end
   
   
feature {NONE} -- private C binding
   
   
   efox__callback_command(idx: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 a: PROCEDURE[ANY,TUPLE[INTEGER]]
      do
	 a ?= find_callback_agent(On_command_event)
	 if a /= Void then
	    a.call([idx])
	 end
      end
   
   efox_option_button__make(this:like Current; hnd,n,keyptr: POINTER) : POINTER is
      external "C"
      end      
   
   efox_option_button__set_current_option(hnd: POINTER; idx: INTEGER) is
      external "C"
      end
   
   efox_option_button__current_option(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_option_button__count(hnd: POINTER) : INTEGER is	 
      external "C"
      end
   
   efox_option_button__clear(hnd: POINTER) is
      external "C"
      end
   
   efox_option_button__append_option(hnd: POINTER; str,ic: POINTER) is
      external "C"
      end
	 
   efox_option_button__insert_option(hnd: POINTER; idx: INTEGER; str,ic: POINTER) is
      external "C"
      end
	 
   efox_option_button__remove_option(hnd: POINTER; idx: INTEGER) is
      external "C"
      end
   
   efox_option_button__option_text(hnd: POINTER; idx: INTEGER) : POINTER is
      external "C"
      end
	 
   efox_option_button__set_option_text(hnd: POINTER; idx: INTEGER; str: POINTER) is
      external "C"
      end
   
   efox_option_button__option_icon(hnd: POINTER; idx: INTEGER) : AGUI_ICON is
      external "C"
      end
   
   efox_option_button__set_option_icon(hnd: POINTER; idx: INTEGER; ic: POINTER) is
      external "C"
      end
   
   efox_option_button__valid_position_for_insert(hnd: POINTER; no: INTEGER) : BOOLEAN is
      external "C"
      end
   
   efox_option_button__valid_position(hnd: POINTER; no: INTEGER) : BOOLEAN is
      external "C"
      end
   
end

   
