deferred class  AGUI_PREDEFINED_DIALOG
   
   
-- IMPROVE  query position and make it possible to restore the 
--          dialog at this position.
   
   
inherit
   AGUI_GLOBALS
      end
   AGUI_INTERNAL_GC_PROTECTOR
      end
   

feature {ANY} -- show and run dialog
   
   
   Placement_default   : INTEGER is 1
   Placement_visible   : INTEGER is 2
   Placement_cursor    : INTEGER is 3
   Placement_owner     : INTEGER is 4
   Placement_screen    : INTEGER is 5
   Placement_maximized : INTEGER is 6
   Placement_fixed     : INTEGER is 7
	 -- This values have the same meaning as the values defined
	 -- in class  AGUI_TOPLEVEL_DIALOG
   
   
   position (x,y,w,h: INTEGER) is
	 -- Position the window relative to the root window.
      require
	 not is_zombie
      do
	 efox_predefined_dialog__position(handle, x,y,w,h)
      end
   
   
   show (place: INTEGER) is
	 -- Show dialog but don't enter a modal loop. You must install
	 -- a 'on_finish' agent to get information when then user
	 -- selects "cancel" or "okay".
      require
	 not is_zombie
      do
	 efox_predefined_dialog__show(handle, place)
      end
   
   
   show_modal (place: INTEGER) is
	 -- Show dialog and enter modal loop. Return when user
	 -- selected "cancel" or "okay".
	 -- If the user pressed "cancel" the variable 'last_status'
	 -- is set to 'false' otherwise it is 'true'.
	 -- Before the methods returns it calls the 'on_finish' agent.
      require
	 not is_zombie
      local
	 n: INTEGER
      do
	 n := efox_predefined_dialog__show_modal(handle, place)
	 last_status := n /= 0;
      end
   
   
   is_visible : BOOLEAN is
	 -- Test if dialog is shown.
      require
	 not is_zombie
      do
	 result := efox_predefined_dialog__is_visible(handle)
      end
   
   
   hide is
	 -- Hide the dialog. If the window is modal this does also
	 -- break the modal loop in the same way as the user pressed
	 -- 'cancel'.
      require
	 not is_zombie
      do
	 efox_predefined_dialog__hide(handle);
      end
   
   
   last_status : BOOLEAN
	 -- 'false' if user canceled the dialog.
   
   
feature {ANY} -- properties
   
   
   title : STRING is
	 -- Get title of dialog.
      do
	 create result.from_external_copy(efox_predefined_dialog__title(handle))
      end
   
   
   set_title (str: STRING) is
	 -- Change title of dialog
      do
	 efox_predefined_dialog__set_title(handle,str.to_external)
      end
      

feature {ANY} -- destruction
   
   
   destroy is
	 -- Destroy the dialog. Remember that you can't destroy a window
	 -- if it is used in a modal loop. You can destroy the window
	 -- safely after the call to 'show_modal' or when processing the
	 -- 'on_finish' agent.
      require
	 not is_zombie
      do
	 efox_predefined_dialog__destroy(handle)
      end
   
   
   is_zombie : BOOLEAN is
	 -- Test if dialog is already destroyed.
      do
	 result := handle = default_pointer
      end
   
      
feature {ANY} -- event handling
   
   
   on_finish : ROUTINE[ANY,TUPLE]
	 -- Agent that is invoked when the user pressed "Okay" or "Cancel"
	 -- in the non-modal dialog. At the moment of the call the window
	 -- is already hidden and can be destroyed safely.
   
   
   set_on_finish (action: ROUTINE[ANY,TUPLE]) is
	 -- Set the 'on_finish' agent.
      do
	 on_finish := action
      end
   
   
feature {NONE} -- private C binding
   
   
   handle: POINTER
	 -- handle to the FOX dialog
   
   temp_key: INTEGER
	 -- temporary storage for gc protector key
   
   
   efox_predefined_dialog__show(hnd: POINTER; place: INTEGER) is
      external "C"
      end      
   
   efox_predefined_dialog__show_modal(hnd: POINTER; place: INTEGER) : INTEGER is
      external "C"
      end      
      	 
   efox_predefined_dialog__position(hnd: POINTER; x,y,w,h: INTEGER) is
      external "C"
      end      
		 
   efox_predefined_dialog__is_visible(p: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_predefined_dialog__hide(p: POINTER) is
      external "C"
      end
   
   efox_predefined_dialog__set_title(h,p: POINTER) is
      external "C"
      end      
      	 
   efox_predefined_dialog__title(hnd: POINTER) : POINTER is
      external "C"
      end      
   
   efox_predefined_dialog__destroy(hnd: POINTER) is
      external "C"
      end
     
   efox__callback_destroy(key: INTEGER) is
      do
	 check efox__window_hash_table.has(temp_key) end
	 efox__window_hash_table.remove(temp_key)
	 handle := default_pointer
      end
   
   efox__callback_finish(res: INTEGER) is
      do
	 last_status := res /= 0;
	 if on_finish /= Void then
	    on_finish.call([])
	 end
      end
   
end
