class  AGUI_PROGRESS_BAR
   
   
inherit
   AGUI_CONTROL
      end
   AGUI_FRAME_MIXIN
      redefine mixin_window
      end
   AGUI_PACKER_MIXIN
      redefine mixin_window
      end
   
creation
   {ANY} make
   
   
feature {ANY}  -- properties
   
   
   progress : INTEGER is
	 -- Return number of steps already done.
      require
	 not is_zombie
      do
	 result := efox_progress_bar__progress(handle)
      end
   
   set_progress (n: INTEGER) is
	 -- Set number of steps already done.
      require
	 not is_zombie
      do
	efox_progress_bar__set_progress(handle,n)
      end
   
   
   total : INTEGER is
	 -- Return total number of steps to go.
      require
	 not is_zombie
      do
	 result := efox_progress_bar__total(handle)
      end
   
   set_total (n: INTEGER) is
	 -- Set total number of steps to go.
      require
	 not is_zombie
      do
	efox_progress_bar__set_total(handle,n)
      end
   
   
feature {ANY} -- visual apperarence
   
   
   bar_width : INTEGER is
      require
	 not is_zombie
      do
	 result := efox_progress_bar__bar_width(handle)
      end
   
   set_bar_width (value : INTEGER) is
      require
	 not is_zombie
      do
	 efox_progress_bar__set_bar_width(handle,value)
      end
   
   
   bar_background_color : AGUI_COLOR is
	 -- Background color of progress bar.
      require
	 not is_zombie
      do
	 result.from_integer(efox_progress_bar__bar_background_color(handle))
      end
   
   set_bar_background_color(color: AGUI_COLOR) is
	 -- Set background color of progress bar.
      require
	 not is_zombie
      do
	 efox_progress_bar__set_bar_background_color(handle, color.to_integer)
      end
   
   
   bar_color : AGUI_COLOR is
	 -- Color of progress bar.
      require
	 not is_zombie
      do
	 result.from_integer(efox_progress_bar__bar_color(handle))
      end
   
   set_bar_color(color: AGUI_COLOR) is
	 -- Set color of progress bar.
      require
	 not is_zombie
      do
	 efox_progress_bar__set_bar_color(handle,color.to_integer)
      end
   
   
   text_color : AGUI_COLOR is
	 -- Color for writing the percentage number.
      require
	 not is_zombie
      do
	 result.from_integer(efox_progress_bar__text_color(handle))
      end
   
   set_text_color(color: AGUI_COLOR) is      
	 -- Set color for writing the percentage number.
      require
	 not is_zombie
      do
	 efox_progress_bar__set_text_color(handle,color.to_integer)
      end
   
   
   text_alternate_color : AGUI_COLOR is
      require
	 not is_zombie
      do
	 result.from_integer(efox_progress_bar__text_alternate_color(handle))
      end
   
   set_text_alternate_color(color: AGUI_COLOR) is
      require
	 not is_zombie
      do
	 efox_progress_bar__set_text_alternate_color(handle,color.to_integer)
      end
   
   
   font : AGUI_FONT is
	 -- Font for writing the percentage number.
      require
	 not is_zombie
      do
	 result := efox_progress_bar__font(handle)
      end
   
   set_font(fnt: AGUI_FONT) is
	 -- Set font for writing the percentage number.
      require
	 not is_zombie
	 fnt /= Void and not fnt.is_zombie
      do
	 efox_progress_bar__set_font(handle, fnt.handle)
      end
   
      
   vertical_orientation : BOOLEAN is
	 -- Test if progress is shown vertical or horizonatal.
      require
	 not is_zombie
      do
	 result := efox_progress_bar__vertical_orientation(handle)
      end
   
   set_vertical_orientation(value : BOOLEAN) is
	 -- Set vertical or horizontal orientation of progress.
      require
	 not is_zombie
      do
	 efox_progress_bar__set_vertical_orientation(handle,value)
      end
   
   
   visible_number : BOOLEAN is
	 -- Test if the percent number is visible.
      require
	 not is_zombie
      do
	 result := efox_progress_bar__visible_number(handle)
      end
   
   set_visible_number(value : BOOLEAN) is
	 -- Set if percent number should be visible or not.
      require
	 not is_zombie
      do
	 efox_progress_bar__set_visible_number(handle,value)
      end
   
   
feature {ANY} -- commands
   
   
   increment (n: INTEGER) is
	 -- Increment the progress by n values.
      require
	 not is_zombie
	 n > 0
      do
	 efox_progress_bar__increment(handle, n)
      end
   
   
feature {ANY}  -- mixin configure
   
   
   mixin_window : AGUI_WINDOW is
	 -- redefine the mixin window references to this window
      do
	 result := Current
      end
   
   
feature {NONE} -- constructor
   
   
   make (p: AGUI_COMPOSITE;  n: STRING) is
      require
	 p /= Void and n /= Void
      do
	 handle := efox_progress_bar__make(Current,p.handle, n.to_external, $temp_px)
	 efox__window_hash_table.put(Current, temp_px)
      end
   
   
feature {NONE} -- private C binding
   
   
   efox_progress_bar__make(this:like Current; hnd,n,key: POINTER) : POINTER is
      external "C"
      end   
   
   efox_progress_bar__increment(hnd: POINTER; n: INTEGER) is
      external "C"
      end
      
   efox_progress_bar__progress(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_progress_bar__set_progress(hnd: POINTER; n: INTEGER) is
      external "C"
      end
	   
   efox_progress_bar__total(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_progress_bar__set_total(hnd: POINTER; n: INTEGER) is
      external "C"
      end
      
   efox_progress_bar__bar_width(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_progress_bar__set_bar_width(hnd: POINTER; val: INTEGER) is
      external "C"
      end
      
   efox_progress_bar__bar_background_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_progress_bar__set_bar_background_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end   
   
   efox_progress_bar__bar_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_progress_bar__set_bar_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end
      
   efox_progress_bar__text_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_progress_bar__set_text_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end
      
   efox_progress_bar__text_alternate_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_progress_bar__set_text_alternate_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end   
   
   efox_progress_bar__font(hnd: POINTER) : AGUI_FONT is
      external "C"
      end
   
   efox_progress_bar__set_font(hnd: POINTER; fhnd: POINTER) is
      external "C"
      end
      
   efox_progress_bar__vertical_orientation(hnd: POINTER) : BOOLEAN is
      external "C"
      end   
   
   efox_progress_bar__set_vertical_orientation(hnd: POINTER; n: BOOLEAN) is
      external "C"
      end
      
   efox_progress_bar__visible_number(hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_progress_bar__set_visible_number(hnd: POINTER; val: BOOLEAN) is
      external "C"
      end
   
	 
end
