class  AGUI_SCROLLABLE_CLIENT_CONTROL
   
-- Sends a AGUI_CHANGE_EVENT whenever the scrollbar changed
-- the position of the viewport. The "index" item of the event
-- is 1, if changed in horizontal direction and 2 if changed
-- vertical
   
-- All coordinates in AGUI_INPUT_EVENTS and the painting context
-- are specified in content coordinates, not window coordinates.
   
--TODO Autoscroll mode
--TODO wenn freigegeben wird, dann muss irgendwie auch der paint 
--context befreit werden, falls vorhanden
   
   
inherit
   AGUI_BASE_CLIENT_CONTROL
      redefine efox__callback_changed,invalidate
      end
   AGUI_SCROLLING_MIXIN
      redefine mixin_window
      end
   
   
creation
   {ANY} make
   
   
feature {ANY}  -- manage size and visible area of content
      
   
   viewport_x : INTEGER is
      do
         result := efox_scrollable_client_control__viewport_x(handle)
      ensure
	 result >= 0
      end

   viewport_y : INTEGER is
      do
         result := efox_scrollable_client_control__viewport_y(handle)
      ensure
	 result >= 0
      end

   viewport_width : INTEGER is
      do
         result := efox_scrollable_client_control__viewport_w(handle)
      ensure
	 result >= 0
      end

   viewport_height : INTEGER is
      do
         result := efox_scrollable_client_control__viewport_h(handle)
      ensure
	 result >= 0
      end
      
   
   content_width : INTEGER
   content_height : INTEGER
   
   set_content_size (w,h: INTEGER) is
         -- notify that the content area changed. Because we are
	 -- doing the layout asynchron from the calling of the function
	 -- you can't assume that the features 'viewport_x' ,
	 -- 'viewport_y' , 'viewport_width' and 'viewport_height' 
	 -- reflect the change immediate after the call. 
	 -- In fact they are unchanged.
      require
         not is_zombie
         w > 0
         h > 0
      do
         content_width := w
         content_height := h
         efox_scrollable_client_control__set_content_size(handle, w,h)
      ensure
	 viewport_x = old viewport_x
	 viewport_y = old viewport_y
	 viewport_width = old viewport_width
	 viewport_height = old viewport_height
      end
   
   
   set_viewport_position (x, y: INTEGER) is
	 -- change the position of the viewport. In contrast to
	 -- 'set_content_size' the effect on the 'viewport_XXXX'
	 -- features is visible after the call (because it requires
	 -- no relayout).
      require
         not is_zombie
      do
         efox_scrollable_client_control__set_viewport(handle, x,y)
      end
      
   
   vertical_page_size : INTEGER
   vertical_line_size : INTEGER
   
   set_vertical_scroll_size(page,line: INTEGER) is
      require
         not is_zombie
         page > 0
         line > 0
      do
         vertical_page_size := page
         vertical_line_size := line
         efox_scrollable_client_control__set_vscroll(handle, page,line)
      end
   
   
   
   horizontal_page_size : INTEGER
   horizontal_line_size : INTEGER
   
   set_horizontal_scroll_size(page,line: INTEGER) is
      require
         not is_zombie
         page > 0
         line > 0
      do
         horizontal_page_size := page
         horizontal_line_size := line
         efox_scrollable_client_control__set_hscroll(handle, page,line)
      end
   
   
   
   is_point_visible (x,y: INTEGER) : BOOLEAN is
      obsolete "Not yet implemented"
      do
      end
   
   is_area_visible (x,y,w,h: INTEGER) : INTEGER is
      obsolete "Not yet implemented"
      do
      end
   

feature {ANY}   -- auto scrolling
   
   
   start_autoscroll (x,y: INTEGER; only_when_inside: BOOLEAN) : BOOLEAN is
         -- from now on start autoscrolling of the widget if the mouse
         -- moves to one of the edges
      do
         result := efox_scrollable_client_control__start_autoscroll(handle, x,y,only_when_inside)
      end
   
   
   stop_autoscroll is
      do
         efox_scrollable_client_control__stop_autoscroll(handle)
      end
   
   
feature {ANY}   -- display contents

   
   invalidate (x,y,w,h: INTEGER) is
      do
         efox_scrollable_client_control__invalidate(handle, x,y,w,h,viewport_width,viewport_height)
      end   
   
   
feature {ANY} -- event handling
   
   
   on_mouse_adjust_motion : ROUTINE[ANY,TUPLE[INTEGER,INTEGER]] is
      do
	 result ?= find_callback_agent(On_mouse_adjust_event)
      end
      
   set_on_mouse_adjust_motion (action: ROUTINE[ANY,TUPLE[INTEGER,INTEGER]]) is
      do
	 set_callback_agent(On_mouse_adjust_event, action)
      end
   
   
   on_changed: ROUTINE[ANY,TUPLE[INTEGER,BOOLEAN]] is
         -- This agent is called every time the text is changed
      do
         result := find_callback_agent(On_change_event)
      end
      
   set_on_changed(action: ROUTINE[ANY,TUPLE[INTEGER,BOOLEAN]]) is
         -- Set the 'on_changed' event handler
      do
         set_callback_agent(On_change_event, action)
      end
   
   

feature {NONE} -- constructor, not for reinitialization
   
   
   make (p: AGUI_COMPOSITE;  n: STRING) is
      require
	 p /= Void and n /= Void
      do
	 handle := efox_scrollable_client_control__make(Current,p.handle, n.to_external, $temp_px)
	 efox__window_hash_table.put(Current, temp_px)
         content_width := 100    
         content_height := 100
         vertical_page_size := 10
         vertical_line_size := 1
         horizontal_page_size := 10
         horizontal_line_size := 1
	 create paint_context_private.make
      end
   
   
feature {ANY}  -- mixin configure
   
   
   mixin_window : AGUI_WINDOW is
	 -- redefine the mixin window references to this window
      do
	 result := Current
      end
   
   
feature {NONE} -- private C binding
  
   counter : INTEGER
   
   efox__callback_changed(what: INTEGER; state: BOOLEAN; obj: ANY) is
      local 
	 a: ROUTINE[ANY,TUPLE[INTEGER,BOOLEAN]]
      do
	 a ?= find_callback_agent(On_change_event)
	 if a /= Void then
	    a.call([what,state])
	 end
      end
      
   efox__callback_adjust_mouse   (x,y: INTEGER) is
      local 
	 a: ROUTINE[ANY,TUPLE[INTEGER,INTEGER]]
      do
	 a ?= find_callback_agent(On_mouse_adjust_event)
	 if a /= Void then
	    a.call([x,y])
	 end
      end
   
   
   efox_scrollable_client_control__make (this:like Current; hnd,n,keyptr: POINTER) : POINTER is
      external "C"
      end
   
   efox_scrollable_client_control__start_autoscroll (hnd: POINTER; x,y: INTEGER; flag: BOOLEAN) : BOOLEAN is
      external "C"
      end
   
   efox_scrollable_client_control__stop_autoscroll (hnd: POINTER) is
      external "C"
      end   
   
   efox_scrollable_client_control__viewport_y (hnd: POINTER) : INTEGER is
      external "C" 
      end

   efox_scrollable_client_control__viewport_x (hnd: POINTER) : INTEGER is
      external "C" 
      end

   efox_scrollable_client_control__viewport_w (hnd: POINTER) : INTEGER is
      external "C" 
      end

   efox_scrollable_client_control__viewport_h (hnd: POINTER) : INTEGER is
      external "C" 
      end

   efox_scrollable_client_control__set_vscroll (hnd: POINTER; p,l: INTEGER) is
      external "C"
      end

   efox_scrollable_client_control__set_hscroll (hnd: POINTER; p,l: INTEGER) is
      external "C"
      end

   efox_scrollable_client_control__set_viewport (hnd: POINTER; x,y: INTEGER) is
      external "C"
      end
   
   efox_scrollable_client_control__set_content_size (hnd: POINTER; x,y: INTEGER) is
      external "C"
      end

   efox_scrollable_client_control__invalidate(hnd: POINTER; x,y,w,h,vw,vh: INTEGER) is
      external "C"
      end
   
end

      
