deferred class  AGUI_SETTINGS
   
-- Valid section names can contain only alphanumerical ASCII 
-- characters or the "-" "_" or " " character. The length of a
-- Section name is restricted to 199 Characters.
   
-- Key names can contain only alphanumerical ASCII 
-- characters or the special characters "-_./\" or the space
-- character. The length of a Key name is restricted to 199 Characters.
   
   
-- TODO String error, no Binary savings
-- TODO error if string value contains a 0 character
   
--TODO Fox Bug: clear loescht nicht alles
   
feature {ANY}  -- misc operations
   
   
   is_modified : BOOLEAN is
	 -- Is the settings database modified, since
	 -- last save
      do
	 result := efox_settings__is_modified(handle)
      end   
     
   
   save is
	 -- Save settings database.
	 -- Set last_status to 'true' if saving was okay
      deferred
      end
   
   
   last_status : BOOLEAN 
	 -- Keeps success status for last 'load' or 'save' operation
   
   
   clear is
	 -- Remove all sections and entries.
      do
	 handle := efox_settings__clear(handle)
      end
   
   
feature {ANY}  -- manage sections
   
   
   delete_section (section: STRING) is
	 -- Delete a section with all the entries.
      require
	 is_valid_section(section)
      do
	 efox_settings__delete_section(handle, section.to_external)
      ensure
	 not exist_section(section)
      end
   
   
   exist_section (section: STRING) : BOOLEAN is
	 -- Test if a section exists
      require
	 is_valid_section(section)
      do
	 result := efox_settings__exist_section(handle, section.to_external)
      end
   
   
   sections : ARRAY[STRING] is
	 -- List names of all sections.
	 -- The array and all strings are newly created on every call.
      do
	 create result.with_capacity(10,1)
	 efox_settings__sections(Current,handle, result)
      ensure
	 result /= Void
	 result.lower = 1
      end
   
   
feature {ANY}  -- manage entries
   
   
   entries (section: STRING) : ARRAY[STRING] is
	 -- List names of all entries in the section.
	 -- The array and all strings are newly created on every call.
      require
	 is_valid_section(section)
      do
	 create result.with_capacity(10,1)
	 efox_settings__entries(Current,handle, section.to_external, result)
      ensure
	 result.lower = 1
	 result /= Void	 
      end
   
   
   delete (section, key: STRING) is
	 -- Delete an entry from database
      require
	 is_valid_section(section)
	 is_valid_key(key)
      do
	 efox_settings__delete(handle, section.to_external, key.to_external)
      ensure
	 not exist(section,key)
      end
   
   
   exist (section,key: STRING) : BOOLEAN is
	 -- Test if there is a specific entry
      require
	 is_valid_section(section)
	 is_valid_key(key)
      do
	 result := efox_settings__exist(handle, section.to_external, key.to_external)
      end
   

feature {ANY}  -- writting information
   
   
   write_integer (section, key: STRING; value: INTEGER) is
      require
	 is_valid_section(section)
	 is_valid_key(key)
      do
	 efox_settings__write_integer(handle, section.to_external, key.to_external, value)
      end
   
   write_double (section, key: STRING; value: DOUBLE) is
      require
	 is_valid_section(section)
	 is_valid_key(key)
      do
	 efox_settings__write_double(handle, section.to_external, key.to_external, value)
      end
   
   write_string (section, key: STRING; value: STRING) is
      require
	 is_valid_section(section)
	 is_valid_key(key)
	 value /= Void
	 not value.has('%U')
      do
	 efox_settings__write_string(handle, section.to_external, key.to_external, value.to_external)
      end
   
   write_color (section, key: STRING; value: AGUI_COLOR) is
      require
	 is_valid_section(section)
	 is_valid_key(key)
      do
	 efox_settings__write_color(handle, section.to_external, key.to_external, value.to_integer)
      end
   
   write_boolean (section, key: STRING; value: BOOLEAN) is
      require
	 is_valid_section(section)
	 is_valid_key(key)
      do
	 efox_settings__write_bool(handle, section.to_external, key.to_external, value)
      end
   
   
feature {ANY}  -- reading information
   
   
   read_integer (section, key: STRING; default_value: INTEGER) : INTEGER is
      require
	 is_valid_section(section)
	 is_valid_key(key)
      do
	 result := efox_settings__read_integer(handle, section.to_external,key.to_external, default_value)
      end
   
   read_double (section, key: STRING; default_value: DOUBLE) : DOUBLE is
      require	 
	 is_valid_section(section)
	 is_valid_key(key)
      do
	 result := efox_settings__read_double(handle, section.to_external,key.to_external, default_value)
      end
   
   read_string (section, key: STRING; default_value: STRING) : STRING is
	 -- TODO error if value contains a 0 character
      require
	 is_valid_section(section)
	 is_valid_key(key)
	 default_value /= Void
      local
	 p : POINTER
      do
	 p := efox_settings__read_string(handle, section.to_external, key.to_external, default_value.to_external)
	 create result.from_external_copy(p)
      ensure
	 result /= Void
      end
   
   read_color (section, key: STRING; default_value: AGUI_COLOR) : AGUI_COLOR is
      require
	 is_valid_section(section)
	 is_valid_key(key)
      do
	 result.from_integer(efox_settings__read_color(handle, section.to_external,key.to_external, default_value.to_integer))
      end   
   
   read_boolean (section, key: STRING; default_value: BOOLEAN) : BOOLEAN is
      require
	 is_valid_section(section)
	 is_valid_key(key)
      do
	 result := efox_settings__read_bool(handle, section.to_external,key.to_external, default_value)
      end
   

feature {ANY}  -- array functions
   
   
   read_string_array (section, key: STRING) : ARRAY[STRING] is
	 -- reads all entries "key-xx" where xx is an increasing number starting
	 -- with '1' into an array
      require
	 is_valid_section(section)
	 is_valid_key(key)
      local
	 i : INTEGER
	 s : STRING
      do
	 create result.with_capacity(8,1)
	 from
	    i := 1
	    s := key + "-1"
	 until
	    not exist(section,s)
	 loop
	    result.add_last(read_string(section,s,""))
	    i := i + 1
	    s.copy(key)
	    s.extend('-')
	    i.append_in(s)
	 end
      ensure
	 result /= Void
      end
   
   
   write_string_array (section, key: STRING; val: ARRAY[STRING]) is
	 -- writes the strings as entries "key-xx" where xx is an increasing 
	 -- number starting with '1'
      require
	 is_valid_section(section)
	 is_valid_key(key)
	 val /= Void
      local
	 i,j,n : INTEGER
	 s : STRING
      do
	 -- first remove existing keys
	 from
	    i := 1
	    s := key + "-1"
	 until
	    not exist(section,s)
	 loop
	    delete(section,s)
	    i := i + 1
	    s := key + "-" + i.to_string
	 end
	 -- now insert values
	 from
	    j := 1
	    i := val.lower
	    n := val.upper
	 until
	    i > n
	 loop
	    write_string(section, key + "-" + j.to_string, val.item(i))
	    j := j + 1
	    i := i + 1
	 end
      end
   
   
   read_integer_array (section, key: STRING) : ARRAY[INTEGER] is
      require
	 is_valid_section(section)
	 is_valid_key(key)
      local
	 i: INTEGER
	 s,s2: STRING
      do
	 create result.with_capacity(8,1)
	 from
	    i := 1
	    s := key + "-1"
	 until
	    not exist(section,s)
	 loop
	    s2 := read_string(section,s,"")
	    if s2.is_integer then
	       result.add_last(s2.to_integer)
	    else
	       result.add_last(0)
	    end
	    i := i + 1
	    s.copy(key)
	    s.extend('-')
	    i.append_in(s)
	 end
      ensure
	 result /= Void
      end
   
   
   write_integer_array (section, key: STRING; val: ARRAY[INTEGER]) is
	 -- writes the strings as entries "key-xx" where xx is an increasing
	 -- number starting with '1'
      require
	 is_valid_section(section)
	 is_valid_key(key)
	 val /= Void
      local
	 i,j,n : INTEGER
	 s : STRING
      do
	 -- first remove existing keys
	 from
	    i := 1
	    s := key + "-1"
	 until
	    not exist(section,s)
	 loop
	    delete(section,s)
	    i := i + 1
	    s := key + "-" + i.to_string
	 end
	 -- now insert values
	 from
	    j := 1
	    i := val.lower
	    n := val.upper
	 until
	    i > n
	 loop
	    write_integer(section, key + "-" + j.to_string, val.item(i))
	    
	    j := j + 1
	    i := i + 1
	 end
      end
   
   
feature {ANY}  -- verifiying data
   
   
   is_valid_key(key: STRING) : BOOLEAN is
      do
	 result := key /= Void and then key.count > 0 and then key.count <= 100
      end   
   
   is_valid_section(section: STRING) : BOOLEAN is
      do
	 result := section /= Void and then section.count > 0 and then section.count <= 100
      end


feature {ANY} -- debugging


   debug_output is
	 -- Output complete settings file
      local
	 i,j: INTEGER
	 ea,sa: ARRAY[STRING]
      do
	 io.put_string("settings%N==================================%N")
	 sa := sections
	 from i := sa.lower until i > sa.upper loop
	    ea := entries(sa.item(i))
	    from j := ea.lower until j > ea.upper loop
	       io.put_string(sa.item(i) + ":" + ea.item(j) + " ==> ")
	       io.put_string(read_string(sa.item(i),ea.item(j),""))
	       io.put_character('%N')
	       j := j + 1
	    end
	    i := i + 1
	 end
	 io.put_string("==================================%N")
      end

   
feature {NONE} -- init
   
   
   init_settings (hnd: POINTER) is
      do
	 handle := hnd
      end
   
   efox__callback(a: ARRAY[STRING]; p: POINTER) is
      local
	 t: STRING
      do
	 create t.from_external_copy(p)
	 a.add_last(t)
      end
   
   
feature {AGUI_APPLICATION}
   
   
   handle : POINTER
	 -- pointer to corresponding FOX object
   
   
feature {NONE} -- private C binding   
   
   
   add_callback (sa: ARRAY[STRING]; s: STRING) is
      do
	 sa.add_last(clone(s))
      end
   
   efox_settings__clear(hnd: POINTER) : POINTER is
      external "C"
      end
   
   efox_settings__is_modified(hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_settings__sections(this:like Current; hnd: POINTER; r: ARRAY[STRING]) is
      external "C"
      end
   
   efox_settings__delete_section(hnd,s: POINTER) is
      external "C"
      end
   
   efox_settings__exist_section(hnd,s: POINTER) : BOOLEAN is
      external "C"
      end
      
      
   efox_settings__entries(this:like Current; hnd,s: POINTER; r: ARRAY[STRING]) is
      external "C"
      end
   
   efox_settings__delete(hnd,s,k: POINTER) is
      external "C"
      end
   
   efox_settings__exist(hnd,s,k: POINTER) : BOOLEAN is
      external "C"
      end
         
      
   efox_settings__read_integer(hnd,s,k: POINTER; dv: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_settings__read_double(hnd,s,k: POINTER; dv: DOUBLE) : DOUBLE is
      external "C"
      end
   
   efox_settings__read_color(hnd,s,k: POINTER; dv: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_settings__read_bool(hnd,s,k: POINTER; dv: BOOLEAN) : BOOLEAN is
      external "C"
      end
   
   efox_settings__read_string(hnd,s,k,dv: POINTER) : POINTER is
      external "C"
      end     

   efox_settings__write_bool(hnd,s,k: POINTER; v: BOOLEAN) is
      external "C"
      end        
   
   efox_settings__write_integer(hnd,s,k: POINTER; v: INTEGER) is
      external "C"
      end     

   efox_settings__write_color(hnd,s,k: POINTER; v: INTEGER) is
      external "C"
      end     

   efox_settings__write_string(hnd,s,k,v: POINTER) is
      external "C"
      end     

   efox_settings__write_double(hnd,s,k: POINTER; v: DOUBLE) is
      external "C"
      end     


end

   
   
	 
