class  AGUI_SLIDER
   
   
-- The slider widget is a valuator widget which provides simple linear value range.
-- Two visual appearances are supported:- the sunken look, which is enabled with
-- the 'Slider_inside_bar' option and the regular look.  The latter may have optional
-- arrows on the slider thumb.
-- While being moved, the slider invokes the 'on_changed' agent;
-- at the end of the interaction, the 'on_command' agent is invoked.
-- The agents parameter represents the current slider value.


inherit
   AGUI_CONTROL
      redefine
	 efox__callback_command,
	 efox__callback_changed
      end
   AGUI_FRAME_MIXIN
      redefine
	 mixin_window
      end
   AGUI_PACKER_MIXIN
      redefine
	 mixin_window
      end
   
   
creation
   {ANY} make

   
   
feature {ANY} -- slider styles
   
   
   Slider_horizontal : INTEGER is 0
	 -- Slider shown horizontally
   
   Slider_vertical : INTEGER is 32768
	 -- Slider shown vertically
   
   Slider_arrow_up : INTEGER is 65536
	 -- Slider has arrow head pointing up
   
   Slider_arrow_down : INTEGER is 131072
	 -- Slider has arrow head pointing down
   
   Slider_arrow_left : INTEGER is 65536
	 -- Slider has arrow head pointing left
   
   Slider_arrow_right : INTEGER is 131072
	 -- Slider has arrow head pointing right
   
   Slider_inside_bar : INTEGER is 262144
	 -- Slider is inside the slot rather than overhanging
   
   
   
   style : INTEGER is
	 -- Get the current slider style
      do
	 result := efox_slider__style(handle)
      end
   
   set_style(n: INTEGER) is
	 -- Change the current slider style
      do
	 efox_slider__set_style(handle,n)
      end
   
   
feature {ANY} -- visual appearance
   
   
   head_size : INTEGER is
	 -- Get the slider's head size
      do
	 result := efox_slider__head_size(handle)
      end
   
   set_head_size (n: INTEGER) is
	 -- Change the slider's head size
      do
	 efox_slider__set_head_size(handle,n)
      end
   
   
   slot_size : INTEGER is
	 -- Get the slider's current slot size
      do
	 result := efox_slider__slot_size(handle)
      end
   
   set_slot_size (n: INTEGER) is
	 -- Change the slider's current slot size
      do
	 efox_slider__set_slot_size(handle,n)
      end
   
   
   slot_color : AGUI_COLOR is
	 -- Change the color of the slot the slider head moves in
      do
	 result.from_integer(efox_slider__slot_color(handle))
      end
   
   set_slot_color (col: AGUI_COLOR) is
	 -- Change the color of the slot the slider head moves in
      do
	 efox_slider__set_slot_color(handle, col.to_integer)
      end
      
      
feature {ANY} -- values
   
   
   value: INTEGER is
	 -- return slider value
      require
	 not is_zombie
      do
	 result := efox_slider__value(handle)
      end
   
   set_value (n: INTEGER) is
	 -- change slider value
      require
	 not is_zombie
      do
	 efox_slider__set_value(handle, n)
      end
   
   
   range : TUPLE[INTEGER,INTEGER] is
	 -- retrieve sliders range
      require
	 not is_zombie
      local
	 low: INTEGER
      do
	 low := efox_slider__range(handle,$temp_px)
	 result := [low,temp_px]
      ensure
	 result /= Void
      end
   
   set_range (low_high: TUPLE[INTEGER,INTEGER]) is
      -- change sliders range
      require
	 not is_zombie
	 low_high /= Void
      do
	 efox_slider__set_range(handle, low_high.first, low_high.second)
      end
   
	 
   increment_value : INTEGER is
	 -- return the auto-increment/auto-decrement value.
      require
	 not is_zombie
      do
	 result := efox_slider__increment_value(handle)
      end   
   
   set_increment_value (n: INTEGER) is
	 -- change the auto-increment/auto-decrement value.
      require
	 not is_zombie
      do
	 efox_slider__set_increment_value(handle,n)
      end
   
   
feature {ANY} -- events
   
   
   on_change : ROUTINE[ANY,TUPLE[INTEGER]] is
	 -- Action which is triggered when the slider is being moved.
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_change_event)
      end
   
   
   on_command : ROUTINE[ANY,TUPLE[INTEGER]] is
	 -- Action that is triggered after a slider was moved.
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_command_event)
      end
   
   
   set_on_changed (action: ROUTINE[ANY,TUPLE[INTEGER]]) is
	 -- Set the change action routine.
      require
	 not is_zombie
      do
	 set_callback_agent(On_change_event, action)
      end
   
   
   set_on_command (action: ROUTINE[ANY,TUPLE[INTEGER]]) is
	 -- Set the command action routine.
      require
	 not is_zombie
      do
	 set_callback_agent(On_command_event, action)
      end
   
	 
feature {ANY}  -- mixin configure
   
   
   mixin_window : AGUI_WINDOW is
	 -- redefine the mixin window references to this window
      do
	 result := Current
      end
   
   
feature {NONE}  -- constructor
   
   
   make (p: AGUI_COMPOSITE;  n: STRING) is
	 -- Create a slider control widget under parent 'p' and give 
	 -- it the name 'n'.
      require
	 p /= Void and n /= Void
      do
	 handle := efox_slider__make(Current,p.handle, n.to_external, $temp_px)
	 efox__window_hash_table.put(Current, temp_px)
      end
   
   
feature {NONE} -- private implementation
   
   
   efox__callback_changed(val: INTEGER; state: BOOLEAN; obj: ANY) is
      require
	 not is_zombie
      local
	 action: ROUTINE[ANY,TUPLE[INTEGER]]
      do
	 action := on_change
	 if action /= Void then action.call([val]) end
      end
   
   efox__callback_command(val: INTEGER; state: BOOLEAN; obj: ANY) is
      require
	 not is_zombie
      local
	 action: ROUTINE[ANY,TUPLE[INTEGER]]
      do
	 action := on_command
	 if action /= Void then action.call([val]) end
      end
   
   
   efox_slider__make(this:like Current; hnd, n, k: POINTER) : POINTER is
      external "C"
      end
   
   efox_slider__set_style(hnd: POINTER; n: INTEGER) is
      external "C"
      end
      
   efox_slider__style(hnd: POINTER) : INTEGER is
      external "C"
      end
	 
   efox_slider__head_size(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_slider__set_head_size(hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_slider__slot_size(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_slider__set_slot_size(hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_slider__slot_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_slider__set_slot_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end      
   
   efox_slider__increment_value(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_slider__set_increment_value(hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_slider__range(hnd, dh: POINTER) : INTEGER is
      external "C"
      end
   
   efox_slider__set_range(hnd: POINTER; l,h: INTEGER) is
      external "C"
      end
   
   efox_slider__value(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_slider__set_value(hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   
end
