class  AGUI_STATUSLINE
   
-- IMPROVE: temporary messages that are shown for a specific amount 
--          of time and then removed automatically.
--
-- If the message contains a newline (%N), then the part before the newline
-- will be displayed in the highlight color, while the part after the newline
-- is shown using the normal text color.


inherit
   AGUI_WINDOW
      end
   AGUI_FRAME_MIXIN
      redefine mixin_window
      end
   AGUI_PACKER_MIXIN
      redefine mixin_window
      end
      

creation 
   {ANY} make

creation 
   {AGUI_APPLICATION} make_internal
   
   
feature {NONE} 
   
   fixed_text : STRING
   timer_task_id : INTEGER
   
   reset_fixed_text is
      do
	 if not is_zombie then
	    if fixed_text /= Void then
	       set_text(fixed_text)
	    else
	       set_text("")
	    end
	 end
      end
   
   
feature {ANY}  -- public features
   
   
   text : STRING is
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_statusline__text(handle))
      ensure
	 result /= Void
      end
   
   
   set_text (str: STRING) is
      require
	 not is_zombie
	 str /= Void
      do
	 if timer_task_id /= 0 then
	    application.remove_timer_task(timer_task_id)
	    timer_task_id := 0
	 end
	 fixed_text := clone(str)
	 efox_statusline__set_text(handle, str.to_external)
      end
   
   
   set_temporary_text (str: STRING) is
      require
	 not is_zombie
	 str /= Void
      do
	 if timer_task_id /= 0 then
	    application.remove_timer_task(timer_task_id)
	    timer_task_id := 0
	 end
	 application.add_timer_task(3000, agent reset_fixed_text)
	 timer_task_id := application.last_timer_task_id
	 efox_statusline__set_text(handle, str.to_external)
      end
   
   
   text_color : AGUI_COLOR is
      require
	 not is_zombie
      do
	 result.from_integer(efox_statusline__text_color(handle))
      end
   
   
   set_text_color (col: AGUI_COLOR) is
      require
	 not is_zombie
      do
	 efox_statusline__set_text_color(handle,col.to_integer)
      end
   
   
   highlight_text_color : AGUI_COLOR is
      require
	 not is_zombie
      do
	 result.from_integer(efox_statusline__highlight_text_color(handle))
      end
   
   
   set_highlight_text_color (col: AGUI_COLOR) is
      require
	 not is_zombie
      do
	 efox_statusline__set_highlight_text_color(handle,col.to_integer)
      end
   
   
   font : AGUI_FONT is 
      require
	 not is_zombie
      do
	 result := efox_statusline__font(handle)
      ensure
	 result /= Void
      end
   
   
   set_font (f: AGUI_FONT) is
      require
	 not is_zombie
	 f /= Void and then not f.is_zombie
      do
	 efox_statusline__set_font(handle, f.handle)
      end
	 
   
feature {ANY}  -- mixin configure
   
   
   mixin_window : AGUI_WINDOW is
	 -- redefine the mixin window references to this window
      do
	 result := Current
      end
   
   
feature {NONE} -- constructor
   
   
   make (p: AGUI_COMPOSITE;  n: STRING) is
      require
	 p /= Void and n /= Void
      do
	 handle := efox_statusline__make(Current,p.handle, n.to_external, $temp_px)
	 efox__window_hash_table.put(Current, temp_px)
      end
   
   
   make_internal (hashkey: INTEGER; hnd: POINTER) is
      require
	 hnd /= default_pointer
      do
	 handle := hnd
	 efox__window_hash_table.put(Current, hashkey)
      end
   
   
feature {NONE} -- private C binding
   
   
   efox_statusline__font(hnd: POINTER) : AGUI_FONT is
      external "C"
      end
   
   efox_statusline__set_font(hnd, fhnd: POINTER) is
      external "C"
      end
      
   efox_statusline__text_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_statusline__set_text_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end
   
   efox_statusline__highlight_text_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_statusline__set_highlight_text_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end
   
   efox_statusline__set_text(hnd, str: POINTER) is
      external "C"
      end
   
   efox_statusline__text(hnd: POINTER) : POINTER is
      external "C"
      end
   
--   efox_statusline__set_temporary_text(hnd, str: POINTER) is
--      external "C"
--      end
--	 
--   efox_statusline__temporary_text(hnd: POINTER) : POINTER is
--      external "C"
--      end
   
   efox_statusline__make(this:like Current; hnd, p, keyptr: POINTER) : POINTER is
      external "C"
      end      
   
end
   
