class  AGUI_TEXT
   
--IMPROVE: change cursor color
   

inherit
   AGUI_TEXT_BASE
      end
   
creation
   {ANY} make
   

feature {NONE} -- constructor

   
   make (p: AGUI_COMPOSITE;  n: STRING) is
      require
	 p /= Void and n /= Void
      do
	 handle := efox_text__make(Current,p.handle, n.to_external, $temp_px)
	 efox__window_hash_table.put(Current, temp_px)
	 set_visible_size(40,10)
      end


feature {ANY} -- misc
   
   
   is_modified : BOOLEAN is
	 -- Test if text is modified by user after
	 -- last call to 'reset_modified' or creation.
      require
	 not is_zombie
      do
	 result := efox_text__is_modified(handle)
      end
   
   
   reset_modified is
      require
	 not is_zombie
      do
	 efox_text__reset_modified(handle)
      end
   
   
feature {ANY} -- margins
   
   
   set_margin_top (n: INTEGER) is
      require
	 not is_zombie
      do
	 efox_text__set_margin_top(handle, n)
      end
   
   margin_top : INTEGER is
      require
	 not is_zombie
      do
	 result := efox_text__margin_top(handle)
      end
   
   set_margin_bottom (n: INTEGER) is
      require
	 not is_zombie
      do
	 efox_text__set_margin_bottom(handle, n)
      end
   
   margin_bottom : INTEGER is
      require
	 not is_zombie
      do
	 result := efox_text__margin_bottom(handle)
      end
   
   set_margin_left (n: INTEGER) is
      require
	 not is_zombie
      do
	 efox_text__set_margin_left(handle, n)
      end
   
   margin_left : INTEGER is
      require
	 not is_zombie
      do
	 result := efox_text__margin_left(handle)
      end
   
   set_margin_right (n: INTEGER) is
      require
	 not is_zombie
      do
	 efox_text__set_margin_right(handle, n)
      end
   
   margin_right : INTEGER is
      require
	 not is_zombie
      do
	 result := efox_text__margin_right(handle)
      end
   
   
feature {ANY} -- wrapping
   
   
   wrap_mode : BOOLEAN is
      require
	 not is_zombie
      do
      end
      
   set_wrap_mode (enable_wrapping: BOOLEAN) is
      require
	 not is_zombie
      do
      end      
   
   wrap_columns : INTEGER is
      require
	 not is_zombie
      do
      end
   
   set_wrap_columns (cols: INTEGER) is
      require
	 not is_zombie
	 cols >= 1
      do
      end
   
      
feature {ANY} -- tabbing 
   
   
   tab_to_space_mode : BOOLEAN is
	 -- if true, than spaces are inserted instead of tabs
      require
	 not is_zombie
      do
	 result := efox_text__tab_to_space_mode(handle)
      end
   
   set_tab_to_space_mode (flag: BOOLEAN) is
	 -- decide if spaces or tabs should be inserted
      require
	 not is_zombie
      do
	 efox_text__set_tab_to_space_mode(handle, flag)
      end
   
   
   tab_columns : INTEGER is
      require
	 not is_zombie
      do
	 result := efox_text__tab_columns(handle)
      end
   
   
   set_tab_columns (n: INTEGER) is
      require
	 not is_zombie
	 n >= 1
      do
	 efox_text__set_tab_columns(handle, n)
      end
   
   
feature {ANY}  -- move content
   
   
   top_line : INTEGER is
	 -- Return the position of top line.
      do
	 result := efox_text__top_line(handle)
      end
   
   set_top_line (pos: INTEGER) is
	 -- Make line containing pos the top line.
      do
	 efox_text__set_top_line(handle,pos)
      end
   
   
   set_bottom_line (pos: INTEGER) is
	 -- Make line containing 'pos' the bottom line.
      do
	 efox_text__set_bottom_line(handle,pos)
      end
   
   bottom_line : INTEGER is
	 -- Return the position of the bottom line.
      do
	 result := efox_text__bottom_line(handle)
      end
   
   
   set_center_line (pos: INTEGER) is
	 -- Make line containing pos the center line.
      do
	 efox_text__set_center_line(handle,pos)
      end
   
      
feature {ANY}   -- cursor management
   
   
   cursor_line : INTEGER is
	 -- returns line of cursor (line index starts with 1)
      require
	 not is_zombie
      do
	 result := efox_text__cursor_line(handle)
      ensure
	 result >= 1
      end
   
   
   cursor_column : INTEGER is
	 -- return column of cursor (column index starts with 0, with
	 -- means before any character, 1 means behind the first 
	 -- character etc.)
      require
	 not is_zombie
      do
	 result := efox_text__cursor_column(handle)
      ensure
	 result >= 0
      end
   
   
   set_cursor_line (line_no: INTEGER) is
      require
	 not is_zombie
	 line_no >= 1
      do
	 efox_text__set_cursor_line(handle,line_no)
      ensure
	 cursor_line <= line_no  -- last line if line_no exceeds the existing range 
      end
   
   
   set_cursor_column (column_no: INTEGER) is
      require
	 not is_zombie
	 column_no >= 0
      do
	 efox_text__set_cursor_column (handle,column_no)
      ensure
	 cursor_column <= column_no -- last character if column_no exceeds the length of the line
      end
   
   
feature {ANY}  -- delimiters
   
   
   delimiters : STRING is
	 -- Return a new string with all word delimiting characters.
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_text__delimiters (handle))
      ensure
	 result /= Void
      end
   
   
   set_delimiters (str: STRING) is
	 -- Use all characters in 'str' as word delimiting characters.
      require
	 not is_zombie
	 str /= Void
      do
	 efox_text__set_delimiters (handle, str.to_external)
      end
   
   
feature {ANY}  -- position
   
   
   position_at(x,y: INTEGER) : INTEGER is
      require
	 not is_zombie	 
      do
	 result := efox_text__position_at(handle, x, y)
      end
      
   
   position_of_begin_of_line(pos: INTEGER) : INTEGER is
	 -- Return position of begin of line containing position pos
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_text__position_begin_of_line(handle,pos)
      end

   position_of_end_of_line(pos: INTEGER) : INTEGER is
	 -- Return position of end of line containing position pos
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_text__position_end_of_line(handle,pos)
      end
   
   position_of_begin_of_next_line(pos: INTEGER) : INTEGER is
	 -- Return start of next line
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_text__position_begin_next_line(handle,pos)
      end
   
   position_of_begin_of_previous_line (pos: INTEGER) : INTEGER is
	 -- Return start of previous line
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_text__position_begin_prev_line(handle,pos)
      end
   
   position_of_row_start(pos: INTEGER) : INTEGER is
	 -- Return row start
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_text__position_row_start(handle,pos)
      end
   
   position_of_row_end(pos: INTEGER) : INTEGER is
	 -- Return row end
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_text__position_end_row(handle, pos)
      end
   
   position_of_begin_of_next_row (pos: INTEGER) : INTEGER is
	 -- Return start of next row
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_text__position_begin_next_row(handle,pos)
      end
   
   position_of_begin_of_previous_row (pos: INTEGER) : INTEGER is
	 -- Return start of previous row
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_text__position_begin_prev_row(handle,pos)
      end
   
   position_of_end_of_previous_word (pos: INTEGER) : INTEGER is      
	 -- Return end of previous word
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_text__position_end_prev_word(handle,pos)
      end
   
   position_of_begin_of_next_word (pos: INTEGER) : INTEGER is
	 -- Return begin of next word
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_text__position_begin_next_word(handle,pos)
      end
   
   position_of_begin_of_word (pos: INTEGER) : INTEGER is 
	 -- Return begin of word
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_text__position_begin_word(handle,pos)
      end
   
   position_of_end_of_word (pos: INTEGER) : INTEGER is
	 -- Return end of word
      require
	 not is_zombie
	 valid_position(pos)
      do
	 result := efox_text__position_end_word(handle,pos)
      end
   

feature {NONE} -- private C binding
   
   
   -- constructor
   
   efox_text__make(this:like Current; hnd,n,keyptr: POINTER) : POINTER is
      external "C"
      end   
   
   -- misc
   
   efox_text__is_modified(hnd: POINTER) : BOOLEAN is
      external "C"
      end   
   
   efox_text__reset_modified(hnd: POINTER) is
      external "C"
      end   
   
   -- margins
   
   efox_text__set_margin_top(hnd: POINTER; n: INTEGER) is
      external "C"
      end   
   
   efox_text__margin_top(hnd: POINTER) : INTEGER is
      external "C"
      end   
   
   efox_text__set_margin_bottom(hnd: POINTER; n: INTEGER) is
      external "C"
      end   
   
   efox_text__margin_bottom(hnd: POINTER) : INTEGER is
      external "C"
      end   
   
   efox_text__set_margin_left(hnd: POINTER; n: INTEGER) is
      external "C"
      end   
   
   efox_text__margin_left(hnd: POINTER) : INTEGER is
      external "C"
      end   
   
   efox_text__set_margin_right(hnd: POINTER; n: INTEGER) is
      external "C"
      end   
   
   efox_text__margin_right(hnd: POINTER) : INTEGER is
      external "C"
      end   
	 
   -- wrapping
   
   efox_text__wrap_mode(hnd: POINTER) : BOOLEAN is
      external "C"      
      end   	 
   
   efox_text__set_wrap_mode(hnd: POINTER; flag: BOOLEAN) is
      external "C"
      end   	 
   
   efox_text__wrap_columns(hnd: POINTER): INTEGER is
      external "C"
      end   	 
   
   efox_text__set_wrap_columns(hnd: POINTER; n: INTEGER) is
      external "C"
      end   	 
   
   -- tabbing      
   
   efox_text__tab_to_space_mode (hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_text__set_tab_to_space_mode (hnd: POINTER; flag: BOOLEAN) is
      external "C"
      end
    
   efox_text__tab_columns(hnd: POINTER) : INTEGER is
      external "C"
      end
      
   efox_text__set_tab_columns (hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   -- move content
   
   efox_text__set_top_line (hnd: POINTER; pos: INTEGER) is
      external "C"
      end   
   
   efox_text__top_line (hnd: POINTER) : INTEGER is
      external "C"
      end   
   
   efox_text__set_bottom_line (hnd: POINTER; pos: INTEGER) is
      external "C"
      end   
   
   efox_text__bottom_line (hnd: POINTER) : INTEGER is
      external "C"
      end   
   
   efox_text__set_center_line (hnd: POINTER; pos: INTEGER) is
      external "C"
      end   
   
   -- cursor management        
   
   efox_text__cursor_line(hnd: POINTER) : INTEGER is
      external "C"
      end   
   
   efox_text__cursor_column(hnd: POINTER) : INTEGER is
      external "C"
      end   
   
   efox_text__set_cursor_line(hnd: POINTER; n: INTEGER) is
      external "C"
      end   
   
   efox_text__set_cursor_column(hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   -- delimiters   
   
   efox_text__delimiters (hnd: POINTER) : POINTER is
      external "C"
      end
   
   efox_text__set_delimiters (hnd,str: POINTER) is
      external "C"
      end
      
   -- position  
   
   efox_text__position_at (hnd: POINTER; x,y: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_text__position_begin_of_line (hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_text__position_end_of_line (hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_text__position_begin_next_line (hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_text__position_begin_prev_line (hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_text__position_row_start (hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_text__position_end_row (hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_text__position_begin_next_row (hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_text__position_begin_prev_row (hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_text__position_end_prev_word (hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_text__position_begin_next_word (hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_text__position_begin_word (hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
   efox_text__position_end_word (hnd: POINTER; pos: INTEGER) : INTEGER is
      external "C"
      end
   
end

