deferred class  AGUI_TEXT_BASE
   
   
inherit
   AGUI_CONTROL
      rename
	 cursor_position as mouse_cursor_position
      redefine 
	 efox__callback_changed
      end
      

feature {ANY}   -- visible size
   
   
   visible_width : INTEGER is
         -- Default width of the text, measured in characters if greater
         -- then 0 or in pixel if less then 0.      
      require
         not is_zombie
      do
         result := efox_text_base__visible_width(handle)
      end
   
   
   visible_height : INTEGER is
         -- Default height of the text, measured in rows if greater
         -- then 0 or in pixel if less then 0.
	 -- In a 'AGUI_TEXT_FIELD' the visible_height can never be
	 -- greater then 1.
      require
         not is_zombie
      do
         result := efox_text_base__visible_height(handle)
      end
   
   
   set_visible_size (vwidth, vheight: INTEGER) is
         -- set visible size
      require
         not is_zombie
      do
         efox_text_base__set_visible_size(handle,vwidth,vheight)
      end


feature {ANY} -- visiual appearance properties
   
   
   font : AGUI_FONT is
      require
         not is_zombie
      do
         result := efox_text_base__font(handle)
      ensure
         result /= Void
      end
   
   
   set_font (f: AGUI_FONT) is
      require
         not is_zombie
         f /= Void and then not f.is_zombie 
      do
         efox_text_base__set_font(handle, f.handle)
      end
   
   
   text_color : AGUI_COLOR is
      require
         not is_zombie
      do
         result.from_integer(efox_text_base__text_color(handle))
      end
   
   
   set_text_color (col: AGUI_COLOR) is
      require
         not is_zombie
      do
         efox_text_base__set_text_color(handle, col.to_integer)
      end
   
   
   selected_background_color : AGUI_COLOR is
      require
         not is_zombie
      do
         result.from_integer(efox_text_base__selected_back_color(handle))
      end
   
   
   set_selected_background_color (col: AGUI_COLOR) is
      require
         not is_zombie
      do
         efox_text_base__set_selected_back_color(handle,col.to_integer)
      end
   
   
   selected_text_color : AGUI_COLOR is
      require
         not is_zombie
      do
         result.from_integer(efox_text_base__selected_text_color(handle))
      end
   
   
   set_selected_text_color (col: AGUI_COLOR) is
      require
         not is_zombie
      do
         efox_text_base__set_selected_text_color(handle,col.to_integer)
      end
   
   
feature {ANY}  -- public features
      
   
   overstrike_mode : BOOLEAN is
      require
         not is_zombie
      do
         result := efox_text_base__overstrike_mode(handle)
      end
   
   
   set_overstrike_mode (flag: BOOLEAN) is
      require
         not is_zombie
      do
         efox_text_base__set_overstrike_mode(handle, flag)
      end
   
      
   editable : BOOLEAN is
      require
         not is_zombie
      do
         result := efox_text_base__editable(handle)
      end   
   
   
   set_editable (flag: BOOLEAN) is
      require
         not is_zombie
      do
         efox_text_base__set_editable(handle, flag)
      end
   
   
   text : STRING is
         -- return new string with text
      require
         not is_zombie   
      do
         create result.from_external_copy(efox_text_base__text(handle))
      ensure
         result /= Void
      end
   
   
   set_text (str: STRING) is
         -- set the displayed text.
      require
         not is_zombie
         str /= Void
      do
         efox_text_base__set_text(handle, str.to_external)
      end
   
   
   count : INTEGER is
         -- return number of characters in widget
      require
         not is_zombie      
      do
         result := efox_text_base__count(handle)
      end
   
   
feature {ANY}  -- position manipulation
   
   
   is_position_visible (pos: INTEGER) : BOOLEAN is
      require
         not is_zombie      
         valid_position (pos)
      do
         result := efox_text_base__is_position_visible (handle, pos)
      end
   
   
   make_position_visible (pos: INTEGER) is
      require
         not is_zombie      
         valid_position (pos)
      do
         efox_text_base__make_position_visible (handle, pos)
      end
   
   
   valid_position (pos: INTEGER) : BOOLEAN is
      require
         not is_zombie
      do
         result := efox_text_base__valid_position(handle, pos)
      end
   
   
   text_cursor_position : INTEGER is
      require
         not is_zombie
      do
         result := efox_text_base__cursor_position (handle)
      end
   
   
   set_text_cursor_position (pos: INTEGER) is
      require
         not is_zombie
      do
         efox_text_base__set_cursor_position(handle, pos)
      end
   
   
feature {ANY}  -- manipulation management
   
   
   clear is
      obsolete "Not yet implemented"
      require
         not is_zombie
      do
         --TODO efox_text_base__clear(handle)
      end
   
   
   append (s: STRING) is
         -- append text at end of existing text
      require
         not is_zombie
      do
         efox_text_base__append(handle,s.to_external)
      end

   
   delete (pos: INTEGER; len: INTEGER) is
      obsolete "No position handling until we know how to handle UNICODE - use only 'text' and 'set_text'"
      require
         not is_zombie
         valid_position(pos)
      do
         efox_text_base__delete(handle,pos,len)
      end
   
   
   insert (pos: INTEGER; str: STRING) is
      obsolete "No position handling until we know how to handle UNICODE - use only 'text' and 'set_text'"
      require
         not is_zombie
         valid_position(pos)
         str /= Void
      do
         efox_text_base__insert(handle,pos,str.to_external)      
      end
   
   
   insert_substring (pos: INTEGER; str: STRING; aidx,eidx: INTEGER) is
      obsolete "No position handling until we know how to handle UNICODE - use only 'text' and 'set_text'"
      require
         not is_zombie
         valid_position(pos)
         str /= Void
         str.valid_index(aidx)
         str.valid_index(eidx)
         aidx <= eidx + 1
      do         
         efox_text_base__insert_substring(handle,pos,str.to_external,aidx,eidx)
      end
   
   
   extract (pos: INTEGER; len: INTEGER) : STRING is
      obsolete "No position handling until we know how to handle UNICODE - use only 'text' and 'set_text'"
      require
         not is_zombie
         valid_position(pos)
         len >= 0
      do
         create result.make(len)
         extract_in(pos,len,result)
      ensure
         result /= Void
      end
   
   
   extract_in (pos: INTEGER; len: INTEGER; str: STRING) is
      obsolete "NYI"
      require
         not is_zombie
         valid_position(pos)
         str /= Void
         len >= 0
      do
      end
   
   
feature {ANY}  -- selection management
   
   
   has_selection : BOOLEAN is
      require
         not is_zombie
      do
	 result := efox_text_base__has_selection(handle)
      end
   
   selection_begin_position : INTEGER is
      require
         not is_zombie
      do
         result := efox_text_base__selection_begin_pos(handle)
      ensure
         valid_position(result)
         result <= selection_end_position
      end
   
   
   selection_end_position : INTEGER is
      require
         not is_zombie
      do
         result := efox_text_base__selection_end_pos(handle)
      ensure
         valid_position(result)
         result >= selection_begin_position
      end
   
   
   selection  : STRING is
         -- return text of selection
      require
         not is_zombie
      do
         create result.from_external_copy(efox_text_base__selection(handle))
      ensure
         result /= Void
      end
   
   
   select_all is
      -- select all text
      require
         not is_zombie
      do
         efox_text_base__select_all(handle)
      ensure
         count > 0 implies has_selection
         --IMPROVE: new Cursorposition undefined
      end
   
   
   unselect_all is
      -- remove the selection
      require
         not is_zombie
      do
         efox_text_base__unselect_all(handle)
      ensure
         not has_selection       
         old text.is_equal(text)
      end
   
   
   select_range (pos: INTEGER; len: INTEGER) is
         -- set the selection to a range of the text [pos ... pos+len)
      require
         not is_zombie
         pos >= 1
         valid_position(pos)
      do
         efox_text_base__select_range(handle, pos, len)
      ensure
         len > 0 implies has_selection
      end
   
    
   is_position_selected (pos: INTEGER) : BOOLEAN is
         -- test if the position is selected
      require
         not is_zombie
         valid_position(pos)
      do
         result := efox_text_base__is_position_selected(handle, pos)
      ensure
         result = true implies has_selection
      end
   

feature {ANY}  -- event handling
   
   
   on_changed: ROUTINE[ANY,TUPLE] is
         -- This agent is called every time the text is changed
      require
	 not is_zombie
      do
         result := find_callback_agent(On_change_event)
      end
   
   
   set_on_changed(action: ROUTINE[ANY,TUPLE]) is
         -- Set the 'on_changed' event handler
      require
	 not is_zombie
      do
         set_callback_agent(On_change_event, action)
      end
   
   
feature {NONE} -- private C binding
   
   
   efox__callback_changed(what: INTEGER; state: BOOLEAN; obj: ANY) is
      local
         a: PROCEDURE[ANY,TUPLE]
      do
         a := find_callback_agent(On_change_event)
         if a /= Void then
            a.call([])
         end
      end
   
   -- visible size
   
   efox_text_base__visible_width(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_text_base__visible_height(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_text_base__set_visible_size(hnd: POINTER; w,h: INTEGER) is
      external "C"
      end   
   
   -- visiual appearance properties   
   
   efox_text_base__font(hnd: POINTER) : AGUI_FONT is
      external "C"
      end
   
   efox_text_base__set_font(hnd,fhnd: POINTER) is
      external "C"
      end
   
   efox_text_base__text_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_text_base__set_text_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end
   
   efox_text_base__selected_back_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_text_base__set_selected_back_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end
   
   efox_text_base__selected_text_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_text_base__set_selected_text_color(hnd: POINTER; col: INTEGER)  is
      external "C"
      end
   
   -- public features   
   
   efox_text_base__overstrike_mode (hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_text_base__set_overstrike_mode (hnd: POINTER; flag: BOOLEAN) is
      external "C"
      end
   
   efox_text_base__editable (hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_text_base__set_editable (hnd: POINTER; flag: BOOLEAN) is
      external "C"
      end
   
   efox_text_base__text (hnd: POINTER) : POINTER is
      external "C"
      end
   
   efox_text_base__set_text (hnd,str: POINTER) is
      external "C"
      end
   
   efox_text_base__count(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   -- position manipulation   
   
   efox_text_base__is_position_visible(hnd: POINTER; pos: INTEGER) : BOOLEAN is
      external "C"
      end
   
   efox_text_base__make_position_visible(hnd: POINTER; pos: INTEGER) is
      external "C"
      end
   
   efox_text_base__valid_position(hnd: POINTER; pos: INTEGER) : BOOLEAN is
      external "C"
      end
   
   efox_text_base__cursor_position(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_text_base__set_cursor_position(hnd: POINTER; pos: INTEGER) is
      external "C"
      end
   
   -- manipulation management
   
   efox_text_base__append(hnd, str: POINTER) is
      external "C"
      end
         
   efox_text_base__delete(hnd: POINTER; pos,len: INTEGER) is
      external "C"
      end   
   
   efox_text_base__insert(hnd: POINTER; pos: INTEGER; str: POINTER) is
      external "C"
      end
         
   efox_text_base__insert_substring(hnd: POINTER; pos: INTEGER; str: POINTER; a,e: INTEGER) is
      external "C"
      end
            
   efox_text_base__extract_in(hnd: POINTER; pos,len: INTEGER) : POINTER is
      external "C"
      end
   
   -- selection management   
   
   efox_text_base__has_selection (hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_text_base__selection_begin_pos (hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_text_base__selection_end_pos (hnd: POINTER) : INTEGER is
      external "C"
      end
         
   efox_text_base__selection(hnd: POINTER) : POINTER is
      external "C"
      end
      
   efox_text_base__select_all(hnd: POINTER) is
      external "C"
      end
   
   efox_text_base__unselect_all(hnd: POINTER) is
      external "C"
      end
      
   efox_text_base__select_range(hnd: POINTER; pos,len: INTEGER) is
      external "C"
      end
   
   efox_text_base__is_position_selected(hnd: POINTER; pos: INTEGER) : BOOLEAN is
      external "C"
      end
   
end
   
