class  AGUI_TEXT_FIELD
   
   
--  A text field is a single-line text entry widget.
--  The text field widget supports clipboard for cut-and-paste
--  operations.
--  Text input may be constrained to a certain format; the built-in
--  capabilities support integer and real number entry constraints.
--  During text entry, the text field invoke the 'on_changed' agent
--  if the text changed and the 'on_command' agent if ENTER is pressed.
--
--  The index position in the text field starts with index '0' !
--  This is contrary to the 'STRING' and most other eiffel  classes.
--
--  IMPROVE:  no verification mode at the moment look at TK to add this
--  IMPROVE:  'is_position_visible' nicht implementiert
--  IMPROVE:  maximum_size   
   
   
inherit
   AGUI_TEXT_BASE
      redefine 	 
         efox__callback_command
      end
   AGUI_FRAME_MIXIN
      redefine mixin_window
      end
   AGUI_PACKER_MIXIN
      redefine mixin_window
      end
   
   
   
creation
   {ANY} make
   
   
feature {NONE} -- constructor

   
   make (p: AGUI_COMPOSITE;  n: STRING) is
      require
         p /= Void and n /= Void
      do
         handle := efox_text_field__make(Current,p.handle, n.to_external, $temp_px)
         efox__window_hash_table.put(Current, temp_px)
      end
   

feature {ANY} -- text justification
   
   Justify_center_x : INTEGER is 0
   Justify_left : INTEGER is 32768
   Justify_right : INTEGER is 65536
   Justify_horizontal_apart : INTEGER is 98304
   -- at the moment only Justify_left and Justify_right are valid values
   
   Justify_center_y : INTEGER is 0   
   Justify_top : INTEGER is 131072
   Justify_bottom  : INTEGER is 262144
   Justify_vertical_apart : INTEGER is 393216
   
   
   justify : INTEGER is
         -- Current text justification.
      require
         not is_zombie
      do
         result := efox_text_field__justify(handle)
      end
   
   
   set_justify (n: INTEGER) is
         -- Set justification mode.
      require
         not is_zombie
      do
         efox_text_field__set_justify(handle, n)
      ensure
         justify = n
      end
   
   
feature {ANY} -- input mode
   
   
   Mode_string : INTEGER is 0 
         -- Input can be any text string
   
   Mode_password : INTEGER is 1
         -- Input can be any text string, but it is hiden
         -- from display.
   
   Mode_integer : INTEGER is 2
         -- Input can only be an integer number.
   
   Mode_real : INTEGER is 3
         -- Input can only be a real number.
   
   
   mode : INTEGER is
         -- Current input mode   
      require
         not is_zombie
      do
         result := efox_text_field__mode(handle)
      end
      
   
   set_mode (new_mode : INTEGER) is
         -- Set input mode
      require
         not is_zombie
         new_mode.in_range(Mode_string, Mode_real)
      do
         efox_text_field__set_mode(handle, new_mode)
      ensure
         mode = new_mode
      end
   
   
feature {ANY} -- events handling
   
   
   on_command: ROUTINE[ANY,TUPLE] is
         -- This agent is called every time when ENTER is pressed
         -- inside the widget.
      require
	 not is_zombie
      do
         result := find_callback_agent(On_command_event)
      end
   
   
   set_on_command(action: ROUTINE[ANY,TUPLE]) is
         -- Sets the 'on_command' agent.
      require
	 not is_zombie
      do
         set_callback_agent(On_command_event, action)
      end
   
   
feature {ANY}  -- mixin configure
   
   
   mixin_window : AGUI_WINDOW is
	 -- redefine the mixin window references to this window
      do
	 result := Current
      end
   
   
feature {NONE} -- private C binding
   
   
   efox__callback_command(what: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 a: PROCEDURE[ANY,TUPLE]
      do
	 a := find_callback_agent(On_command_event)
	 if a /= Void then
	    a.call([])
	 end
      end
      
   efox_text_field__make(this:like Current; hnd,n,keyptr: POINTER) : POINTER is
      external "C"
      end   
   
   efox_text_field__set_justify(hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_text_field__justify(hnd: POINTER) : INTEGER is
      external "C"      
      end
   
   efox_text_field__set_mode(hnd: POINTER; n: INTEGER) is
      external "C"
      end   
   
   efox_text_field__mode(hnd: POINTER) : INTEGER is
      external "C"
      end   
   
end
