class  AGUI_TOGGLE_BUTTON
   
inherit
   AGUI_LABEL_BASE
      redefine
	 efox__callback_command
      end          
   
   
creation
   {ANY} make
   
   
feature {NONE} -- constructor, not for reinitialization
   
   
   make (p: AGUI_COMPOSITE;  n: STRING) is
      require
	 p /= Void and n /= Void
      do
	 handle := efox_toggle_button__make(Current,p.handle,n.to_external,$temp_px)
	 efox__window_hash_table.put(Current, temp_px)
      end
   
   
feature {ANY}  -- event handler
   
   
   on_command: PROCEDURE[ANY,TUPLE[BOOLEAN]] is
	 -- Agent that is called when the user clicks the button.
	 -- The argument is 'true' if the button is now in the
	 -- alternate state.
      require
	 not is_zombie
      do
	 result ?= find_callback_agent(On_command_event)
      end
   
   
   set_on_command (action: PROCEDURE[ANY,TUPLE[BOOLEAN]]) is
	 -- Sets the agent for the click action.
      require
	 not is_zombie
      do
	 set_callback_agent(On_command_event, action)
      end
   
   
feature {ANY}  -- public properties
   
   
   alternate_state : BOOLEAN is
	 -- Test if button is in alternate state.
      require
	 not is_zombie
      do
	 result := efox_toggle_button__alternate_state(handle)
      end
   
   
   set_alternate_state (flag: BOOLEAN) is
	 -- Set button to normal or alternate state.
      require
	 not is_zombie
      do
	 efox_toggle_button__set_alternate_state(handle,flag)
      end
	
   
   alternate_icon : AGUI_ICON is
	 -- Gets the icon that is displayed when button is in
	 -- the alternate state.
      require
	 not is_zombie
      do
	 result := efox_toggle_button__alternate_icon(handle)
      end
   
   
   set_alternate_icon (new_value: AGUI_ICON) is
	 -- Sets the icon that is displayed when button is in
	 -- the alternate state.
      require
	 not is_zombie
	 new_value /= Void implies not new_value.is_zombie
      local
	 p : POINTER
      do
	 if new_value /= Void then p := new_value.handle end
	 efox_toggle_button__set_alternate_icon(handle, p)
      end
   
   
   alternate_text : STRING is
	 -- Gets the text that is displayed when button is in
	 -- the alternate state.
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_toggle_button__alternate_text(handle))
      ensure
	 result /= Void
      end
   
   
   set_alternate_text (new_value: STRING) is
	 -- Sets the text that is displayed when button is in
	 -- the alternate state.
      require
	 not is_zombie
	 new_value /= Void
      do
	 efox_toggle_button__set_alternate_text(handle, new_value.to_external)
      end
   
   
   toolbar_style : BOOLEAN is
	 -- Test if button is drawn in toolbar style.
      require
	 not is_zombie
      do
	 result := efox_toggle_button__toolbar_style(handle)
      end
   
   
   set_toolbar_style (flag: BOOLEAN) is
	 -- Set/Remove toolbar style for drawing.
      require
	 not is_zombie
      do
	 efox_toggle_button__set_toolbar_style(handle, flag)
      end
   
   
feature {ANY} --  tool and help tips for the alternate state
   
   
   set_alternate_help_text (str: STRING) is
	 -- Sets the help text message.
      require
	 not is_zombie
	 str /= Void
      do	 
	 efox_toggle_button__set_alternate_help_text(handle, str.to_external)
      end
   
   
   alternate_help_text : STRING is
	 -- Returns a new string with the help text message.
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_toggle_button__alternate_help_text(handle))
      ensure
	 result /= Void
      end
   
   
   set_alternate_tooltip_text (str: STRING) is
	 -- Sets the tool tip message.
      require
	 not is_zombie
	 str /= Void
      do
	 efox_toggle_button__set_alternate_tooltip_text(handle, str.to_external)
      end
   
   
   alternate_tooltip_text : STRING is
	 -- Returns a new string with the tooltip message.
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_toggle_button__alternate_tooltip_text(handle))
      end
   

feature {NONE} -- private C binding
   
   
   efox__callback_command(idx: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 a: PROCEDURE[ANY,TUPLE[BOOLEAN]]
      do
	 a ?= find_callback_agent(On_command_event)
	 if a /= Void then
	    a.call([state])
	 end
      end
   
   efox_toggle_button__make(this:like Current; hnd,n,keyptr: POINTER) : POINTER is
      external "C"
      end   
   
   efox_toggle_button__set_toolbar_style(hnd: POINTER; fl: BOOLEAN) is
      external "C"
      end
   
   efox_toggle_button__toolbar_style(hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_toggle_button__set_alternate_help_text(hnd,str: POINTER) is
      external "C"
      end
         
   efox_toggle_button__alternate_help_text(hnd: POINTER) : POINTER is
      external "C"
      end
   
   efox_toggle_button__set_alternate_tooltip_text(hnd,str: POINTER) is
      external "C"
      end
   
   efox_toggle_button__alternate_tooltip_text(hdn: POINTER) : POINTER is
      external "C"
      end
      
   efox_toggle_button__set_alternate_state(hnd: POINTER; fl: BOOLEAN) is
      external "C"
      end
	 
   efox_toggle_button__alternate_state(hnd: POINTER) : BOOLEAN is
      external "C"
      end   
      
   efox_toggle_button__set_alternate_text(hnd,val: POINTER) is
      external "C"
      end
   
   efox_toggle_button__alternate_text(hnd: POINTER) : POINTER is
      external "C"
      end   
   
   efox_toggle_button__set_alternate_icon(hnd, ihnd: POINTER) is
      external "C"
      end
   
   efox_toggle_button__alternate_icon(hnd: POINTER) : AGUI_ICON is
      external "C"
      end
   
   
end
