class AGUI_TOPLEVEL_WINDOW
   
   
inherit
   AGUI_SHELL_WINDOW
      end
   AGUI_PACKER_MIXIN
      end
   
   
creation
   {ANY} make
   
   
feature {ANY} -- position of window
   
   
   Placement_default   : INTEGER is 1
   Placement_visible   : INTEGER is 2
   Placement_cursor    : INTEGER is 3
   Placement_owner     : INTEGER is 4
   Placement_screen    : INTEGER is 5
   Placement_maximized : INTEGER is 6
   Placement_fixed     : INTEGER is 7
	 -- don't change the current position of the window 
   
   
	 -- Note: You can always use the negative value than the behaviour
	 -- change to the following:
	 --
	 -- Check the "default_settings" from application if there is a
	 -- saved position for this window under the "[Windows]" section.
	 -- If yes than use the position specified there, else use 
	 -- placement screen.
	 -- When the window is closed the position is recorded in the
	 -- settings file.
   
   
   place (kind: INTEGER) is
	 -- move/size window to a predefined position
      require
	 kind.abs.in_range(Placement_default, Placement_fixed)
      do
	 efox_toplevel_window__place(handle, kind)
      end
   
   activate is
      do
	 efox_toplevel_window__activate(handle)
      end
   
   
feature {ANY} -- event handling
   
   
   on_query_destroy : FUNCTION[ANY,TUPLE,BOOLEAN]
	 -- Agent that is called every time the user wants to close
	 -- the window. This agent can give a veto to this action by
	 -- returning 'false'. If 'true' is returned or no agent 
	 -- is specified the window will be destroyed.
   
   set_on_query_destroy (action: FUNCTION[ANY,TUPLE,BOOLEAN]) is
	 -- Set the agent for the 'on_query_close' action
      require
	 not is_zombie
      do
	 on_query_destroy := action
      end
   
   
feature {ANY} -- titles
   
   
   title : STRING is
	 -- Return title as new string.
      do
	 create result.from_external_copy(efox_toplevel_window__title(handle))
      ensure
	 result /= Void
      end
   

   set_title (str: STRING) is
	 -- Sets the windows title.
      require
	 str /= Void
      do
	 efox_toplevel_window__set_title(handle,str.to_external)
      end     
   
   
feature {ANY} -- icons and iconify
   
   
   normal_icon : AGUI_ICON is
	 -- Return the normal icon for the window.
      require
	 not is_zombie
      do
	 result := efox_toplevel_window__normal_icon(handle)
      end
   
   
   set_normal_icon (new_value: AGUI_ICON) is
	 -- Set the normal icon for the window.
      require
	 not is_zombie
	 new_value /= Void implies not new_value.is_zombie
      do
	 if new_value /= Void then
	    efox_toplevel_window__set_normal_icon(handle, new_value.handle)
	 else
	    efox_toplevel_window__set_normal_icon(handle, default_pointer)
	 end
      ensure
	 normal_icon = new_value
      end
   
   
   mini_icon : AGUI_ICON is
	 -- Gets the small sized icon for the window.
      require
	 not is_zombie
      do
	 result := efox_toplevel_window__mini_icon(handle)
      end
   
   
   set_mini_icon (new_value: AGUI_ICON) is
	 -- Sets the small sized icon for the window.
      require
	 not is_zombie
	 new_value /= Void implies not new_value.is_zombie
      do 
	 if new_value /= Void then
	    efox_toplevel_window__set_mini_icon(handle, new_value.handle)
	 else
	    efox_toplevel_window__set_mini_icon(handle, default_pointer)
	 end
      ensure
	 mini_icon = new_value
      end
   
   
   iconify_state : BOOLEAN is
	 -- Test if window is in icon mode.
      require
	 not is_zombie
      do
	 result := efox_toplevel_window__iconify_state(handle)
      end
   
   
   set_iconify_state (new_value: BOOLEAN) is
	 -- Set the window to icon mode or restore from this mode.
	 -- The windowing system is not forced to obey this command.
      require
	 not is_zombie
      do
	 efox_toplevel_window__set_iconify_state(handle, new_value)
      end
      
   
feature {ANY} -- decorations   
   
   
   -- Change style setting; note, under Windows, if we want a minimize,
   -- maximize, or close button, we also need a window menu style as well.
   -- Also, if you want a title, you will need a border.
   -- The system menu is always visible if one of the min/max/close 
   -- buttons is visible.
   
   
   Decoration_title : INTEGER is 2
	 -- Show the window title
   
   Decoration_minimize : INTEGER is 4
	 -- Show minimmize button
   
   Decoration_maximize : INTEGER is 8
	 -- Show maximize button
   
   Decoration_close : INTEGER is 16
	 -- Show close button
   
   Decoration_border : INTEGER is 32
	 -- Show window border 
   
   Decoration_resize : INTEGER is 64
	 -- Window is resizeable
   
   Decoration_menu : INTEGER is 128
	 -- Show window system menu
   
   Decoration_all : INTEGER is 254
	 -- Window with all features
   
   Decoration_none  : INTEGER is 0   
	 -- Borderless window
   
   Decoration_dialog : INTEGER is 98
	 -- Typical modal dialog
   
   
   set_decoration (new_value: INTEGER) is
	 -- Sets the decoration. Note that not all combinations
	 -- have the correct result.
      require
	 not is_zombie
      do
	 efox_toplevel_window__set_decoration(handle,new_value)
      end
   
   
   decoration : INTEGER is
	 -- Return current decoration
      require
	 not is_zombie
      do
	 result := efox_toplevel_window__decoration(handle)
      end
   
   
feature {NONE} -- constructor
   
   
   make (name: STRING) is
	 -- create a new toplevel window
      require
	 name /= Void
      do
	 handle := efox_toplevel_window__make(Current,default_pointer,name.to_external,$temp_px)
	 efox__window_hash_table.put(Current, temp_px)
	 create accelerator_table.make(handle)
      end
   
   
feature {ANY}  -- mixin configure
   
   
   mixin_window : AGUI_WINDOW is
	 -- redefine the mixin window references to this window
      do
	 result := Current
      end
   
   
feature {NONE} -- private C binding   
   
   
   efox__callback_close : BOOLEAN is
      do
	 result := true
	 if on_query_destroy /= Void then
	    result := on_query_destroy.item([])
	 end
	 if result then
	    application.add_working_task(agent efox_safe_destroy)
	 end
      end
   
   efox_safe_destroy is
      do
	 if not is_zombie then
	    destroy
	 end
      end
   
   -- position of window
   
   efox_toplevel_window__place (hnd: POINTER; k: INTEGER) is
      external "C"
      end
   
   efox_toplevel_window__activate(hnd: POINTER) is
      external "C"
      end
   
      -- titles   
   
   efox_toplevel_window__set_title (hnd,s: POINTER) is
      external "C"
      end
   
   efox_toplevel_window__title (hnd: POINTER): POINTER is
      external "C"
      end
   
   -- icons and iconify   
   
   efox_toplevel_window__normal_icon(hnd: POINTER): AGUI_ICON is
      external "C"
      end
   
      efox_toplevel_window__set_normal_icon(hnd,ic: POINTER) is
      external "C"
      end
   
   efox_toplevel_window__mini_icon(hnd: POINTER): AGUI_ICON is
      external "C"
      end
   
   efox_toplevel_window__set_mini_icon(hnd,ic: POINTER) is
      external "C"
      end
   
   efox_toplevel_window__iconify_state(hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_toplevel_window__set_iconify_state(hnd: POINTER; fl: BOOLEAN) is
      external "C"
      end
   
 -- decorations   
   
   efox_toplevel_window__set_decoration(hnd: POINTER; nv: INTEGER) is
      external "C"
      end
   
   efox_toplevel_window__decoration(hnd: POINTER) : INTEGER is
      external "C"
      end
   
 -- constructor   
   
   efox_toplevel_window__make(this:like Current; hnd, n, keyptr: POINTER) : POINTER is
      external "C"
      end
   
end
   
