deferred class AGUI_TREE_BASE
   
--
--  Base class for all tree widgets
--
   
inherit
   AGUI_CONTROL
      redefine
	 efox__callback_command,
	 efox__callback_changed
      end
   
   
feature {ANY}  -- styles
   
   
   Tree_show_lines : INTEGER is 8
	 -- Lines shown
   
   Tree_show_boxes : INTEGER is 16
	 -- Boxes to expand shown
   
   Tree_show_root_boxes : INTEGER is 32
	 -- Root Boxes are shown
   
   
   style : INTEGER is
	 -- Return tree style.
      require
         not is_zombie
      do
	 result := efox_tree_base__style(handle)
      end
   
   
   set_style (n: INTEGER) is
	 -- Change tree style.
      require
         not is_zombie
      do
	 efox_tree_base__set_style(handle,n)
      end


feature {ANY}  -- appearance
   
   
   font : AGUI_FONT is
	 -- Default font for items.
      require
         not is_zombie
      do
         result := efox_tree_base__font(handle)
      ensure
         result /= Void
      end
   
   
   set_font (f: AGUI_FONT) is
	 -- Set default font for items.
      require
         not is_zombie
         f /= Void
         not f.is_zombie
      do
         efox_tree_base__set_font(handle, f.handle)
      end
   
   
   text_color : AGUI_COLOR is
	 -- Default text color for items.
      require
         not is_zombie
      do
         result.from_integer(efox_tree_base__text_color(handle))
      end
   
   set_text_color (col: AGUI_COLOR) is
	 -- Set default text color for items.
      require
         not is_zombie
      do
         efox_tree_base__set_text_color(handle, col.to_integer)
      end
   
   
   selected_background_color : AGUI_COLOR is
	 -- Default background color for selected items.
      require
         not is_zombie
      do
         result.from_integer(efox_tree_base__sel_back_color(handle))
      end
   
   
   set_selected_background_color (col: AGUI_COLOR) is
	 -- Default background color for selected items.
      require
         not is_zombie
      do
         efox_tree_base__set_sel_back_color(handle,col.to_integer)
      end
   
   
   selected_text_color : AGUI_COLOR is
	 -- Default color for selected items.
      require
         not is_zombie
      do
         result.from_integer(efox_tree_base__sel_text_color(handle))
      end
   
   
   set_selected_text_color (col: AGUI_COLOR) is
	 -- Default color for selected items.
      require
         not is_zombie
      do
         efox_tree_base__set_sel_text_color(handle,col.to_integer)
      end
   
   
   indent : INTEGER is
	 -- Parent-child indent amount.
      require
         not is_zombie
      do
         result := efox_tree_base__indent(handle)
      end
   
   
   set_indent (n: INTEGER) is
	 -- Change parent-child indent amount.
      require
         not is_zombie
      do
         efox_tree_base__set_indent(handle,n)
      end      
   
   
feature  {ANY} -- current item
      
   
   current_item : AGUI_TREE_BASE_ITEM is
	 -- Return current item or Void.
      require
	 not is_zombie
      do
	 result := efox_tree_base__current_item(handle)
      end
   
   
   set_current_item (item: AGUI_TREE_BASE_ITEM) is
	 -- Change current item.
      require
	 not is_zombie
      do
	 efox_tree_base__set_current_item(handle, item.handle)
      end
   
   
feature {ANY}  -- misc
   
   
   clear is
	 -- Remove all items from tree.
      require
	 not is_zombie
      do
	 efox_tree_base__clear(handle)
      end
   
   
   count : INTEGER is
	 -- Return number of items in tree.
      obsolete "Not yet implemented"
      require
	 not is_zombie
      do
	 result := efox_tree_base__count (handle)
      end
   
   
   root_count : INTEGER is
      -- Return number of root items.
      require
	 not is_zombie
      do
	 result := efox_tree_base__root_count(handle)
      end
   
   
   root(idx: INTEGER): AGUI_TREE_BASE_ITEM is
	 -- Return idx'th root item in this tree.
      require
	 not is_zombie
	 idx.in_range(1,root_count)
      do
	 result := efox_tree_base__root (handle,idx)
      end


feature {ANY}  -- misc
   
   
   append_in (c: COLLECTION[AGUI_TREE_ITEM]; selected_only: BOOLEAN) is
	 -- Travers the tree and add all (selected) items to the collection.
	 -- We use the preorder traversion strategy.
      require
	 c /= Void
	 not is_zombie
      local
	 i,n: INTEGER
      do
	 from
	    i := 1
	    n := root_count
	 until
	    i > n
	 loop
	    if selected_only then
	       append_recursive_sel(c, root(i))
	    else
	       append_recursive(c, root(i))
	    end
	    i := i + 1
	 end
      end
   
   
   do_all (action: ROUTINE[ANY,TUPLE[AGUI_TREE_BASE_ITEM]]) is
      local
	 i,n: INTEGER
	 li: AGUI_TREE_BASE_ITEM
      do
	 from
	    i := 1
	    n := root_count
	 until
	    i > n
	 loop
	    li := root(i)
	    if li /= Void then
	       do_all_recursive(li,action)
	    end
	    i := i + 1
	 end
      end
   
   
   find_data (what: ANY) : AGUI_TREE_BASE_ITEM is
      local
	 i,n: INTEGER
	 ti: AGUI_TREE_BASE_ITEM
      do
	 from
	    i := 1
	    n := root_count
	 until
	    i > n or else result /= Void
	 loop
	    ti := root(i)
	    if ti /= Void then
	       result := find_data_recursive(ti,what)
	    end
	    i := i + 1
	 end
      end
   
   
   fast_find_data (what: ANY) : AGUI_TREE_BASE_ITEM is
      local
	 i,n: INTEGER
	 ti: AGUI_TREE_BASE_ITEM
      do
	 from
	    i := 1
	    n := root_count
	 until
	    i > n or else result /= Void
	 loop
	    ti := root(i)
	    if ti /= Void then
	       result := fast_find_data_recursive(ti,what)
	    end
	    i := i + 1
	 end
      end
   
      
feature {ANY}  -- sorting
   
   
   sort_root_items is
	 -- Sort root items
      do
	 efox_tree_base__sort_root_items(handle)
      end
   
   
   sort_all_items is
	 -- Sort all items
      do
	 efox_tree_base__sort_all_items(handle)
      end
   
   
   set_ascending_sorter is
	 -- Set agent which sort in ascending 'text' order.
	 -- The sort is case sensitive.
      
      do
	 efox_tree_base__set_ascending_sorter(handle)
      end
   
   
   set_descending_sorter is
	 -- Set agent which sort in descending 'text' order.
	 -- The sort is case sensitive.
      do
	 efox_tree_base__set_descending_sorter(handle)
      end



feature {ANY}  -- event handlers
   
   
   set_on_changed(a : PROCEDURE[ANY,TUPLE[AGUI_TREE_BASE_ITEM]]) is
      do
	 set_callback_agent(On_change_event, a)
      end
   
   
   on_changed : PROCEDURE[ANY,TUPLE[AGUI_TREE_BASE_ITEM]] is
      do
	 result ?= find_callback_agent(On_change_event)
      end
   
   
   set_on_command(a : PROCEDURE[ANY,TUPLE[AGUI_TREE_BASE_ITEM]]) is
      do
	 set_callback_agent(On_command_event, a)
      end
   
   
   on_command : PROCEDURE[ANY,TUPLE[AGUI_TREE_BASE_ITEM]] is
      do
	 result ?= find_callback_agent(On_command_event)
      end
   
   
feature {NONE} -- private implementation
   
   
   do_all_recursive(li: AGUI_TREE_BASE_ITEM; action: ROUTINE[ANY,TUPLE[AGUI_TREE_BASE_ITEM]]) is
      require
	 li /= Void
      local
	 iter: AGUI_TREE_BASE_ITEM
      do
	 action.call([li])
	 from
	    iter := li.first_item
	 until
	    iter = Void 
	 loop
	    do_all_recursive(iter,action)
	    iter := iter.next_item
	 end	 
      end
   
   
   append_recursive (c: COLLECTION[AGUI_TREE_BASE_ITEM]; item: AGUI_TREE_BASE_ITEM) is
      local
	 p : AGUI_TREE_BASE_ITEM
      do
	 c.add_last(item)
	 from
	    p := item.first_item
	 until
	    p = Void
	 loop
	    append_recursive (c, p)
	    p := p.next_item
	 end
      end
   
   
   append_recursive_sel (c: COLLECTION[AGUI_TREE_BASE_ITEM]; item: AGUI_TREE_BASE_ITEM) is
      local
	 p : AGUI_TREE_BASE_ITEM
      do
	 if item.selected then
	    c.add_last(item)
	 end
	 from
	    p := item.first_item
	 until
	    p = Void
	 loop
	    append_recursive_sel (c, p)
	    p := p.next_item
	 end
      end
   
   
   find_data_recursive(item: AGUI_TREE_BASE_ITEM; what: ANY) : AGUI_TREE_BASE_ITEM is
      local
	 p : AGUI_TREE_BASE_ITEM
      do
	 if what.is_equal(item.data) then
	    result := item
	 else
	    from
	       p := item.first_item
	    until
	       p = Void or else result /= Void
	    loop
	       result := find_data_recursive (p, what)
	       p := p.next_item
	    end
	 end
      end
   
   fast_find_data_recursive(item: AGUI_TREE_BASE_ITEM; what: ANY) : AGUI_TREE_BASE_ITEM is
      local
	 p : AGUI_TREE_BASE_ITEM
      do
	 if what = item.data then
	    result := item
	 else
	    from
	       p := item.first_item
	    until
	       p = Void or else result /= Void
	    loop
	       result := fast_find_data_recursive (p, what)
	       p := p.next_item
	    end
	 end
      end
   
   
feature {AGUI_TREE_BASE_ITEM, AGUI_TREE_BASE}
   
   
   item_dict : DICTIONARY[AGUI_TREE_BASE_ITEM, INTEGER]
	 -- Dictionary with all items. We must hold the
	 -- items to protect them from garbage collection.
   
   
   
feature {NONE} -- private C binding   
   
   
   efox__callback_changed(what: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 a: ROUTINE[ANY, TUPLE[AGUI_TREE_BASE_ITEM]]
	 i: AGUI_TREE_BASE_ITEM
      do
	 a := on_changed
	 if a /= Void then
	    i ?= obj
	    a.call([i])
	 end
      end
   
   
   efox__callback_command(idx: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 a: ROUTINE[ANY, TUPLE[AGUI_TREE_BASE_ITEM]]
	 i: AGUI_TREE_BASE_ITEM
      do
	 a := on_command
	 if a /= Void then
	    i ?= obj
	    a.call([i])
	 end
      end      
   
   
   -- styles
   
   efox_tree_base__set_style(hnd: POINTER; n: INTEGER) is
      external "C"
      end     
      
   efox_tree_base__style(hnd: POINTER) : INTEGER is
      external "C"
      end     

   -- appearance
   
   efox_tree_base__font(hnd: POINTER) : AGUI_FONT is
      external "C"
      end
   
   efox_tree_base__set_font(hnd,fhnd: POINTER) is
      external "C"
      end
   
   efox_tree_base__text_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_tree_base__set_text_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end   
   
   efox_tree_base__sel_back_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_tree_base__set_sel_back_color(hnd: POINTER; col: INTEGER) is
      external "C"            
      end      
   
   efox_tree_base__sel_text_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_tree_base__set_sel_text_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end

   efox_tree_base__set_indent (hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_tree_base__indent (hnd: POINTER) : INTEGER is
      external "C"
      end
   
   -- current item
   
   efox_tree_base__set_current_item(hnd, ihnd: POINTER) is
      external "C"
      end
   
   efox_tree_base__current_item(hnd: POINTER) : AGUI_TREE_BASE_ITEM is
      external "C"
      end
   
   --  misc
   
   efox_tree_base__clear (hnd: POINTER) is
      external "C"
      end
   
   efox_tree_base__count (hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_tree_base__root (hnd: POINTER; idx: INTEGER) : AGUI_TREE_BASE_ITEM is
      external "C"
      end
   
   efox_tree_base__root_count (hnd: POINTER) : INTEGER is
      external "C"
      end
      
   -- sorting
   
   efox_tree_base__set_descending_sorter (hnd: POINTER) is
      external "C"
      end
   
   efox_tree_base__set_ascending_sorter (hnd: POINTER) is
      external "C"
      end
   
   efox_tree_base__sort_root_items (hnd: POINTER) is
      external "C"
      end
   
   efox_tree_base__sort_all_items (hnd: POINTER) is
      external "C"
      end
   
end
