deferred class  AGUI_TREE_BASE_ITEM
   

feature {ANY}  -- public attributes
   
   
   text : STRING is
	 -- get a new string with text of this item
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_tree_item__text(handle))
      ensure
	 text /= Void
      end
   
   set_text (s: STRING) is
	 -- sets the text for this item
      require
	 not is_zombie
	 s /= Void
      do
	 efox_tree_item__set_text(handle,s.to_external)
      end
   
   ----------
   
   opened_icon : AGUI_ICON is
	 -- get the open icon or 'Void'
      require
	 not is_zombie
      do
	 result := efox_tree_item__opened_icon(handle)
      end
   
   set_opened_icon (ic: AGUI_ICON) is
	 -- sets the open icon or removes the current if 'ic = Void'
      require
	 not is_zombie
	 ic /= Void implies not ic.is_zombie
      do
	 if ic /= Void then
	    efox_tree_item__set_opened_icon(handle, ic.handle)
	 else
	    efox_tree_item__set_opened_icon(handle, default_pointer)
	 end
      end
   
   ----------
   
   closed_icon : AGUI_ICON is
	 -- get the close icon or 'Void'
      require
	 not is_zombie
      do
	 result := efox_tree_item__closed_icon(handle)
      end
   
   set_closed_icon (ic: AGUI_ICON) is
	 -- sets the close icon or removes the current if 'ic = Void'
      require
	 not is_zombie
	 ic /= Void implies not ic.is_zombie
      do
	 if ic /= Void then
	    efox_tree_item__set_closed_icon(handle, ic.handle)
	 else
	    efox_tree_item__set_closed_icon(handle, default_pointer)
	 end
      end
         
   ----------
   
   data : ANY
	 -- corresponding data item
   
   set_data (d: ANY) is
	 -- set the data item
      require
	 not is_zombie
      do
	 data := d
      end
   
   ----------
   
   enabled : BOOLEAN is
	 --TODO
      require
	 not is_zombie
      do
	 result := efox_tree_item__enabled (handle)
      end
   
   set_enabled (flag: BOOLEAN) is
	 --TODO
      require
	 not is_zombie
      do
	 efox_tree_item__set_enabled(handle, flag)
      end
   
   ----------
   
   collapsed : BOOLEAN is
	 --TODO
      require
	 not is_zombie
      do
	 result := efox_tree_item__collapsed (handle)
      end
   
   set_collapsed (flag: BOOLEAN) is
	 --TODO
      require
	 not is_zombie
      do
	 efox_tree_item__set_collapsed (handle,flag)
      end
	 
   ----------
   
   selected : BOOLEAN is
	 --TODO
      require
	 not is_zombie
      do
	 result := efox_tree_item__selected (handle)
      end
   
   set_selected (flag: BOOLEAN) is
	 --TODO
      require
	 not is_zombie
      do
	 efox_tree_item__set_selected (handle, flag)
      end
   
   ----------
   
   draggable : BOOLEAN is
	 --TODO
      require
	 not is_zombie
      do
	 result := efox_tree_item__draggable (handle)
      end
   
   set_draggable (flag: BOOLEAN) is
	 --TODO
      require
	 not is_zombie
      do
	 efox_tree_item__set_draggable (handle,flag)
      end
	 
   ----------
   
   opened : BOOLEAN is
	 --TODO
      require
	 not is_zombie
      do
	 result := efox_tree_item__opened (handle)
      end
   
   set_opened (flag: BOOLEAN) is
	 --TODO
      require
	 not is_zombie
      do
	 efox_tree_item__set_opened (handle,flag)
      end
	 
   
feature {ANY}  -- queries
   
   
   is_zombie : BOOLEAN is
	 -- test if this item is already destroyed, because it is
	 -- explicitly removed or the list is destroyed
      do
	 result := handle.is_null
      end
   
   tree_window : AGUI_TREE_BASE is
	 -- get the tree that contains this item
      require
	 not is_zombie
      do
	 result := efox_tree_item__tree_window(handle)
      end
   
   parent_item : AGUI_TREE_ITEM is
	 -- return parent
      require
	 not is_zombie
      do
	 result := efox_tree_item__parent(handle)
      end
   
   first_item : AGUI_TREE_ITEM is
	 -- return first child
      require
	 not is_zombie
      do
	 result := efox_tree_item__first(handle)
      end
   
   last_item : AGUI_TREE_ITEM is
	 -- return last child
      require
	 not is_zombie
      do
	 result := efox_tree_item__last(handle)
      end
   
   next_item : AGUI_TREE_ITEM is
	 -- return next sibling
      require
	 not is_zombie
      do
	 result := efox_tree_item__next(handle)
      end
   
   previous_item : AGUI_TREE_ITEM is
	 --return previous sibling
      require
	 not is_zombie
      do
	 result := efox_tree_item__previous(handle)
      end
   
   above_item : AGUI_TREE_ITEM is
	 --TODO
      require
	 not is_zombie
      do
	 result := efox_tree_item__above(handle)
      end
   
   below_item : AGUI_TREE_ITEM is
	 --TODO
      require
	 not is_zombie
      do
	 result := efox_tree_item__below(handle)
      end
   
   has_focus : BOOLEAN is
	 -- test if item has the keyboard focus
      require
	 not is_zombie
      do
	 result := efox_tree_item__has_focus(handle)
      end
   
   number_of_children : INTEGER is
	 -- get number of children
      require
	 not is_zombie
      do
	 result := efox_tree_item__number_of_children(handle)
      end
   
   child (no: INTEGER) : AGUI_TREE_ITEM is
	 -- get child at index 'no'
      require
	 not is_zombie
	 no.in_range(1,number_of_children)
      do
	 result := efox_tree_item__child(handle,no)
      end   
   
     
   is_visible : BOOLEAN is
	 -- test if item is visible
      require	 
	 not is_zombie
      do
	 result := efox_tree_item__is_visible(handle)
      end
   
   is_current : BOOLEAN is
	 -- test if this is the current item in the list
      require	 
	 not is_zombie
      do
	 result := efox_tree_item__is_current(handle)
      end
   
   is_leaf : BOOLEAN is
	 -- test if this is a leaf item.
      require	 
	 not is_zombie
      do
	 result := efox_tree_item__is_leaf(handle)
      end
   
   is_child_of (other: AGUI_TREE_ITEM) : BOOLEAN is
	 -- test if item is a (indirect) child of other
      require
	 not is_zombie
	 other /= Void
	 not other.is_zombie
	 tree_window = other.tree_window
      do
	 result := efox_tree_item__is_child_of(handle,other.handle)
      end
   
   hit (x,y: INTEGER) : INTEGER is
	 -- return  item hit code: 0 outside, 1 icon, 2 text, 3 box
      require	 
	 not is_zombie
      do
	 result := efox_tree_item__hit(handle,x,y)
      end
   
   
feature {AGUI_TREE_BASE_ITEM,AGUI_TREE_BASE} -- private implementation
   
   
   handle : POINTER
	 -- pointer to FXTreeItem
   
   tree_kind : INTEGER
	 -- determine the underlying type
	 --   1  is  AGUI_TREE
	 --   2  is  AGUI_TREE_BOX
   
   
feature {ANY}  -- commands
   
   
   make_visible is
	 --scroll to make item visible
      require	 
	 not is_zombie
      do
	 efox_tree_item__make_visible(handle)
      end
   
   
   reparent (new_parent: AGUI_TREE_ITEM) is
	 -- set item as last child of 'new_parent'
      require
	 not is_zombie
	 new_parent /= Void
	 not new_parent.is_zombie
	 tree_window = new_parent.tree_window
      do
	 efox_tree_item__reparent(handle,new_parent.handle)
      end
   
   
   move_before (reference_item: AGUI_TREE_ITEM) is
	 -- moves this item before 'reference_item'
      require
	 not is_zombie
	 reference_item /= Void
	 not reference_item.is_zombie
	 tree_window = reference_item.tree_window
      do
	 efox_tree_item__move_before(handle, reference_item.handle)
      end
   
   
   move_after (reference_item: AGUI_TREE_ITEM) is
	 -- moves this item after 'reference_item'
      require
	 not is_zombie
	 reference_item /= Void
	 not reference_item.is_zombie
	 tree_window = reference_item.tree_window
      do
	 efox_tree_item__move_after(handle, reference_item.handle)
      end   
   
   
   sort_children is
	 -- Sort all childrens
      require
	 not is_zombie
      do
	 efox_tree_item__sort_childs(handle)
      end
   
   
   destroy is
	 -- delete item (and all children) from tree
      require
	 not is_zombie
      do
	 efox_tree_item__destroy(handle)
      ensure
	 is_zombie
      end
   
   
feature {NONE} -- private C binding
   
   
   efox_tree_item__tree_window(hnd: POINTER) : AGUI_TREE is
      external "C"
      end
   
   efox_tree_item__parent(hnd: POINTER) : AGUI_TREE_ITEM is
      external "C"
      end
   
   efox_tree_item__first(hnd: POINTER) : AGUI_TREE_ITEM is
      external "C"
      end
   
   efox_tree_item__last(hnd: POINTER) : AGUI_TREE_ITEM is
      external "C"
      end
   
   efox_tree_item__next(hnd: POINTER) : AGUI_TREE_ITEM is
      external "C"
      end
   
   efox_tree_item__previous(hnd: POINTER) : AGUI_TREE_ITEM is
      external "C"
      end
   
   efox_tree_item__above(hnd: POINTER) : AGUI_TREE_ITEM is
      external "C"
      end
   
   efox_tree_item__below(hnd: POINTER) : AGUI_TREE_ITEM is
      external "C"
      end
   
   efox_tree_item__number_of_children (hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_tree_item__child(hnd: POINTER; no: INTEGER) : AGUI_TREE_ITEM is
      external "C"
      end
   
   efox_tree_item__has_focus (hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_tree_item__is_visible(hnd: POINTER) : BOOLEAN is
      external "C"
      end
      
   efox_tree_item__is_current(hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_tree_item__is_leaf(hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_tree_item__hit(hnd: POINTER; x,y: INTEGER) : INTEGER is
      external "C"
      end   
   
   efox_tree_item__is_child_of(hnd, ohnd: POINTER) : BOOLEAN is
      external "C"
      end   
   
   ------  properites  ------

   efox_tree_item__draggable(ptr: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_tree_item__set_draggable(ptr: POINTER; flag: BOOLEAN) is
      external "C"
      end
   
   efox_tree_item__text(ptr: POINTER) : POINTER is
      external "C"
      end
   
   efox_tree_item__set_text(ptr,str: POINTER) is
      external "C"
      end
   
   efox_tree_item__opened_icon(ptr: POINTER) : AGUI_ICON is
      external "C"
      end
   
   efox_tree_item__set_opened_icon(ptr,ic: POINTER) is
      external "C"
      end
   
   efox_tree_item__closed_icon(ptr: POINTER)  : AGUI_ICON is
      external "C"
      end
   
   efox_tree_item__set_closed_icon(ptr,ic: POINTER) is
      external "C"
      end
   
   efox_tree_item__enabled (ptr: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_tree_item__set_enabled (ptr: POINTER; flag: BOOLEAN) is
      external "C"
      end
   
   efox_tree_item__collapsed (ptr: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_tree_item__set_collapsed (ptr: POINTER; flag: BOOLEAN) is
      external "C"
      end
   
   efox_tree_item__selected (ptr: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_tree_item__set_selected (ptr: POINTER; flag: BOOLEAN) is
      external "C"
      end
   
   efox_tree_item__opened(ptr: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_tree_item__set_opened(ptr: POINTER; flag: BOOLEAN) is
      external "C"
      end   
   
   -------- commands  ---------
   
   efox_tree_item__update (hnd: POINTER) is
      external "C"
      end
	 
   efox_tree_item__make_visible (hnd: POINTER) is
      external "C"
      end
   
   efox_tree_item__reparent(hnd, ptr: POINTER) is
      external "C"
      end   
   
   efox_tree_item__move_first_child(hnd, ptr: POINTER) is
      external "C"
      end   
   
   efox_tree_item__move_last_child(hnd, ptr: POINTER) is
      external "C"
      end   
   
   efox_tree_item__move_after(hnd, ptr: POINTER) is
      external "C"
      end   
   
   efox_tree_item__move_before(hnd, ptr: POINTER) is
      external "C"
      end   
   
   efox_tree_item__sort_childs(hnd: POINTER) is
      external "C"
      end   
   
   efox_tree_item__destroy(hnd: POINTER) is
      external "C"
      end
   
   efox__callback_destroy is
      do
	 --TODO remove from parent
	 handle := default_pointer
      end
end
   
