deferred class AGUI_WINDOW


inherit
   AGUI_GLOBALS
      end
   AGUI_INTERNAL_GC_PROTECTOR
      end
   

feature {ANY}  -- navigation in window tree
   
      
   parent_window : AGUI_WINDOW is
	 -- Return parent window
      require
	 not is_zombie
      do
	 result := efox_window__parent_window(handle)
      end
   
   
   next_window : AGUI_WINDOW is
	 -- Return next (sibling) window
	 -- Pure FOX Windows are skipped.
      require
	 not is_zombie
      do
	 result := efox_window__next_window(handle)
      end
   
   
   first_child_window : AGUI_WINDOW is
	 -- Return first child window, if any   
	 -- Pure FOX Windows are skipped.
      require
	 not is_zombie
      do
	 result := efox_window__first_child_window(handle)
      end
   
   
   shell_window : AGUI_SHELL_WINDOW is
	 -- Return the shell window.
      do
	 result := efox_window__shell_window(handle)
      end
   

feature {ANY}  -- window naming
   
   
   window_name : STRING is
	 -- Name of window.
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_window__window_name(handle))
      ensure
	 result /= Void	 
      end
   
   
   window_path_name : STRING is
	 -- Complete path name of window.
      require
	 not is_zombie
      do
	 create result.from_external_copy(efox_window__window_path_name(handle))
      ensure
	 result /= Void
      end
   
   
   find_window_by_name (name: STRING) : AGUI_WINDOW is
	 -- Find child window with given name.
	 -- We do a recursive deepth first order search.
      require
	 not is_zombie
	 name /= Void
      do
	 result := efox_window__find_window_by_name(handle, name.to_external)
      end
      
   
feature {ANY} -- metadata
   
   
   is_zombie : BOOLEAN is
	 -- Test if window is already deleted but not yet
	 -- garbage collected.
      do
	 result := handle = default_pointer
      end
   
   
   window_type_id : INTEGER is
	 -- Return the unique type id for the window.
	 -- Every Eiffel-FOX window has an own assigned id.
	 -- See 'AGUI_RESOURCE_MANAGER' for constants.
      require
	 not is_zombie
      do
	 result := efox_window__window_type_id(handle)
      end


feature {ANY} -- keyboard focus
   
   
   can_focus : BOOLEAN is
	 -- Test if window can handle the keyboard focus.
      require
	 not is_zombie
      do
	 result := efox_window__can_focus(handle)
      end
   
   
   set_can_focus (flag: BOOLEAN) is
	 -- Set if window can handle keyboard focus.
      require
	 not is_zombie
      do
	 efox_window__set_can_focus(handle,flag)
      end
   
   
   has_focus : BOOLEAN is
	 -- Test if window has the keyboard focus
      do
	 result := efox_window__has_focus(handle)
      end
   
   
   set_focus (flag: BOOLEAN) is
	 -- Gives this window the keyboard focus or
	 -- removes the focus from the window
      do
	 efox_window__set_focus(handle,flag)
      end   

   
feature {ANY} -- Layout Manager   
   
   
   Layout_side_top : INTEGER is 0
	 -- Pack on top side
   
   Layout_side_bottom : INTEGER is 1
	 -- Pack on bottom side
   
   Layout_side_left : INTEGER is 2
	 -- Pack on left side
   
   Layout_side_right : INTEGER is 3
	 -- Pack on right side
      
   Layout_left : INTEGER is 0
	 -- Stick on left
   
   Layout_right : INTEGER is 4
	 -- Stick on right
   
   Layout_top : INTEGER is 0
	 -- Stick on top
   
   Layout_bottom : INTEGER is 16
	 -- Stick on bottom
   
   Layout_center_x : INTEGER is 8
	 -- Center horizontally
      
   Layout_center_y : INTEGER is 32
	 -- Center vertically
   
   Layout_fill_x : INTEGER is 1024
	 -- Stretch horizontally
   
   Layout_fill_y : INTEGER is 2048
	 -- Stretch vertically
   
   Layout_fill_column : INTEGER is 1
	 -- Matrix column is stretchable
   
   Layout_fill_row : INTEGER is 2
	 -- Matrix row is stretchable
   
   
   set_layout_hints (n: INTEGER) is
	 -- Set combination of layout hints. They are the
	 -- combined using the '+' operator.
      require
	 not is_zombie
      do
 	 efox_window__set_layout_hints(handle, n)
      end
   
   
   recalculate is
	 -- Redo layout calculations
      require
	 not is_zombie
      do
	 efox_window__recalculate(handle)
      end
   

feature {ANY}  -- Cursor operations
   
   
   cursor_position : TUPLE[INTEGER,INTEGER,BIT Integer_bits] is
	 -- Return the cursor position and mouse button-state
      require
	 not is_zombie
      do
	 efox_window__cursor_position(handle,$temp_px,$temp_py,$temp_pz)
	 result := [temp_px,temp_py, temp_pz.to_bit]
      end
   
   
   set_cursor_position (x,y: INTEGER) is
	 -- Warp the cursor to the new position.
      require
	 not is_zombie
      do
	 efox_window__set_cursor_position(handle,x,y)
      end
   
   
   default_cursor : AGUI_CURSOR is
	 -- Get the default cursor for this window.
      require
	 not is_zombie
      do
	 result := efox_window__default_cursor(handle)
      end
   
   
   set_default_cursor (cur : AGUI_CURSOR) is
	 -- Set the default cursor for this window.
      require
	 not is_zombie
	 cur /= Void and then not cur.is_zombie
      do
	 efox_window__set_default_cursor(handle, cur.handle)
      end
   

feature {ANY}  -- coordinate methods
   
   
   contains (x,y: INTEGER) : BOOLEAN is
	 -- Return true if window logically contains the given point.
	 -- X and Y must be given in parent coordinates.
      require
	 not is_zombie
      do
	 result := efox_window__contains(handle, x,y)
      end
  
   
   translate_coordinates_from (from_window: AGUI_WINDOW;  x_y: TUPLE[INTEGER,INTEGER]) is
	 --  Translate coordinates from 'from_window's coordinate space to this 
	 --  window's coordinate space.
      require
	 not is_zombie
	 from_window /= Void and then not from_window.is_zombie
	 x_y /= Void
      do
	 efox_window__translate_coordinates_from(handle, $temp_px,$temp_py, x_y.first, x_y.second)
	 x_y.set_first(temp_px)
	 x_y.set_second(temp_py)
      end
   
   
   translate_coordinates_to (to_window: AGUI_WINDOW;  x_y: TUPLE[INTEGER,INTEGER]) is
	 -- Translate coordinates from this window's coordinate space to 
	 -- 'to_window's coordinate space
      require
	 not is_zombie
	 to_window /= Void and then not to_window.is_zombie
	 x_y /= Void
      do
	 efox_window__translate_coordinates_to(handle, $temp_px,$temp_py, x_y.first, x_y.second)
	 x_y.set_first(temp_px)
	 x_y.set_second(temp_py)
      end
 
   
   translate_coordinates_from_root (x_y: TUPLE[INTEGER,INTEGER]) is
	 --  Translate coordinates from root window coordinate space to this 
	 --  window's coordinate space.
      require
	 not is_zombie
	 x_y /= Void
      do
	 efox_window__translate_coordinates_from_root(handle, $temp_px,$temp_py, x_y.first, x_y.second)
	 x_y.set_first(temp_px)
	 x_y.set_second(temp_py)
      end
   
   
   translate_coordinates_to_root (x_y: TUPLE[INTEGER,INTEGER]) is
	 -- Translate coordinates from this window's coordinate space to 
	 -- root window coordinate space
      require
	 not is_zombie
	 x_y /= Void
      do
	 efox_window__translate_coordinates_to_root(handle, $temp_px,$temp_py, x_y.first, x_y.second)
	 x_y.set_first(temp_px)
	 x_y.set_second(temp_py)
      end

	 
feature {ANY}  -- size of widget
   
   
   xpos : INTEGER is
	 -- X position of window
      require
	 not is_zombie
      do
	 result := efox_window__xpos(handle)
      end
   
   
   ypos : INTEGER is
	 -- Y position of window
      require
	 not is_zombie
      do
	 result := efox_window__ypos(handle)
      end
   
   
   width : INTEGER is
	 -- Width of window
      require
	 not is_zombie
      do
	 result := efox_window__width(handle)
      end
   
   
   height : INTEGER is
	 -- Height of window
      require
	 not is_zombie
      do
	 result := efox_window__height(handle)
      end
   
   
   set_width (n: INTEGER) is
      require
	 not is_zombie
      do
	 efox_window__set_width(handle, n)
      end
   
   set_fixed_width (n: INTEGER) is
	 -- Set or remove fixed height. If n < 0 then the fixed width size is removed
      require
	 not is_zombie
      do
	 efox_window__set_fixed_width(handle, n)
      end
   
   
   set_fixed_height (n: INTEGER) is
	 -- Set or remove fixed height. If n < 0 then the fixed width size is removed
      require
	 not is_zombie
      do
	 efox_window__set_fixed_height(handle, n)
      end
   
   
   set_height (n: INTEGER) is
      require
	 not is_zombie
      do
	 efox_window__set_height(handle, n)
      end
   
   
   default_width : INTEGER is
	 -- Return the default width.
      require
	 not is_zombie
      do
	 result := efox_window__default_width(handle)
      end
   
   
   default_height : INTEGER is
	 -- Return the default height.
      require
	 not is_zombie
      do
	 result := efox_window__default_height(handle)
      end
         

feature {ANY}  -- move in z direction
   
   
   lower is
	 -- Move widget to lowest place in Z-Order
      require
	 not is_zombie
      do
	 efox_window__lower(handle)
      end
   
   
   raise is
	 -- Move widget to highest place in Z-Order
      require
	 not is_zombie
      do
	 efox_window__raise(handle)
      end
   
   
   move_before (reference: AGUI_WINDOW) is
	 -- Move window before another one. This
	 -- does a reparent if neccessary.
      require
	 not is_zombie
	 reference /= Void and then not reference.is_zombie
      do
	 efox_window__move_before(handle, reference.handle)
      end
   
   
   move_after (reference: AGUI_WINDOW) is
	 -- Move window after another one. This
	 -- does a reparent if neccessary.
      require
	 not is_zombie
	 reference /= Void and then not reference.is_zombie
      do
	 efox_window__move_after(handle, reference.handle)      
      end
   
   
   reparent (reference: AGUI_WINDOW) is
	 -- Move window under the given window.
      require
	 not is_zombie
	 reference /= Void and then not reference.is_zombie
      do
	 efox_window__reparent(handle, reference.handle)
      end
   
   
feature {ANY}  --  visibility 
   
   
   visible : BOOLEAN is
	 -- Test if window is visible. This also returns 'true' if
	 -- the window is completely covered by another window.
      require
	 not is_zombie
      do
	 result := efox_window__visible(handle)
      end
   
   
   set_visible (flag: BOOLEAN) is
	 -- Show or hide a window.
      require
	 not is_zombie
      do
	 efox_window__set_visible(handle,flag)
      end
   
   
feature {ANY}  --  data attachment
   
   
   data : ANY
	 -- Data value attached to this window
   
   
   set_data (new_value: ANY) is
	 -- Set new data value.
      require
	 not is_zombie
      do
	 data := new_value
      ensure
	 data = new_value
      end


feature {ANY}  --  background color
   
   
   background_color : AGUI_COLOR is
	 -- Return background color. 
      do
	 result.from_integer(efox_window__background_color(handle))
      end
   
   
   set_background_color (col: AGUI_COLOR) is
	 -- Set background color. 
      do
	 efox_window__set_background_color(handle, col.to_integer)
      end
   

feature {ANY}  --  destruction


   destroy is
	 -- Destroy the widget. It will become a Zombie until it
	 -- is garbage collected.
      require
	 not is_zombie
      do
	 efox_window__destroy(handle)
      ensure
	 is_zombie
      end


feature {ANY} -- event handler
   
   
   on_destroy: PROCEDURE[ANY,TUPLE] is
	 -- Agent that is called whenever the window is destroyed.
      do
	 result := find_callback_agent (On_destroy_event)
      end
   
   set_on_destroy (action: PROCEDURE[ANY,TUPLE]) is
	 -- Set the "on destroy" action.
      do
	 set_callback_agent(On_destroy_event, action)
      end
   
   
   on_focus_in: PROCEDURE[ANY,TUPLE] is
	 -- Agent that is called when the window gets the keyboard focus.
      do
	 result := find_callback_agent(On_key_focus_in_event)
      end
   
   set_on_focus_in (action: PROCEDURE[ANY,TUPLE]) is
	 -- Set the "focus in" action
      do
	 set_callback_agent(On_key_focus_in_event, action)
      end
   
   
   on_focus_out: PROCEDURE[ANY,TUPLE] is
	 -- Agent that is called when the window looses the keyboard focus.
      do
	 result := find_callback_agent(On_key_focus_out_event)
      end
   
   set_on_focus_out (action: PROCEDURE[ANY,TUPLE]) is
	 -- Set the "focus out" action.
      do
	 set_callback_agent(On_key_focus_out_event, action)
      end


   on_key_press: ROUTINE[ANY, TUPLE[BIT Integer_bits,INTEGER,STRING]] is
      do
	 result ?= find_callback_agent(On_key_press_event)
      end
   
   set_on_key_press (action: ROUTINE[ANY, TUPLE[BIT Integer_bits,INTEGER,STRING]]) is
      do
	 set_callback_agent(On_key_press_event, action)
      end
     
     
   on_key_release: ROUTINE[ANY, TUPLE[BIT Integer_bits,INTEGER]] is
      do
	 result ?= find_callback_agent(On_key_release_event)
      end
   
   set_on_key_release (action: ROUTINE[ANY,TUPLE[BIT Integer_bits,INTEGER]]) is
      do
	 set_callback_agent(On_key_release_event, action)
      end
   
   
   on_mouse_enter : PROCEDURE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER]] is
      do
	 result ?= find_callback_agent(On_mouse_enter_event)
      end
   
   set_on_mouse_enter(action: PROCEDURE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER]]) is
      do
	 set_callback_agent(On_mouse_enter_event, action)
      end
   
   
   on_mouse_leave : PROCEDURE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER]] is
      do
	 result ?= find_callback_agent(On_mouse_leave_event)
      end
   
   set_on_mouse_leave(action: PROCEDURE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER]]) is
      do
	 set_callback_agent(On_mouse_leave_event, action)
      end
   
   
   on_mouse_motion : PROCEDURE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER]] is
      do
	 result ?= find_callback_agent(On_mouse_motion_event)
      end
   
   set_on_mouse_motion(action: PROCEDURE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER]]) is
      do
	 set_callback_agent(On_mouse_motion_event, action)
      end
   
   
   on_mouse_press : PROCEDURE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER,INTEGER,INTEGER]] is
      do
	 result ?= find_callback_agent(On_mouse_press_event)
      end
   
   set_on_mouse_press(action: PROCEDURE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER,INTEGER,INTEGER]]) is
      do
	 set_callback_agent(On_mouse_press_event, action)
      end
   
   
   on_mouse_release : PROCEDURE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER,INTEGER,INTEGER]] is
      do
	 result ?= find_callback_agent(On_mouse_release_event)
      end
   
   set_on_mouse_release(action: PROCEDURE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER,INTEGER,INTEGER]]) is 
      do
	 set_callback_agent(On_mouse_release_event, action)
      end
   
   consume_event is
      external "C"
      alias "efox_window__consume_event"
      end
	 
					
feature {ANY} -- private implementation
   
   
   handle : POINTER
	 -- Handle to the FXWindow derived C object.
   
   
feature {NONE} -- private callback code
   
   
   temp_px, temp_py, temp_pz: INTEGER
	 -- Temporary used variables.
   
   On_window_specific2_event : INTEGER is 17
   On_window_specific1_event : INTEGER is 16
   On_mouse_adjust_event : INTEGER is 15
   On_paint_event : INTEGER is 14
   On_context_menu_event : INTEGER is 13
   On_tooltip_event : INTEGER is 12
   On_change_event : INTEGER is 11
   On_command_event : INTEGER is 10
   On_mouse_release_event : INTEGER is 9
   On_mouse_press_event : INTEGER is 8
   On_mouse_motion_event : INTEGER is 7
   On_mouse_leave_event : INTEGER is 6
   On_mouse_enter_event : INTEGER is 5
   On_key_press_event : INTEGER is 4
   On_key_release_event : INTEGER is 3
   On_key_focus_in_event : INTEGER is 1
   On_key_focus_out_event : INTEGER is  2
   On_destroy_event : INTEGER is 0
   
   
   find_callback_agent (n: INTEGER) : ROUTINE[ANY,TUPLE] is
      require
	 n.in_range(On_destroy_event, On_window_specific2_event)
      local
	 i : INTEGER
      do
	 if event_agents /= Void then
	    from 
	       i := 0
	    until
	       i > event_agents.upper
	    loop
	       if event_agents.item(i).event = n then
		  result := event_agents.item(i).action
		  i := event_agents.upper
	       end
	       i := i + 1
	    end
	 end
      end
   
   
   set_callback_agent (n: INTEGER; action: ROUTINE[ANY,TUPLE]) is
      require
	 not is_zombie
	 n.in_range(On_destroy_event, On_window_specific2_event)
      local
	 node : AGUI_EVENT_NODE
	 i : INTEGER
      do
	 if event_agents = Void then
	    create event_agents.with_capacity(2)
	 else
	    -- remove existing handler for this event
	    from
	       i := 0
	    until
	       i > event_agents.upper
	    loop
	       if event_agents.item(i).event = n then
		  event_agents.remove(i)
		  i := event_agents.upper
	       end
	       i := i + 1
	    end	    
	 end
	 if action /= Void then
	    -- add a new handler
	    create node.make(n,action)
	    event_agents.add_last(node)
	    efox_window__enable_event_handler(handle, n)
	 else
	    efox_window__disable_event_handler(handle, n)
	 end
      end   
   
      
   event_agents : FIXED_ARRAY[AGUI_EVENT_NODE]
   
   
   efox__shared_context_menu : AGUI_POPUP_MENU is
      once
	 create result.make	 
      end
   
   efox__shared_tooltip_string : STRING is
      once
	 create result.make(128)
      end
   
   efox__callback_context_menu (idx: INTEGER; obj: ANY) : POINTER is
      do
      end
   
   efox__callback_query_tip (major,minor: INTEGER; obj: ANY) : POINTER is
      do
      end
   
   efox__callback_changed(what: INTEGER; state: BOOLEAN; obj: ANY) is
      do
      end
   
   efox__callback_specific1(what: INTEGER; state: BOOLEAN obj: ANY) is
      do
      end
   
   efox__callback_specific2(what: INTEGER; state: BOOLEAN obj: ANY) is
      do
      end
   
   efox__callback_paint(x,y,w,h: INTEGER) is
      local
	 a: ROUTINE[ANY,TUPLE[INTEGER,INTEGER,INTEGER,INTEGER]]
      do
	 a ?= find_callback_agent(On_paint_event)
	 if a /= Void then
	    a.call([x,y,w,h])
	 end
      end
   
   efox__callback_mouse_adjust(x,y: INTEGER) is
      local
	 a: ROUTINE[ANY,TUPLE[INTEGER,INTEGER]]
      do
	 a ?= find_callback_agent(On_mouse_adjust_event)
	 if a /= Void then
	    a.call([x,y])
	 end
      end
   
   efox__callback_command(idx: INTEGER; state: BOOLEAN; obj: ANY) is
      do
      end
   
   efox__callback_destroy(key: INTEGER) is
      local
	 a : ROUTINE[ANY,TUPLE]
      do
	 check
	    efox__window_hash_table.has(key)
	    efox__window_hash_table.at(key) = Current
	 end
	 efox__window_hash_table.remove(key)
	 handle := default_pointer
	 a := on_destroy
	 if a /= Void then 
	    a.call([])
	 end
      end
   
   efox__callback_focus_in is
      local
	 a: ROUTINE[ANY,TUPLE]
      do
	 a := on_focus_in
	 if a /= Void then
	    a.call([])
	 end
      end
   
   efox__callback_focus_out is
      local
	 a: ROUTINE[ANY,TUPLE]
      do
	 a := on_focus_out
	 if a /= Void then
	    a.call([])
	 end
      end
   
   efox__callback_key_press (ev,code: INTEGER; s: POINTER) is --TODO call ev is BIT Integer_bits
      local
	 a : ROUTINE[ANY,TUPLE[BIT Integer_bits,INTEGER,STRING]]
	 str : STRING
      do
	 create str.from_external_copy(s) --TODO IMPORT: speedup !!
	 a := on_key_press
	 if a /= Void then
	    a.call([ev.to_bit,code,str])
	 end
      end
   
   efox__callback_key_release (ev,code: INTEGER) is --TODO call ev is BIT Integer_bits
      local
	 a : ROUTINE[ANY,TUPLE[BIT Integer_bits,INTEGER]]
      do
	 a := on_key_release
	 if a /= Void then
	    a.call([ev.to_bit,code])
	 end
      end
   
   efox__callback_mouse_enter (mod: BIT Integer_bits; x,y: INTEGER) is
      local
	 a : ROUTINE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER]]
      do
	 a := on_mouse_enter
	 if a /= Void then
	    a.call([mod,x,y])
	 end
      end
   
   efox__callback_mouse_leave (mod: BIT Integer_bits; x,y: INTEGER) is
      local
	 a : ROUTINE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER]]
      do
	 a := on_mouse_leave
	 if a /= Void then
	    a.call([mod,x,y])
	 end
      end
   
   efox__callback_mouse_motion (mod: BIT Integer_bits; x,y: INTEGER) is
      local
	 a : ROUTINE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER]]
      do
	 a := on_mouse_motion
	 if a /= Void then
	    a.call([mod,x,y])
	 end
      end
   
   efox__callback_mouse_press (mod: BIT Integer_bits; x,y,button,clicks: INTEGER) is
      local
	 a : ROUTINE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER,INTEGER,INTEGER]]
      do
	 a := on_mouse_press
	 if a /= Void then
	    a.call([mod,x,y,button,clicks])
	 end
      end
   
   efox__callback_mouse_release (mod: BIT Integer_bits;  x,y,button,clicks: INTEGER) is
      local
	 a : ROUTINE[ANY,TUPLE[BIT Integer_bits,INTEGER,INTEGER,INTEGER,INTEGER]]
      do
	 a := on_mouse_release
	 if a /= Void then
	    a.call([mod,x,y,button,clicks])
	 end
      end
   

feature {NONE} -- private C binding
   
   
   -- navigation in window tree   
   
   efox_window__parent_window(h: POINTER): AGUI_WINDOW is
      external "C"
      end
   
   efox_window__next_window(h: POINTER): AGUI_WINDOW is
      external "C"
      end
   
   efox_window__first_child_window(h: POINTER): AGUI_WINDOW is
      external "C"
      end
   
   efox_window__shell_window(hnd: POINTER) : AGUI_SHELL_WINDOW is
      external "C"
      end
      
   -- window naming
   
   efox_window__window_path_name (h: POINTER) : POINTER is
      external "C"
      end
   
   efox_window__window_name (h: POINTER) : POINTER is
      external "C"
      end
      
   efox_window__find_window_by_name (h,s: POINTER) : AGUI_WINDOW is
      external "C"
      end
   
   -- metadata   
   
   efox_window__window_type_id (hnd: POINTER) : INTEGER is
      external "C"
      end
   
   -- keyboard focus    
   
   efox_window__can_focus(hnd: POINTER) : BOOLEAN is
      external "C"
      end
      
   efox_window__set_can_focus(hnd: POINTER; flag: BOOLEAN) is
      external "C"
      end
   
   efox_window__has_focus(hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_window__set_focus(hnd: POINTER; flag: BOOLEAN) is
      external "C"
      end   
   
   -- layout operation   
   
   efox_window__set_layout_hints(hnd: POINTER; n: INTEGER) is
      external "C"
      end
 
   efox_window__recalculate(h: POINTER) is
      external "C"
      end

   -- cursor operations
   
   efox_window__cursor_position(hnd, tpx, tpy, tpz: POINTER) is
      external "C"
      end
   
   efox_window__set_cursor_position(hnd: POINTER; x,y: INTEGER) is
      external "C"
      end
   
   efox_window__default_cursor(hnd: POINTER) : AGUI_CURSOR is
      external "C"
      end
   
   efox_window__set_default_cursor(hnd,chnd: POINTER) is
      external "C"
      end
   
   -- coordinate methods
   
   efox_window__contains (hnd: POINTER; x,y: INTEGER) : BOOLEAN is
      external "C"
      end
   
   efox_window__translate_coordinates_from (hnd,xres,yres: POINTER; x,y:INTEGER) is
      external "C"
      end
   
   efox_window__translate_coordinates_to (hnd,xres,yres: POINTER; x,y:INTEGER) is
      external "C"
      end
   
   efox_window__translate_coordinates_from_root (hnd,xres,yres: POINTER; x,y:INTEGER) is
      external "C"
      end
   
   efox_window__translate_coordinates_to_root (hnd,xres,yres: POINTER; x,y:INTEGER) is
      external "C"
      end
   
  -- size of widget   
   
   efox_window__xpos(hnd: POINTER) : INTEGER is
      external "C"
      end

   efox_window__ypos(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_window__width(hnd: POINTER) : INTEGER is
      external "C"
      end

   efox_window__height(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_window__set_fixed_width(hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_window__set_width(hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_window__set_fixed_height(hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_window__set_height(hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_window__default_width(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_window__default_height(hnd: POINTER) : INTEGER is
      external "C"
      end
   
  -- move in z direction
   
   efox_window__lower(hnd: POINTER) is
   	external "C"
   	end   	
   	
   efox_window__raise(hnd: POINTER) is
   	external "C"
   	end
   	
   efox_window__move_before(h,rh: POINTER) is
      external "C"
      end

   efox_window__move_after(h,rh: POINTER) is
      external "C"
      end
   
   efox_window__reparent(h,rh: POINTER) is
      external "C"
      end
   
   --  visibility
   
   efox_window__visible(hnd: POINTER) : BOOLEAN is
      external "C"
      end
   
   efox_window__set_visible(hnd: POINTER; flag: BOOLEAN) is
      external "C"
      end

  --  background color   
   
   efox_window__background_color (hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_window__set_background_color (hnd: POINTER; col: INTEGER) is
      external "C"
      end
   
   --  destruction
   
   efox_window__destroy (hnd: POINTER) is
      external "C"
      end
   
   -- event handling
   
   efox_window__enable_event_handler(hnd: POINTER; n: INTEGER) is
      external "C"
      end   
   
   efox_window__disable_event_handler(hnd: POINTER; n: INTEGER) is
      external "C"
      end   
   
end
   
