class NET_CLIENT_STREAM_SOCKET
   
feature
   
   on_connect: ROUTINE[ANY,TUPLE]
   on_read: ROUTINE[ANY,TUPLE]
   on_write: ROUTINE[ANY,TUPLE]
   
   ---------------------------------------
   
   
   connect_to (host: STRING; port: INTEGER) is 
      deferred
      end
   
   is_connected : BOOLEAN is
      deferred
      end
   
   disconnect (mode: INTEGER) is
      deferred
      end
   
   ---------------------------------------
   
   read(str: STRING) is
	 -- Read as many bytes as possible from the socket and
	 -- append them to the given string. 'last_transfered' 
	 -- contains the number of bytes that were read.
      require
	 str /= Void
      deferred
      end
   
   write (str: STRING; start_ofs, len: INTEGER) is
	 -- Write as many bytes as possible to the socket without
	 -- blocking. 'last_transfered' contains the number of
	 -- bytes that were written.
      require
	 str /= Void
	 str.valid_offset(start_ofs)
	 str.valid_offset(start_ofs+len-1)
      deferred
      end
   
   
   last_transfered : INTEGER
   
   
   last_error : INTEGER is
	 -- Zero or a OS dependent error number for the last occurred error.
      deferred
      end
      
   last_error_message : STRING is
	 -- A human readable message that describes the last occurred error.
	 -- This is the empty string if 'last_error' is 0.
      deferred
      end
      
   ----------------
   
   observe_event (wait_connect, wait_read, wait_write: INTEGER) is
      deferred
      end
   
   wait_event (wait_connect, wait_read, wait_write: INTEGER) is
      deferred
      end
   
   stop is
      do
      end
      
end


--------------------------------------------------------------


class NET_LINE_CLIENT_SOCKET
   
inherit
   NET_CLIENT_STREAM_SOCKET
      end
   
feature
   
   write_line (str: STRING; action: ROUTINE[ANY,TUPLE]) is
      require
	 str /= Void and then not str.is_emtpy
      do
	 write_line_agent(str, 1, action)
      end
   
   read_line  (str: STRING; action: ROUTINE[ANY,TUPLE]) is
      do
	 
      end
   
   -----
   
   write_line_agent (str: STRING; n: INTEGER; action: ROUTINE[ANY,TUPLE]) is
      do
	 write(str, n, str.count-n+1)
	 n = n + last_transfered
	 if n /= str.upper then
	    set_on_write(agent write_line_agent(str,n,action))
	    observe_event(false,false,true)
	 else
	    action.call([])
	 end
      end
   
   read_line_agent
   
end


------------------------------------------------------------


class  NET_POP3_ASYNCHRON
   

inherit
   NET_LINE_CLIENT_SOCKET
      end
   
feature
   
   user: STRING
   password: STRING
   
   
   login (finish: ROUTINE[ANY,TUPLE]) is
      do
	 finish_callback := finish
	 write_line("user", agent read_response_line(agent user_response)
      end
   
   count (finish: ROUTINE[ANY,TUPLE]) is
      do
	 finish_callback := finish
	 write_line("cnt", agent read_response_line(agent count_response(finish))
      end   
      
   message_box_size (finish: ROUTINE[ANY,TUPLE]) is
      do
	 finish_callback := finish
	 write_line("cnt", agent read_response_line(agent message_box_size_response(finish))
      end
      
   delete (msg_no: INTEGER; finish: ROUTINE[ANY,TUPLE]) is
      do
	 finish_callback := finish
	 write_line("cnt", agent read_response_line(agent delete_response(finish))
      end
   
   save_header_in_file (msg_no: INTEGER; filepath: STRING; finish: ROUTINE[ANY,TUPLE]) is
      do
      end
   
   save_message_in_file (msg_no: INTEGER; filepath: STRING; finish: ROUTINE[ANY,TUPLE]) is
      do
      end
   
   retrieve_header (msg_no: INTEGER; string: STRING; finish: ROUTINE[ANY,TUPLE]) is
      do
      end
   
   retrieve_message (msg_no: INTEGER; string: STRING; finish: ROUTINE[ANY,TUPLE]) is
      do
      end
   
   
feature {NONE} -- command response parsing
   
   
   count_response is
      do
	 call_finish
      end
   
   user_response is
      do
	 write_line("pass", agent read_response_line(agent pass_response))
      end
      
   pass_response is
      do
	 call_finish
      end
   
   delete_response is
      do
	 call_finish
      end
      
   message_box_size_response is
      do
	 call_finish
      end
   
   
feature {NONE} -- private helper methods
   
   dot_shift_read is
      do
      end
   
   read_response_line(next: ROUTINE[ANY,TUPLE]) is
      do
	 read_line(agent count_response_received(finish))
      end
   
   call_finish is
      do
	 if finish_callback /= Void then finish_callback.call([]) end
      end
   
   finish_callback : ROUTINE[ANY,TUPLE]
   
end




------------------------------------------------------------




class  NET_POP3
   
   
   
   count : INTEGER is
	 --  Return number of messages currently on the POP3 server.
      do
	 async.count(agent count_callback)
	 wait_for_finish
	 result := count_result
      end
   
   delete (msg_no: INTEGER) is
      do
      end
   
   retrieve_mail_box_size : INTEGER is
      do
      end
   
   
   retrieve_message(str: STRING) is
      do
      end
   
   retrieve_message_in(str: STRING) is
      do
      end
   
   retrieve_header(file: STRING) is
      do
      end
   
   retrieve_header_in(str: STRING) is
      do
      end
   
   last_status : BOOLEAN
   
	 
	 
	 ----------------
   
   count_callback (n: INTEGER) is
      do
	 count_result := n
      end
   
   count_result : INTEGER
   
end
