   
   
feature {NONE}  -- constructor
   
   
   make (p: AGUI_COMPOSITE;  n: STRING) is
      require
	 p /= Void and n /= Void
      do
	 create item_list.with_capacity(128)
         handle := efox_tree__make(Current,p.handle, n.to_external)
         scroll_area_widget := Current
         p.register(Current)
      end
   
   --TODO remove
   make_internal (p: AGUI_COMPOSITE; ptr: POINTER) is
      do
	 create item_list.with_capacity(128)
         scroll_area_widget := Current
	 handle := ptr
      end
   
   
   
feature {ANY}  -- install event handlers
   
   set_on_query_tip(a : PROCEDURE[ANY,TUPLE[AGUI_TREE_ITEM,STRING]]) is
      do
	 on_query_tip := a
      end
   
   set_on_context_menu(a : PROCEDURE[ANY,TUPLE[AGUI_TREE_ITEM,AGUI_POPUP_MENU]]) is
      do
	 on_context_menu := a
      end
   
   set_on_changed(a : PROCEDURE[ANY,TUPLE[AGUI_TREE_ITEM]]) is
      do
	 on_changed := a
      end
   
   set_on_collapsed(a : PROCEDURE[ANY,TUPLE[AGUI_TREE_ITEM]]) is
      do
	 on_collapsed := a
      end
   
   set_on_expanded(a : PROCEDURE[ANY,TUPLE[AGUI_TREE_ITEM]]) is
      do
	 on_expanded := a
      end
   
   set_on_command(a : PROCEDURE[ANY,TUPLE[AGUI_TREE_ITEM]]) is
      do
	 on_command := a
      end
   
feature {NONE} -- private implementation
   
   
   sort_agent : AGENT_BINDING
   
   
   do_all_recursive(li: AGUI_TREE_ITEM; action: ROUTINE[ANY,TUPLE[AGUI_TREE_ITEM]]) is
      require
	 li /= Void
      local
	 iter: AGUI_TREE_ITEM
      do
	 action.call([li])
	 from
	    iter := li.first_item
	 until
	    iter = Void 
	 loop
	    do_all_recursive(iter,action)
	    iter := iter.next_item
	 end	 
      end
   
   append_recursive (c: COLLECTION[AGUI_TREE_ITEM]; item: AGUI_TREE_ITEM) is
      local
	 p : AGUI_TREE_ITEM
      do
	 c.add_last(item)
	 from
	    p := item.first_item
	 until
	    p = Void
	 loop
	    append_recursive (c, p)
	    p := p.next_item
	 end
      end
   
   
   append_recursive_sel (c: COLLECTION[AGUI_TREE_ITEM]; item: AGUI_TREE_ITEM) is
      local
	 p : AGUI_TREE_ITEM
      do
	 if item.selected then
	    c.add_last(item)
	 end
	 from
	    p := item.first_item
	 until
	    p = Void
	 loop
	    append_recursive_sel (c, p)
	    p := p.next_item
	 end
      end
   
   
   find_data_recursive(item: AGUI_TREE_ITEM; what: ANY) : AGUI_TREE_ITEM is
      local
	 p : AGUI_TREE_ITEM
      do
	 if what.is_equal(item.data) then
	    result := item
	 else
	    from
	       p := item.first_item
	    until
	       p = Void or else result /= Void
	    loop
	       result := find_data_recursive (p, what)
	       p := p.next_item
	    end
	 end
      end
   
   fast_find_data_recursive(item: AGUI_TREE_ITEM; what: ANY) : AGUI_TREE_ITEM is
      local
	 p : AGUI_TREE_ITEM
      do
	 if what = item.data then
	    result := item
	 else
	    from
	       p := item.first_item
	    until
	       p = Void or else result /= Void
	    loop
	       result := fast_find_data_recursive (p, what)
	       p := p.next_item
	    end
	 end
      end
   
   on_query_tip : PROCEDURE[ANY,TUPLE[AGUI_TREE_ITEM,STRING]]
   on_context_menu : PROCEDURE[ANY,TUPLE[AGUI_TREE_ITEM,AGUI_POPUP_MENU]]
   on_changed : PROCEDURE[ANY,TUPLE[AGUI_TREE_ITEM]]
   on_collapsed : PROCEDURE[ANY,TUPLE[AGUI_TREE_ITEM]]
   on_expanded : PROCEDURE[ANY,TUPLE[AGUI_TREE_ITEM]]
   on_command : PROCEDURE[ANY,TUPLE[AGUI_TREE_ITEM]]
   
   
feature {NONE} -- callbacks
   
   
   efox__callback_query_tip (major,minor: INTEGER; obj: ANY; s: STRING) is
      local
	 i : AGUI_TREE_ITEM
      do
	 i ?= obj
	 if on_query_tip /= Void then
	    on_query_tip.call([i,s])
	 end
      end
   
   efox__callback_context_menu (m: AGUI_POPUP_MENU; idx: INTEGER; obj: ANY) is
      local
	 i : AGUI_TREE_ITEM
      do
	 i ?= obj
	 if on_context_menu /= Void then
	    on_context_menu.call([obj,m])
	 end
      end
   
   efox__callback_changed(what: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 i : AGUI_TREE_ITEM
      do
	 i ?= obj
	 if what = 0 then
	    -- change of current item
	    if on_changed /= Void then
	       on_changed.call([i])
	    end
	 elseif what = 1 then
	    -- collapse item
	    if on_collapsed /= Void then
	       check i /= Void end
	       on_collapsed.call([i])
	    end
	 elseif what = 2 then
	    -- expand item
	    if on_expanded /= Void then
	       check i /= Void end
	       on_expanded.call([i])
	    end	    
	 end
      end
   
   efox__callback_command(idx: INTEGER; state: BOOLEAN; obj: ANY) is
      local
	 i : AGUI_TREE_ITEM
      do
	 i ?= obj
	 if i /= Void and then on_command /= Void then 
	    on_command.call([i]) 
	 end
      end
   
feature {NONE} -- private C binding   
   
   
   efox_tree__make(this:like Current; hnd, n: POINTER) : POINTER is
      external "C"
      end
   
   efox_tree__set_indent (hnd: POINTER; n: INTEGER) is
      external "C"
      end
   
   efox_tree__indent (hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_tree__font(hnd: POINTER) : AGUI_FONT is
      external "C"
      end
   
   efox_tree__set_font(hnd,fhnd: POINTER) is
      external "C"
      end
   
   efox_tree__text_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_tree__set_text_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end
   
   efox_tree__sel_back_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_tree__set_sel_back_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end
   
   efox_tree__sel_text_color(hnd: POINTER) : INTEGER is
      external "C"
      end
   
   efox_tree__set_sel_text_color(hnd: POINTER; col: INTEGER) is
      external "C"
      end
   efox_tree__set_style(hnd: POINTER; n: INTEGER) is
      external "C"
      end     
      
   efox_tree__style(hnd: POINTER) : INTEGER is
      external "C"
      end     

   efox_tree__set_sort_agent(hnd,method: POINTER; obj: ANY) is
      external "C"
      end
   
   efox_tree__set_anchor_item(hnd, ihnd: POINTER) is
      external "C"
      end
   
   efox_tree__anchor_item(hnd: POINTER) : AGUI_TREE_ITEM is
      external "C"
      end
   
   efox_tree__cursor_item(hnd: POINTER) : AGUI_TREE_ITEM is
      external "C"
      end
   
   efox_tree__kill_selection(hnd: POINTER) is
      external "C"
      end
   
   efox_tree__extend_selection(hnd, ihnd: POINTER) is
      external "C"
      end
   
   efox_tree__item_at (hnd: POINTER; x,y: INTEGER) : AGUI_TREE_ITEM is
      external "C"
      end

   efox_tree__find(hnd,str: POINTER; flag: BOOLEAN) : AGUI_TREE_ITEM is
      external "C"
      end

end
   
