class  TEST_REGULAR_EXPRESSION
   
inherit
   AGUI_GLOBALS
      end
   

creation
   make
   

feature
      
   
   test_search (s, expr: STRING; pos: TUPLE[INTEGER,INTEGER]; icase: BOOLEAN) is
      local
	 re : AGUI_REGULAR_EXPRESSION
      do
	 create re.make
	 re.compile(expr,icase)
	 check re.is_compiled end
	 re.optimize
	 re.search(s)
	 if pos /= Void then
	    check re.count_captured_substrings = 1 end
	    io.put_string("found: [" + s.substring(re.match_start_offset, re.match_end_offset) + "]%N")
	    check pos.first = re.match_start_offset and pos.second = re.match_end_offset end
	 else
	    check re.count_captured_substrings = 0 end
	 end
      end
   
   
   test_match (s, expr: STRING; pos: TUPLE[INTEGER,INTEGER]; icase: BOOLEAN) is
      local
	 re : AGUI_REGULAR_EXPRESSION
      do
	 create re.make
	 re.compile(expr,icase)
	 check re.is_compiled end
	 re.optimize
	 re.match(s)
	 if pos /= Void then
	    check re.count_captured_substrings = 1 end
	    io.put_string("matched: [" + s.substring(re.match_start_offset, re.match_end_offset) + "]%N")
	    check pos.first = re.match_start_offset and pos.second = re.match_end_offset end
	 else
	    check re.count_captured_substrings = 0 end
	 end
      end
   
   
   test_search_multiline (s, expr: STRING; pos: TUPLE[INTEGER,INTEGER]; icase: BOOLEAN) is
      local
	 re : AGUI_REGULAR_EXPRESSION
      do
	 create re.make
	 re.compile_multiline(expr,icase)
	 check re.is_compiled end
	 re.optimize
	 re.search(s)
	 if pos /= Void then
	    check re.count_captured_substrings = 1 end
	    io.put_string("found: [" + s.substring(re.match_start_offset, re.match_end_offset) + "]%N")
	 else
	    check re.count_captured_substrings = 0 end
	 end
      end
   
   
   test_wrong_compile is
      local
	 re : AGUI_REGULAR_EXPRESSION
	 expr: STRING
	 s: STRING
      do
	 expr := "test(this"
	 create re.make
	 re.compile (expr, false)
	 check not re.is_compiled end
	 io.put_string("error:   " + re.last_compile_error + "%N")
	 io.put_string("pattern: " + expr + "%N")
	 s := clone("^%N")
	 s.precede_multiple(' ',re.last_compile_error_offset)
	 io.put_string("         " + s)
      end
      
   
   test_non_existing_match is
      local
	 re : AGUI_REGULAR_EXPRESSION
	 expr: STRING
	 s: STRING
	 t: TUPLE[INTEGER,INTEGER]
      do
	 s := "abc"
	 expr := "(a|(z))(bc)"
	 create re.make
	 re.compile (expr, false)
	 check re.is_compiled end
	 re.search(s)
	 check re.count_captured_substrings = 4 end
	 create t
	 re.captured_substring(1,t)
	 check t.first = 1 and t.second = 3 end
	 re.captured_substring(2,t)
	 check t.first = 1 and t.second = 1 end
	 re.captured_substring(3,t)
	 check t.first = -1 and t.second = -1 end
	 re.captured_substring(4,t)
	 check t.first = 2 and t.second = 3 end	 
	 io.put_string("found: [" + s.substring(re.match_start_offset, re.match_end_offset) + "]%N")
      end   
   
   
   test_search_substring is
      local
	 re : AGUI_REGULAR_EXPRESSION
	 expr: STRING
	 s: STRING
      do
	 s := "Mississipi"
	 expr := "\Biss\B"
	 create re.make
	 re.compile (expr, false)
	 check re.is_compiled end
	 re.search_substring(s,1)
	 check re.count_captured_substrings = 1 end
	 check re.match_start_offset = 2 and  re.match_end_offset = 4 end
	 re.search_substring(s,5)
	 check re.count_captured_substrings = 1 end
	 check re.match_start_offset = 5 and  re.match_end_offset = 7 end
	 re.search_substring(s,8)
	 check re.count_captured_substrings = 0 end
      end
   
   
   make is
      local
	 eiffel_bug:  BUG_WORKAROUND
      do
	 application.init("test_regular_expression","MidnightSoft")
	 eiffel_bug.avoid
	 
	 test_wrong_compile
	 test_non_existing_match
	 test_search_substring
	 
	 test_search("match foobar", "foo", [7,9],false)
	 test_search("match foobar", "Foo", Void, false)
	 test_search("match foobar", "Foo", [7,9], true)
	 
	 test_search_multiline("match%Nfoobar", "^foo", [7,9],false)
	 test_search_multiline("match%Nfoobar", "ch$", [4,5],false)
	 test_search_multiline("match%Nfoobar", "ar$", [11,12],false)
	 
	 test_match("match foobar", "foo", Void, false)
	 test_match("match foobar", "[a-z]*", [1,5], false)
	 test_match("match foobar", "Match", [1,5], true)
	 test_match("match foobar", "Match", Void, false)	 
      end
   
end
