class   TEST_SETTINGS

-- Test of classes:
--
--    AGUI_SETTINGS 
--    AGUI_FILE_SETTINGS
--    AGUI_REGISTRY_SETTINGS
--
   
inherit
   AGUI_GLOBALS
      end
   
   
creation
   make
   
   
feature
   
   p(s: STRING) is
      do
	 io.put_string(s + "%N")
      end
   
   
   black_color: AGUI_COLOR
   red_color: AGUI_COLOR
   
   
   read (init : AGUI_SETTINGS) is
      local
	 astr: ARRAY[STRING]
	 ai: ARRAY[INTEGER]
      do
	 check not init.is_modified end
	 check init.entries("dummy").count = 2 end
	 check init.exist("dummy", "value1") end
	 check not init.exist("dummy", "value2") end
	 init.delete("dummy","value1")
	 check not init.exist("dummy", "value1") end
	 check init.entries("dummy").count = 1 end
	 check init.sections.count = 4 end
	 check init.exist_section("dummy") end
	 init.delete_section("dummy")
	 check not init.exist_section("dummy") end	 
	 init.save
	 check not init.is_modified end
	 
	 -- find all
	 check init.exist_section("test") end
	 check init.read_integer("test", "integer", 0) = 10 end
	 check (init.read_double("test", "double", 0.1) - 3.1415).abs < 0.00001 end
	 check init.read_string("test", "string", "foo").is_equal("foobar ist gut") end
	 check init.read_color("test", "color", black_color) = red_color end
	 check init.read_boolean("test", "boolean-true", false) = true end
	 check init.read_boolean("test", "boolean-false", true) = false end
	 astr := init.read_string_array("string","keys")
	 check astr.count = 6 end
	 check astr.item(1).is_equal("this"); astr.item(4).is_equal("list"); astr.item(6).is_equal("words") end
	 ai := init.read_integer_array("integer", "ints")
	 check ai.count = 3 end
	 check ai.item(1) = 1; ai.item(2) = 666; ai.item(3) = 999 end
	 
	 -- find nothing
	 check init.read_integer("test-non-exist", "integer", 10) = 10 end
	 check init.read_double("test-non-exist", "double", 3.1415) = 3.1415 end
	 check init.read_string("test-non-exist", "string", "foobar ist gut").is_equal("foobar ist gut") end
	 check init.read_color("test-non-exist", "color", red_color) = red_color end
	 check init.read_boolean("test-non-exist", "boolean-true", true) = true end
	 check init.read_boolean("test-non-exist", "boolean-false", false) = false end      
	 astr := init.read_string_array("foo","keys")
	 check astr.count = 0 end
	 ai := init.read_integer_array("test","foo")
	 check ai.count = 0 end
	 
	 check not init.is_modified end
	 init.clear
	 check init.is_modified end
	 check init.sections.count = 0 end
	 check init.read_integer("test", "integer", 1) = 1 end
	 init.write_integer("test", "integer", 666)
      end
   
   
   write (init : AGUI_SETTINGS) is
      do
	 check not init.is_modified end
	 init.delete("test","integer") 
	 check init.is_modified end 
	 init.write_string("dummy", "value1", "foo")
	 check init.is_modified end
	 init.write_string("dummy", "value3", "foo")
	 -----
	 init.write_integer("test", "integer", 10)
         init.write_double("test", "double", 3.1415)
	 init.write_string("test", "string", "foobar ist gut")
	 init.write_color("test", "color", red_color)
	 init.write_boolean("test", "boolean-true", true)
	 init.write_boolean("test", "boolean-false", false)
	 ----
	 init.write_string_array("string","keys", <<"this","is","a","list","of","words">>)
         init.write_integer_array("integer","ints", <<1,666,999>>)
      end
   
   
   test_file_settings is
      local
	 f : AGUI_FILE_SETTINGS
	 ft: AGUI_FILE_TOOLS
      do
	 ft.remove("foo.ini")
	 ft.remove("foo2.ini")
	 create f.make
	 f.set_path("foo.ini")
	 check f.path.is_equal("foo.ini") end
	 check not f.is_modified end
	 write(f)
	 check f.is_modified end
	 f.save
	 check f.last_status end
	 check not f.is_modified end
	 f.clear
	 
	 create f.make
	 f.set_path("foo.ini")
	 f.load
	 check f.last_status end
	 read(f)
	 f.set_path("foo2.ini")
	 f.save
	 check f.last_status end
	 check f.sections.count = 1 end
	 
	 -- check that loading a non existing file does not exist
	 f.set_path("does-not-exist")
	 f.load
	 check not f.last_status end
	 check f.sections.count = 0 end
	 f.clear
      end
   
   
   test_registry_settings is
      local
	 r : AGUI_REGISTRY_SETTINGS
      do
	 r := application.registry
	 r.clear
	 r.save
	 write(r)
	 r.save
	 read(r)
	 r.clear
	 check r.sections.count = 0 end
	 r.save
	 check not r.is_modified end
	 check r.sections.count = 0 end
      end
   
   
   make is
      local
	 eiffel_bug: BUG_WORKAROUND
	 sys: AGUI_SYSTEM
      do
	 application.init("test_settings","MidnightSoft")
	 eiffel_bug.avoid
	 
	 black_color.from_rgb(0,0,0)
	 red_color.from_rgb(255,0,0)
	 
	 test_file_settings
	 test_registry_settings
	 
	 application.registry.write_string("foo","last","thats the end my friend")
      end
   
end
